@@ -33,8 +33,12 @@
 #include "gabble-connection-manager-glue.h"
 #include "gabble-connection-manager-signals-marshal.h"
 
-#define BUS_NAME        "org.freedesktop.Telepathy.ConnectionManager.gabble"
-#define OBJECT_PATH     "/org/freedesktop/Telepathy/ConnectionManager/gabble"
+#define TP_TYPE_PARAM (dbus_g_type_get_struct ("GValueArray", \
+      G_TYPE_STRING, \
+      G_TYPE_UINT, \
+      G_TYPE_STRING, \
+      G_TYPE_VALUE, \
+      G_TYPE_INVALID))
 
 G_DEFINE_TYPE(GabbleConnectionManager, gabble_connection_manager, G_TYPE_OBJECT)
 
@@ -57,14 +61,18 @@ struct _GabbleConnectionManagerPrivate
   GHashTable *connections;
 };
 
-#define GABBLE_CONNECTION_MANAGER_GET_PRIVATE(o)     (G_TYPE_INSTANCE_GET_PRIVATE ((o), GABBLE_TYPE_CONNECTION_MANAGER, GabbleConnectionManagerPrivate))
+#define GABBLE_CONNECTION_MANAGER_GET_PRIVATE(obj) \
+    ((GabbleConnectionManagerPrivate *)obj->priv)
 
 /* type definition stuff */
 
 static void
-gabble_connection_manager_init (GabbleConnectionManager *obj)
+gabble_connection_manager_init (GabbleConnectionManager *self)
 {
-  GabbleConnectionManagerPrivate *priv = GABBLE_CONNECTION_MANAGER_GET_PRIVATE (obj);
+  GabbleConnectionManagerPrivate *priv = G_TYPE_INSTANCE_GET_PRIVATE (self,
+      GABBLE_TYPE_CONNECTION_MANAGER, GabbleConnectionManagerPrivate);
+
+  self->priv = priv;
 
   priv->connections = g_hash_table_new (g_direct_hash, g_direct_equal);
 }
@@ -153,17 +161,9 @@ struct _GabbleParams {
   gchar *stun_server;
   guint stun_port;
   gboolean ignore_ssl_errors;
-};
-
-typedef struct _GabbleParamSpec GabbleParamSpec;
-
-struct _GabbleParamSpec {
-  const gchar *name;
-  const gchar *dtype;
-  const GType gtype;
-  gboolean mandatory;
-  const gpointer def;
-  const gsize offset;
+  gchar *alias;
+  gchar *auth_mac;
+  gchar *auth_btid;
 };
 
 enum {
@@ -181,28 +181,41 @@ enum {
     JABBER_PARAM_FALLBACK_CONFERENCE_SERVER,
     JABBER_PARAM_STUN_SERVER,
     JABBER_PARAM_STUN_PORT,
-    JABBER_PARAM_IGNORE_SSL_ERRORS
+    JABBER_PARAM_IGNORE_SSL_ERRORS,
+    JABBER_PARAM_ALIAS,
+    JABBER_PARAM_AUTH_MAC,
+    JABBER_PARAM_AUTH_BTID,
+    LAST_JABBER_PARAM
 };
 
 static const GabbleParamSpec jabber_params[] = {
-  { "account", DBUS_TYPE_STRING_AS_STRING, G_TYPE_STRING, TRUE, NULL, G_STRUCT_OFFSET(GabbleParams, account) },
-  { "password", DBUS_TYPE_STRING_AS_STRING, G_TYPE_STRING, TRUE, NULL, G_STRUCT_OFFSET(GabbleParams, password) },
-  { "server", DBUS_TYPE_STRING_AS_STRING, G_TYPE_STRING, FALSE, NULL, G_STRUCT_OFFSET(GabbleParams, server) },
-  { "resource", DBUS_TYPE_STRING_AS_STRING, G_TYPE_STRING, FALSE, GABBLE_PARAMS_DEFAULT_RESOURCE, G_STRUCT_OFFSET(GabbleParams, resource) },
-  { "priority", DBUS_TYPE_INT16_AS_STRING, G_TYPE_INT, FALSE, GINT_TO_POINTER(0), G_STRUCT_OFFSET(GabbleParams, priority) },
-  { "port", DBUS_TYPE_UINT16_AS_STRING, G_TYPE_UINT, FALSE, GINT_TO_POINTER(GABBLE_PARAMS_DEFAULT_PORT), G_STRUCT_OFFSET(GabbleParams, port) },
-  { "old-ssl", DBUS_TYPE_BOOLEAN_AS_STRING, G_TYPE_BOOLEAN, FALSE, GINT_TO_POINTER(FALSE), G_STRUCT_OFFSET(GabbleParams, old_ssl) },
-  { "register", DBUS_TYPE_BOOLEAN_AS_STRING, G_TYPE_BOOLEAN, FALSE, GINT_TO_POINTER(FALSE), G_STRUCT_OFFSET(GabbleParams, do_register) },
-  { "low-bandwidth", DBUS_TYPE_BOOLEAN_AS_STRING, G_TYPE_BOOLEAN, FALSE, GINT_TO_POINTER(FALSE), G_STRUCT_OFFSET(GabbleParams, low_bandwidth) },
-  { "https-proxy-server", DBUS_TYPE_STRING_AS_STRING, G_TYPE_STRING, FALSE, NULL, G_STRUCT_OFFSET(GabbleParams, https_proxy_server) },
-  { "https-proxy-port", DBUS_TYPE_UINT16_AS_STRING, G_TYPE_UINT, FALSE, GINT_TO_POINTER(GABBLE_PARAMS_DEFAULT_HTTPS_PROXY_PORT), G_STRUCT_OFFSET(GabbleParams, https_proxy_port) },
-  { "fallback-conference-server", DBUS_TYPE_STRING_AS_STRING, G_TYPE_STRING, FALSE, NULL, G_STRUCT_OFFSET(GabbleParams, fallback_conference_server) },
-  { "stun-server", DBUS_TYPE_STRING_AS_STRING, G_TYPE_STRING, FALSE, NULL, G_STRUCT_OFFSET(GabbleParams, stun_server) },
-  { "stun-port", DBUS_TYPE_UINT16_AS_STRING, G_TYPE_UINT, FALSE, GINT_TO_POINTER(GABBLE_PARAMS_DEFAULT_STUN_PORT), G_STRUCT_OFFSET(GabbleParams, stun_port) },
-  { "ignore-ssl-errors", DBUS_TYPE_BOOLEAN_AS_STRING, G_TYPE_BOOLEAN, FALSE, GINT_TO_POINTER(FALSE), G_STRUCT_OFFSET(GabbleParams, ignore_ssl_errors) },
+  { "account", DBUS_TYPE_STRING_AS_STRING, G_TYPE_STRING, TP_CONN_MGR_PARAM_FLAG_REQUIRED | TP_CONN_MGR_PARAM_FLAG_REGISTER, NULL, G_STRUCT_OFFSET(GabbleParams, account) },
+  { "password", DBUS_TYPE_STRING_AS_STRING, G_TYPE_STRING, TP_CONN_MGR_PARAM_FLAG_REQUIRED | TP_CONN_MGR_PARAM_FLAG_REGISTER, NULL, G_STRUCT_OFFSET(GabbleParams, password) },
+  { "server", DBUS_TYPE_STRING_AS_STRING, G_TYPE_STRING, 0, NULL, G_STRUCT_OFFSET(GabbleParams, server) },
+  { "resource", DBUS_TYPE_STRING_AS_STRING, G_TYPE_STRING, TP_CONN_MGR_PARAM_FLAG_HAS_DEFAULT, GABBLE_PARAMS_DEFAULT_RESOURCE, G_STRUCT_OFFSET(GabbleParams, resource) },
+  { "priority", DBUS_TYPE_INT16_AS_STRING, G_TYPE_INT, TP_CONN_MGR_PARAM_FLAG_HAS_DEFAULT, GINT_TO_POINTER(0), G_STRUCT_OFFSET(GabbleParams, priority) },
+  { "port", DBUS_TYPE_UINT16_AS_STRING, G_TYPE_UINT, TP_CONN_MGR_PARAM_FLAG_HAS_DEFAULT, GINT_TO_POINTER(GABBLE_PARAMS_DEFAULT_PORT), G_STRUCT_OFFSET(GabbleParams, port) },
+  { "old-ssl", DBUS_TYPE_BOOLEAN_AS_STRING, G_TYPE_BOOLEAN, TP_CONN_MGR_PARAM_FLAG_HAS_DEFAULT, GINT_TO_POINTER(FALSE), G_STRUCT_OFFSET(GabbleParams, old_ssl) },
+  { "register", DBUS_TYPE_BOOLEAN_AS_STRING, G_TYPE_BOOLEAN, TP_CONN_MGR_PARAM_FLAG_HAS_DEFAULT, GINT_TO_POINTER(FALSE), G_STRUCT_OFFSET(GabbleParams, do_register) },
+  { "low-bandwidth", DBUS_TYPE_BOOLEAN_AS_STRING, G_TYPE_BOOLEAN, TP_CONN_MGR_PARAM_FLAG_HAS_DEFAULT, GINT_TO_POINTER(FALSE), G_STRUCT_OFFSET(GabbleParams, low_bandwidth) },
+  { "https-proxy-server", DBUS_TYPE_STRING_AS_STRING, G_TYPE_STRING, 0, NULL, G_STRUCT_OFFSET(GabbleParams, https_proxy_server) },
+  { "https-proxy-port", DBUS_TYPE_UINT16_AS_STRING, G_TYPE_UINT, TP_CONN_MGR_PARAM_FLAG_HAS_DEFAULT, GINT_TO_POINTER(GABBLE_PARAMS_DEFAULT_HTTPS_PROXY_PORT), G_STRUCT_OFFSET(GabbleParams, https_proxy_port) },
+  { "fallback-conference-server", DBUS_TYPE_STRING_AS_STRING, G_TYPE_STRING, 0, NULL, G_STRUCT_OFFSET(GabbleParams, fallback_conference_server) },
+  { "stun-server", DBUS_TYPE_STRING_AS_STRING, G_TYPE_STRING, 0, NULL, G_STRUCT_OFFSET(GabbleParams, stun_server) },
+  { "stun-port", DBUS_TYPE_UINT16_AS_STRING, G_TYPE_UINT, TP_CONN_MGR_PARAM_FLAG_HAS_DEFAULT, GINT_TO_POINTER(GABBLE_PARAMS_DEFAULT_STUN_PORT), G_STRUCT_OFFSET(GabbleParams, stun_port) },
+  { "ignore-ssl-errors", DBUS_TYPE_BOOLEAN_AS_STRING, G_TYPE_BOOLEAN, TP_CONN_MGR_PARAM_FLAG_HAS_DEFAULT, GINT_TO_POINTER(FALSE), G_STRUCT_OFFSET(GabbleParams, ignore_ssl_errors) },
+  { "alias", DBUS_TYPE_STRING_AS_STRING, G_TYPE_STRING, 0, NULL, G_STRUCT_OFFSET(GabbleParams, alias) },
+  { "mac", DBUS_TYPE_STRING_AS_STRING, G_TYPE_STRING, 0, NULL, G_STRUCT_OFFSET(GabbleParams, auth_mac) },
+  { "btid", DBUS_TYPE_STRING_AS_STRING, G_TYPE_STRING, 0, NULL, G_STRUCT_OFFSET(GabbleParams, auth_btid) },
   { NULL, NULL, 0, 0, NULL, 0 }
 };
 
+static const GabbleProtocolSpec _gabble_protocols[] = {
+  { "jabber", jabber_params },
+  { NULL, NULL }
+};
+const GabbleProtocolSpec *gabble_protocols = _gabble_protocols;
+
 /* private methods */
 
 static gboolean
@@ -216,8 +229,8 @@ get_parameters (const char *proto, const GabbleParamSpec **params, GError **erro
     {
       g_debug ("%s: unknown protocol %s", G_STRFUNC, proto);
 
-      *error = g_error_new (TELEPATHY_ERRORS, NotImplemented,
-                            "unknown protocol %s", proto);
+      g_set_error (error, TELEPATHY_ERRORS, NotImplemented,
+          "unknown protocol %s", proto);
 
       return FALSE;
     }
@@ -225,65 +238,43 @@ get_parameters (const char *proto, const GabbleParamSpec **params, GError **erro
   return TRUE;
 }
 
-static gboolean
-list_parameters (const GabbleParamSpec *params, gboolean mandatory, GHashTable **ret)
+static GValue *param_default_value (const GabbleParamSpec *params, int i)
 {
-  int i;
+  GValue *value;
 
-  *ret = g_hash_table_new(g_str_hash, g_str_equal);
+  value = g_new0(GValue, 1);
+  g_value_init(value, params[i].gtype);
 
-  for (i = 0; params[i].name; i++)
+  /* TODO: this check could be stricter if we knew whether register
+  was true. In practice REQUIRED and REGISTER always go together in
+  the Gabble params, though */
+  if (params[i].flags & TP_CONN_MGR_PARAM_FLAG_REQUIRED & TP_CONN_MGR_PARAM_FLAG_REGISTER)
     {
-      if (params[i].mandatory == mandatory)
-        g_hash_table_insert (*ret, (gpointer) params[i].name, (gpointer) params[i].dtype);
+      g_assert(params[i].def == NULL);
+      goto OUT;
     }
 
-  return TRUE;
-}
-
-static gboolean
-parameter_defaults (const GabbleParamSpec *params, GHashTable **ret)
-{
-  int i;
-
-  *ret = g_hash_table_new_full (g_str_hash, g_str_equal, NULL, g_free);
-
-  for (i = 0; params[i].name; i++)
+  switch (params[i].dtype[0])
     {
-      GValue *value;
-
-      if (params[i].mandatory)
-        {
-          g_assert (params[i].def == NULL);
-          continue;
-        }
-
-      value = g_new0(GValue, 1);
-      g_value_init (value, params[i].gtype);
-
-      switch (params[i].dtype[0])
-        {
-          case DBUS_TYPE_STRING:
-            g_value_set_static_string (value, (const gchar*) params[i].def);
-            break;
-          case DBUS_TYPE_INT16:
-            g_value_set_int (value, GPOINTER_TO_INT (params[i].def));
-            break;
-          case DBUS_TYPE_UINT16:
-            g_value_set_uint (value, GPOINTER_TO_INT (params[i].def));
-            break;
-          case DBUS_TYPE_BOOLEAN:
-            g_value_set_boolean (value, GPOINTER_TO_INT (params[i].def));
-            break;
-          default:
-            g_error ("parameter_defaults: encountered unknown type %s on argument %s",
-                     params[i].dtype, params[i].name);
-        }
-
-      g_hash_table_insert (*ret, (gpointer) params[i].name, value);
+      case DBUS_TYPE_STRING:
+        g_value_set_static_string (value, (const gchar*) params[i].def);
+        break;
+      case DBUS_TYPE_INT16:
+        g_value_set_int (value, GPOINTER_TO_INT (params[i].def));
+        break;
+      case DBUS_TYPE_UINT16:
+        g_value_set_uint (value, GPOINTER_TO_INT (params[i].def));
+        break;
+      case DBUS_TYPE_BOOLEAN:
+        g_value_set_boolean (value, GPOINTER_TO_INT (params[i].def));
+        break;
+      default:
+        g_error("parameter_defaults: encountered unknown type %s on argument %s",
+                params[i].dtype, params[i].name);
     }
 
-  return TRUE;
+OUT:
+  return value;
 }
 
 static gboolean
@@ -298,10 +289,10 @@ set_param_from_value (const GabbleParamSpec *paramspec,
                G_STRFUNC,
                g_type_name (paramspec->gtype), paramspec->name,
                G_VALUE_TYPE_NAME (value));
-      *error = g_error_new (TELEPATHY_ERRORS, InvalidArgument,
-                            "expected type %s for account parameter %s, got %s",
-                            g_type_name (paramspec->gtype), paramspec->name,
-                            G_VALUE_TYPE_NAME (value));
+      g_set_error (error, TELEPATHY_ERRORS, InvalidArgument,
+          "expected type %s for account parameter %s, got %s",
+          g_type_name (paramspec->gtype), paramspec->name,
+          G_VALUE_TYPE_NAME (value));
       return FALSE;
     }
 
@@ -342,24 +333,30 @@ parse_parameters (const GabbleParamSpec *paramspec,
 {
   int unhandled;
   int i;
+  guint mandatory_flag = TP_CONN_MGR_PARAM_FLAG_REQUIRED;
+  GValue *value;
 
   unhandled = g_hash_table_size (provided);
 
+  value = g_hash_table_lookup (provided, "register");
+  if (value != NULL && G_VALUE_TYPE(value) == G_TYPE_BOOLEAN &&
+      g_value_get_boolean(value))
+    {
+      mandatory_flag = TP_CONN_MGR_PARAM_FLAG_REGISTER;
+    }
+
   for (i = 0; paramspec[i].name; i++)
     {
-      GValue *value;
-
       value = g_hash_table_lookup (provided, paramspec[i].name);
 
       if (value == NULL)
         {
-          if (paramspec[i].mandatory)
+          if (paramspec[i].flags & mandatory_flag)
             {
               g_debug ("%s: missing mandatory param %s",
                        G_STRFUNC, paramspec[i].name);
-              *error = g_error_new (TELEPATHY_ERRORS, InvalidArgument,
-                                    "missing mandatory account parameter %s",
-                                    paramspec[i].name);
+              g_set_error (error, TELEPATHY_ERRORS, InvalidArgument,
+                  "missing mandatory account parameter %s", paramspec[i].name);
               return FALSE;
             }
           else
@@ -371,7 +368,11 @@ parse_parameters (const GabbleParamSpec *paramspec,
       else
         {
           if (!set_param_from_value (&paramspec[i], value, params, error))
-            return FALSE;
+            {
+              g_set_error (error, TELEPATHY_ERRORS, InvalidArgument,
+                  "invalid value for parameter %s", paramspec[i].name);
+              return FALSE;
+            }
 
           params->set_mask |= 1 << i;
 
@@ -402,8 +403,8 @@ parse_parameters (const GabbleParamSpec *paramspec,
   if (unhandled)
     {
       g_debug ("%s: unknown argument name provided", G_STRFUNC);
-      *error = g_error_new (TELEPATHY_ERRORS, InvalidArgument,
-                            "unknown argument name provided");
+      g_set_error (error, TELEPATHY_ERRORS, InvalidArgument,
+          "unknown argument name provided");
       return FALSE;
     }
 
@@ -420,6 +421,9 @@ free_params (GabbleParams *params)
   g_free (params->https_proxy_server);
   g_free (params->fallback_conference_server);
   g_free (params->stun_server);
+  g_free (params->alias);
+  g_free (params->auth_mac);
+  g_free (params->auth_btid);
 }
 
 /**
@@ -466,7 +470,7 @@ _gabble_connection_manager_register (GabbleConnectionManager *self)
   bus_proxy = tp_get_bus_proxy ();
 
   if (!dbus_g_proxy_call (bus_proxy, "RequestName", &error,
-                          G_TYPE_STRING, BUS_NAME,
+                          G_TYPE_STRING, GABBLE_CONN_MGR_BUS_NAME,
                           G_TYPE_UINT, DBUS_NAME_FLAG_DO_NOT_QUEUE,
                           G_TYPE_INVALID,
                           G_TYPE_UINT, &request_name_result,
@@ -474,41 +478,128 @@ _gabble_connection_manager_register (GabbleConnectionManager *self)
     g_error ("Failed to request bus name: %s", error->message);
 
   if (request_name_result == DBUS_REQUEST_NAME_REPLY_EXISTS)
-    g_error ("Failed to acquire bus name, connection manager already running?");
+    {
+      g_warning ("Failed to acquire bus name, connection manager already running?");
+      exit (1);
+    }
 
-  dbus_g_connection_register_g_object (bus, OBJECT_PATH, G_OBJECT (self));
+  dbus_g_connection_register_g_object (bus, GABBLE_CONN_MGR_OBJECT_PATH, G_OBJECT (self));
 }
 
 /* dbus-exported methods */
 
+/**
+ * gabble_connection_manager_get_parameters
+ *
+ * Implements D-Bus method GetParameters
+ * on interface org.freedesktop.Telepathy.ConnectionManager
+ *
+ * @error: Used to return a pointer to a GError detailing any error
+ *         that occurred, D-Bus will throw the error only if this
+ *         function returns FALSE.
+ *
+ * Returns: TRUE if successful, FALSE if an error was thrown.
+ */
+gboolean
+gabble_connection_manager_get_parameters (GabbleConnectionManager *self,
+                                          const gchar *proto,
+                                          GPtrArray **ret,
+                                          GError **error)
+{
+  const GabbleParamSpec *params = NULL;
+  int i;
+
+  if (!get_parameters (proto, &params, error))
+    return FALSE;
+
+  *ret = g_ptr_array_new ();
+
+  for (i = 0; params[i].name; i++)
+    {
+      GValue *def_value;
+      GValue param = { 0, };
+
+      g_value_init (&param, TP_TYPE_PARAM);
+      g_value_set_static_boxed (&param,
+        dbus_g_type_specialized_construct (TP_TYPE_PARAM));
+      
+      def_value = param_default_value (params, i);
+      dbus_g_type_struct_set (&param,
+        0, params[i].name,
+        1, params[i].flags,
+        2, params[i].dtype,
+        3, def_value,
+        G_MAXUINT);
+      g_value_unset(def_value);
+      g_free(def_value);
+
+      g_ptr_array_add (*ret, g_value_get_boxed (&param));
+    }
+
+  return TRUE;
+}
+
+
+/**
+ * gabble_connection_manager_list_protocols
+ *
+ * Implements D-Bus method ListProtocols
+ * on interface org.freedesktop.Telepathy.ConnectionManager
+ *
+ * @error: Used to return a pointer to a GError detailing any error
+ *         that occurred, D-Bus will throw the error only if this
+ *         function returns FALSE.
+ *
+ * Returns: TRUE if successful, FALSE if an error was thrown.
+ */
+gboolean
+gabble_connection_manager_list_protocols (GabbleConnectionManager *self,
+                                          gchar ***ret,
+                                          GError **error)
+{
+  const char *protocols[] = { "jabber", NULL };
+
+  *ret = g_strdupv ((gchar **)protocols);
+
+  return TRUE;
+}
+
+
 #define SET_PROPERTY_IF_PARAM_SET(prop, param, member) \
   if ((params.set_mask & (1 << param)) != 0) \
     { \
       g_object_set (conn, prop, member, NULL); \
     }
 
+
 /**
- * gabble_connection_manager_connect
+ * gabble_connection_manager_request_connection
  *
- * Implements DBus method Connect
+ * Implements D-Bus method RequestConnection
  * on interface org.freedesktop.Telepathy.ConnectionManager
  *
  * @error: Used to return a pointer to a GError detailing any error
- *         that occured, DBus will throw the error only if this
- *         function returns false.
+ *         that occurred, D-Bus will throw the error only if this
+ *         function returns FALSE.
  *
  * Returns: TRUE if successful, FALSE if an error was thrown.
  */
-gboolean gabble_connection_manager_connect (GabbleConnectionManager *obj, const gchar * proto, GHashTable * parameters, gchar ** bus_name, gchar ** object_path, GError **error)
+gboolean
+gabble_connection_manager_request_connection (GabbleConnectionManager *self,
+                                              const gchar *proto,
+                                              GHashTable *parameters,
+                                              gchar **bus_name,
+                                              gchar **object_path,
+                                              GError **error)
 {
   GabbleConnectionManagerPrivate *priv;
   GabbleConnection *conn;
   const GabbleParamSpec *paramspec;
   GabbleParams params = { 0, };
 
-  g_assert (GABBLE_IS_CONNECTION_MANAGER (obj));
+  g_assert (GABBLE_IS_CONNECTION_MANAGER (self));
 
-  priv = GABBLE_CONNECTION_MANAGER_GET_PRIVATE (obj);
+  priv = GABBLE_CONNECTION_MANAGER_GET_PRIVATE (self);
 
   if (!get_parameters (proto, &paramspec, error))
     return FALSE;
@@ -551,6 +642,11 @@ gboolean gabble_connection_manager_connect (GabbleConnectionManager *obj, const
   SET_PROPERTY_IF_PARAM_SET ("ignore-ssl-errors",
                               JABBER_PARAM_IGNORE_SSL_ERRORS,
                               params.ignore_ssl_errors);
+  SET_PROPERTY_IF_PARAM_SET ("alias", JABBER_PARAM_ALIAS, params.alias);
+  SET_PROPERTY_IF_PARAM_SET ("auth-mac", JABBER_PARAM_AUTH_MAC,
+                             params.auth_mac);
+  SET_PROPERTY_IF_PARAM_SET ("auth-btid", JABBER_PARAM_AUTH_BTID,
+                             params.auth_btid);
 
   /* split up account into username, stream-server and resource */
   if (!_gabble_connection_set_properties_from_account (conn, params.account, error))
@@ -572,21 +668,13 @@ gboolean gabble_connection_manager_connect (GabbleConnectionManager *obj, const
   /* bind to status change signals from the connection object */
   g_signal_connect (conn, "disconnected",
                              G_CALLBACK (connection_disconnected_cb),
-                             obj);
+                             self);
 
-  /* store the connection, using a hash table as a set*/
+  /* store the connection, using a hash table as a set */
   g_hash_table_insert (priv->connections, conn, GINT_TO_POINTER(TRUE));
 
-  /* commence connecting */
-  if (!_gabble_connection_connect (conn, error))
-    {
-      g_debug ("%s failed: %s", G_STRFUNC, (*error)->message);
-
-      goto ERROR;
-    }
-
   /* emit the new connection signal */
-  g_signal_emit (obj, signals[NEW_CONNECTION], 0, *bus_name, *object_path, proto);
+  g_signal_emit (self, signals[NEW_CONNECTION], 0, *bus_name, *object_path, proto);
 
   return TRUE;
 
@@ -596,94 +684,3 @@ ERROR:
 
   return FALSE;
 }
-
-/**
- * gabble_connection_manager_get_mandatory_parameters
- *
- * Implements DBus method GetMandatoryParameters
- * on interface org.freedesktop.Telepathy.ConnectionManager
- *
- * @error: Used to return a pointer to a GError detailing any error
- *         that occured, DBus will throw the error only if this
- *         function returns false.
- *
- * Returns: TRUE if successful, FALSE if an error was thrown.
- */
-gboolean gabble_connection_manager_get_mandatory_parameters (GabbleConnectionManager *obj, const gchar * proto, GHashTable ** ret, GError **error)
-{
-  const GabbleParamSpec *params = NULL;
-
-  if (!get_parameters (proto, &params, error))
-    return FALSE;
-
-  return list_parameters (params, TRUE, ret);
-}
-
-
-/**
- * gabble_connection_manager_get_optional_parameters
- *
- * Implements DBus method GetOptionalParameters
- * on interface org.freedesktop.Telepathy.ConnectionManager
- *
- * @error: Used to return a pointer to a GError detailing any error
- *         that occured, DBus will throw the error only if this
- *         function returns false.
- *
- * Returns: TRUE if successful, FALSE if an error was thrown.
- */
-gboolean gabble_connection_manager_get_optional_parameters (GabbleConnectionManager *obj, const gchar * proto, GHashTable ** ret, GError **error)
-{
-  const GabbleParamSpec *params = NULL;
-
-  if (!get_parameters (proto, &params, error))
-    return FALSE;
-
-  return list_parameters (params, FALSE, ret);
-}
-
-
-/**
- * gabble_connection_manager_get_parameter_defaults
- *
- * Implements DBus method GetParameterDefaults
- * on interface org.freedesktop.Telepathy.ConnectionManager
- *
- * @error: Used to return a pointer to a GError detailing any error
- *         that occured, DBus will throw the error only if this
- *         function returns false.
- *
- * Returns: TRUE if successful, FALSE if an error was thrown.
- */
-gboolean gabble_connection_manager_get_parameter_defaults (GabbleConnectionManager *obj, const gchar * proto, GHashTable ** ret, GError **error)
-{
-  const GabbleParamSpec *params = NULL;
-
-  if (!get_parameters (proto, &params, error))
-    return FALSE;
-
-  return parameter_defaults (params, ret);
-}
-
-
-/**
- * gabble_connection_manager_list_protocols
- *
- * Implements DBus method ListProtocols
- * on interface org.freedesktop.Telepathy.ConnectionManager
- *
- * @error: Used to return a pointer to a GError detailing any error
- *         that occured, DBus will throw the error only if this
- *         function returns false.
- *
- * Returns: TRUE if successful, FALSE if an error was thrown.
- */
-gboolean gabble_connection_manager_list_protocols (GabbleConnectionManager *obj, gchar *** ret, GError **error)
-{
-  const char *protocols[] = { "jabber", NULL };
-
-  *ret = g_strdupv ((gchar **)protocols);
-
-  return TRUE;
-}
-
