@@ -21,13 +21,22 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "namespaces.h"
+/* when five DIFFERENT guys report the same caps for a given bundle, it'll be enough */
+#define CAPABILITY_BUNDLE_ENOUGH_TRUST 5
+#define DEBUG_FLAG GABBLE_DEBUG_PRESENCE
 
+#include "debug.h"
+#include "disco.h" /* \o\ \o/ /o/ */
 #include "gabble-presence.h"
+#include "namespaces.h"
 #include "util.h"
+#include "handle-set.h"
+#include "gintset.h"
 
 #include "gabble-presence-cache.h"
 
+#include "gabble-presence-cache-signals-marshal.h"
+
 G_DEFINE_TYPE (GabblePresenceCache, gabble_presence_cache, G_TYPE_OBJECT);
 
 /* properties */
@@ -42,6 +51,7 @@ enum
 {
   PRESENCE_UPDATE,
   NICKNAME_UPDATE,
+  CAPABILITIES_UPDATE,
   LAST_SIGNAL
 };
 
@@ -59,10 +69,141 @@ struct _GabblePresenceCachePrivate
   LmMessageHandler *lm_message_cb;
 
   GHashTable *presence;
+  GabbleHandleSet *presence_handles;
+
+  GHashTable *capabilities;
+  GHashTable *disco_pending;
+  guint caps_serial;
 
   gboolean dispose_has_run;
 };
 
+typedef struct _DiscoWaiter DiscoWaiter;
+
+struct _DiscoWaiter
+{
+  GabbleHandleRepo *repo;
+  GabbleHandle handle;
+  gchar *resource;
+  guint serial;
+  gboolean disco_requested;
+};
+
+/**
+ * disco_waiter_new ()
+ */
+static DiscoWaiter *
+disco_waiter_new (GabbleHandleRepo *repo, GabbleHandle handle, const gchar *resource, guint serial)
+{
+  DiscoWaiter *waiter;
+
+  g_assert (repo);
+  gabble_handle_ref (repo, TP_HANDLE_TYPE_CONTACT, handle);
+
+  waiter = g_new0 (DiscoWaiter, 1);
+  waiter->repo = repo;
+  waiter->handle = handle;
+  waiter->resource = g_strdup (resource);
+  waiter->serial = serial;
+
+  DEBUG ("created waiter %p for handle %u with serial %u", waiter, handle, serial);
+
+  return waiter;
+}
+
+static void
+disco_waiter_free (DiscoWaiter *waiter)
+{
+  g_assert (NULL != waiter);
+
+  DEBUG ("freeing waiter %p for handle %u with serial %u", waiter, waiter->handle, waiter->serial);
+
+  gabble_handle_unref (waiter->repo, TP_HANDLE_TYPE_CONTACT, waiter->handle);
+
+  g_free (waiter->resource);
+  g_free (waiter);
+}
+
+static void
+disco_waiter_list_free (GSList *list)
+{
+  GSList *i;
+
+  DEBUG ("list %p", list);
+
+  for (i = list; NULL != i; i = i->next)
+    disco_waiter_free ((DiscoWaiter *) i->data);
+
+  g_slist_free (list);
+}
+
+static guint
+disco_waiter_list_get_request_count (GSList *list)
+{
+  guint c = 0;
+  GSList *i;
+
+  for (i = list; i; i = i->next)
+    {
+      DiscoWaiter *waiter = (DiscoWaiter *) i->data;
+
+      if (waiter->disco_requested)
+        c++;
+    }
+
+  return c;
+}
+
+typedef struct _CapabilityInfo CapabilityInfo;
+
+struct _CapabilityInfo
+{
+  GabblePresenceCapabilities caps;
+  GIntSet *guys;
+  guint trust;
+};
+
+static CapabilityInfo *
+capability_info_get (GabblePresenceCache *cache, const gchar *node,
+    GabblePresenceCapabilities caps)
+{
+  GabblePresenceCachePrivate *priv = GABBLE_PRESENCE_CACHE_PRIV (cache);
+  CapabilityInfo *info = g_hash_table_lookup (priv->capabilities, node);
+
+  if (NULL == info)
+    {
+      info = g_new0 (CapabilityInfo, 1);
+      info->caps = caps;
+      info->guys = g_intset_new ();
+      g_hash_table_insert (priv->capabilities, g_strdup (node), info);
+    }
+
+  return info;
+}
+
+static guint
+capability_info_recvd (GabblePresenceCache *cache, const gchar *node,
+        GabbleHandle handle, GabblePresenceCapabilities caps)
+{
+  CapabilityInfo *info = capability_info_get (cache, node, caps);
+
+  /* Detect inconsistency in reported caps */
+  if (info->caps != caps)
+    {
+      g_intset_clear (info->guys);
+      info->caps = caps;
+      info->trust = 0;
+    }
+
+  if (!g_intset_is_member (info->guys, handle))
+    {
+      g_intset_add (info->guys, handle);
+      info->trust++;
+    }
+
+  return info->trust;
+}
+
 static void gabble_presence_cache_init (GabblePresenceCache *presence_cache);
 static GObject * gabble_presence_cache_constructor (GType type, guint n_props,
     GObjectConstructParam *props);
@@ -122,6 +263,13 @@ gabble_presence_cache_class_init (GabblePresenceCacheClass *klass)
     0,
     NULL, NULL,
     g_cclosure_marshal_VOID__UINT, G_TYPE_NONE, 1, G_TYPE_UINT);
+  signals[CAPABILITIES_UPDATE] = g_signal_new (
+    "capabilities-update",
+    G_TYPE_FROM_CLASS (klass),
+    G_SIGNAL_RUN_LAST,
+    0,
+    NULL, NULL,
+    gabble_presence_cache_marshal_VOID__UINT_UINT_UINT, G_TYPE_NONE, 3, G_TYPE_UINT, G_TYPE_UINT, G_TYPE_UINT);
 }
 
 static void
@@ -133,6 +281,10 @@ gabble_presence_cache_init (GabblePresenceCache *cache)
   cache->priv = priv;
 
   priv->presence = g_hash_table_new_full (NULL, NULL, NULL, g_object_unref);
+  priv->capabilities = g_hash_table_new (g_str_hash, g_str_equal);
+  priv->disco_pending = g_hash_table_new_full (g_str_hash, g_str_equal,
+    g_free, (GDestroyNotify) disco_waiter_list_free);
+  priv->caps_serial = 1;
 }
 
 static GObject *
@@ -161,7 +313,7 @@ gabble_presence_cache_dispose (GObject *object)
   if (priv->dispose_has_run)
     return;
 
-  g_debug ("%s: dispose called", G_STRFUNC);
+  DEBUG ("dispose called");
 
   priv->dispose_has_run = TRUE;
 
@@ -172,6 +324,9 @@ gabble_presence_cache_dispose (GObject *object)
   g_hash_table_destroy (priv->presence);
   priv->presence = NULL;
 
+  handle_set_destroy (priv->presence_handles);
+  priv->presence_handles = NULL;
+
   if (G_OBJECT_CLASS (gabble_presence_cache_parent_class)->dispose)
     G_OBJECT_CLASS (gabble_presence_cache_parent_class)->dispose (object);
 }
@@ -179,7 +334,7 @@ gabble_presence_cache_dispose (GObject *object)
 static void
 gabble_presence_cache_finalize (GObject *object)
 {
-  g_debug ("%s called with %p", G_STRFUNC, object);
+  DEBUG ("called with %p", object);
 
   G_OBJECT_CLASS (gabble_presence_cache_parent_class)->finalize (object);
 }
@@ -211,10 +366,23 @@ gabble_presence_cache_set_property (GObject     *object,
 {
   GabblePresenceCache *cache = GABBLE_PRESENCE_CACHE (object);
   GabblePresenceCachePrivate *priv = GABBLE_PRESENCE_CACHE_PRIV (cache);
+  GabbleHandleSet *new_presence_handles;
 
   switch (property_id) {
     case PROP_CONNECTION:
       priv->conn = g_value_get_object (value);
+      new_presence_handles = handle_set_new (priv->conn->handles, TP_HANDLE_TYPE_CONTACT);
+
+      if (priv->presence_handles)
+        {
+          const GIntSet *add;
+          GIntSet *tmp;
+          add = handle_set_peek (priv->presence_handles);
+          tmp = handle_set_update (new_presence_handles, add);
+          handle_set_destroy (priv->presence_handles);
+          g_intset_destroy (tmp);
+        }
+      priv->presence_handles = new_presence_handles;
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -222,54 +390,41 @@ gabble_presence_cache_set_property (GObject     *object,
   }
 }
 
+#if 0
 static gboolean
 _presence_node_has_google_voice (LmMessageNode *pres_node)
 {
   LmMessageNode *node;
+  const gchar *cap_ext;
+  gchar **features, **tmp;
+  gboolean found = FALSE;
 
-  for (node = pres_node->children; node; node = node->next)
+  node = lm_message_node_get_child_with_namespace (pres_node, "c", NS_CAPS);
+
+  if (node == NULL);
+    return FALSE;
+
+  cap_ext = lm_message_node_get_attribute (node, "ext");
+
+  if (cap_ext == NULL);
+    return FALSE;
+
+  features = g_strsplit (cap_ext, " ", 0);
+
+  for (tmp = features; *tmp; tmp++)
     {
-      const gchar *cap_xmlns, *cap_ext;
-
-      if (strcmp (node->name, "c") != 0)
-        continue;
-
-      cap_xmlns = lm_message_node_get_attribute (node, "xmlns");
-      if (NULL == cap_xmlns || 0 != strcmp (cap_xmlns, NS_CAPS))
-        continue;
-
-      cap_ext = lm_message_node_get_attribute (node, "ext");
-      if (NULL != cap_ext)
+      if (!g_strdiff (tmp, "voice-v1"))
         {
-          gchar **features, **tmp;
-          gboolean found = FALSE;
-
-          features = g_strsplit (cap_ext, " ", 0);
-
-          for (tmp = features; *tmp; tmp++)
-            {
-              if (0 == strcmp (*tmp, "voice-v1"))
-                {
-                  found = TRUE;
-                  break;
-                }
-            }
-
-          g_strfreev (features);
-
-          if (found)
-            {
-              return TRUE;
-            }
-        }
-      else
-        {
-          continue;
+          found = TRUE;
+          break;
         }
     }
 
-  return FALSE;
+  g_strfreev (features);
+
+  return found;
 }
+#endif
 
 static void
 gabble_presence_cache_status_changed_cb (GabbleConnection *conn,
@@ -327,7 +482,7 @@ _presence_node_get_status (LmMessageNode *pres_node)
   if (!child_node)
     {
       /*
-      HANDLER_DEBUG (pres_node,
+      NODE_DEBUG (pres_node,
         "<presence> without <show> received from server, "
         "setting presence to available");
       */
@@ -339,7 +494,7 @@ _presence_node_get_status (LmMessageNode *pres_node)
   if (!presence_show)
     {
       /*
-      HANDLER_DEBUG (pres_node,
+      NODE_DEBUG (pres_node,
         "empty <show> tag received from server, "
         "setting presence to available");
       */
@@ -356,7 +511,7 @@ _presence_node_get_status (LmMessageNode *pres_node)
     return GABBLE_PRESENCE_XA;
   else
     {
-      HANDLER_DEBUG (pres_node,
+       NODE_DEBUG (pres_node,
         "unrecognised <show/> value received from server, "
         "setting presence to available");
       return GABBLE_PRESENCE_AVAILABLE;
@@ -370,26 +525,20 @@ _grab_nickname (GabblePresenceCache *cache,
                 LmMessageNode *node)
 {
   const gchar *nickname;
-  const gchar *xmlns;
   GabblePresence *presence;
 
-  node = lm_message_node_find_child (node, "nick");
+  node = lm_message_node_get_child_with_namespace (node, "nick", NS_NICK);
 
   if (NULL == node)
     return;
 
-  xmlns = lm_message_node_get_attribute (node, "xmlns");
-
-  if (0 != strcmp(xmlns, NS_NICK))
-    return;
-
   presence = gabble_presence_cache_get (cache, handle);
 
   if (NULL == presence)
     return;
 
   nickname = lm_message_node_get_value (node);
-  g_debug ("got nickname \"%s\" for %s", nickname, from);
+  DEBUG ("got nickname \"%s\" for %s", nickname, from);
 
   if (g_strdiff (presence->nickname, nickname))
     {
@@ -401,6 +550,321 @@ _grab_nickname (GabblePresenceCache *cache,
     }
 }
 
+static GSList *
+_extract_cap_bundles (LmMessageNode *lm_node)
+{
+  const gchar *node, *ver, *ext;
+  GSList *uris = NULL;
+  LmMessageNode *cap_node;
+
+  cap_node = lm_message_node_get_child_with_namespace (lm_node, "c", NS_CAPS);
+
+  if (NULL == cap_node)
+    return NULL;
+
+  node = lm_message_node_get_attribute (cap_node, "node");
+
+  if (NULL == node)
+    return NULL;
+
+  ver = lm_message_node_get_attribute (cap_node, "ver");
+
+  if (NULL != ver)
+    uris = g_slist_prepend (uris, g_strdup_printf ("%s#%s", node, ver));
+
+  ext = lm_message_node_get_attribute (cap_node, "ext");
+
+  if (NULL != ext)
+    {
+      gchar **exts, **i;
+
+      exts = g_strsplit (ext, " ", 0);
+
+      for (i = exts; NULL != *i; i++)
+        uris = g_slist_prepend (uris, g_strdup_printf ("%s#%s", node, *i));
+
+      g_strfreev (exts);
+    }
+
+  return uris;
+}
+
+static void
+_caps_disco_cb (GabbleDisco *disco,
+                GabbleDiscoRequest *request,
+                const gchar *jid,
+                const gchar *node,
+                LmMessageNode *query_result,
+                GError *error,
+                gpointer user_data)
+{
+  GSList *waiters, *i;
+  LmMessageNode *child;
+  GabblePresenceCache *cache;
+  GabblePresenceCachePrivate *priv;
+  gchar *full_jid = NULL;
+  GabblePresenceCapabilities caps = 0;
+  guint trust;
+  GabbleHandle handle;
+
+  cache = GABBLE_PRESENCE_CACHE (user_data);
+  priv = GABBLE_PRESENCE_CACHE_PRIV (cache);
+
+  if (NULL == node)
+    {
+      g_warning ("got disco response with NULL node, ignoring");
+      return;
+    }
+
+  waiters = g_hash_table_lookup (priv->disco_pending, node);
+
+  if (NULL != error)
+    {
+      DiscoWaiter *waiter = NULL;
+
+      DEBUG ("disco query failed: %s", error->message);
+
+      for (i = waiters; NULL != i; i = i->next)
+        {
+          waiter = (DiscoWaiter *) i->data;
+
+          if (!waiter->disco_requested)
+            {
+              const gchar *jid;
+
+              jid = gabble_handle_inspect (priv->conn->handles,
+                                           TP_HANDLE_TYPE_CONTACT,
+                                           waiter->handle);
+              full_jid = g_strdup_printf ("%s/%s", jid, waiter->resource);
+
+              gabble_disco_request (disco, GABBLE_DISCO_TYPE_INFO, full_jid, node,
+                                    _caps_disco_cb, cache, G_OBJECT(cache), NULL);
+              waiter->disco_requested = TRUE;
+              break;
+            }
+        }
+
+      if (NULL != i)
+        {
+          DEBUG ("sent a retry disco request to %s for URI %s", full_jid, node);
+        }
+      else
+        {
+          DEBUG ("failed to find a suitable candidate to retry disco request for URI %s", node);
+          /* FIXME do something very clever here? */
+          g_hash_table_remove (priv->disco_pending, node);
+        }
+
+      goto OUT;
+    }
+
+  for (child = query_result->children; NULL != child; child = child->next)
+    {
+      const gchar *var;
+
+      if (0 != strcmp (child->name, "feature"))
+        continue;
+
+      var = lm_message_node_get_attribute (child, "var");
+
+      if (NULL == var)
+        continue;
+
+      /* TODO: use a table that equates disco features to caps */
+      if (0 == strcmp (var, NS_GOOGLE_TRANSPORT_P2P))
+        caps |= PRESENCE_CAP_GOOGLE_TRANSPORT_P2P;
+      else if (0 == strcmp (var, NS_GOOGLE_FEAT_VOICE))
+        caps |= PRESENCE_CAP_GOOGLE_VOICE;
+      else if (0 == strcmp (var, NS_JINGLE))
+        caps |= PRESENCE_CAP_JINGLE;
+      else if (0 == strcmp (var, NS_JINGLE_DESCRIPTION_AUDIO))
+        caps |= PRESENCE_CAP_JINGLE_DESCRIPTION_AUDIO;
+      else if (0 == strcmp (var, NS_JINGLE_DESCRIPTION_VIDEO))
+        caps |= PRESENCE_CAP_JINGLE_DESCRIPTION_VIDEO;
+    }
+
+  handle = gabble_handle_for_contact (priv->conn->handles, jid, FALSE);
+  trust = capability_info_recvd (cache, node, handle, caps);
+
+  for (i = waiters; NULL != i;)
+    {
+      DiscoWaiter *waiter;
+      GabblePresence *presence;
+
+      waiter = (DiscoWaiter *) i->data;
+
+      if (trust >= CAPABILITY_BUNDLE_ENOUGH_TRUST || waiter->handle == handle)
+        {
+          GSList *tmp;
+          /* trusted reply */
+          presence = gabble_presence_cache_get (cache, waiter->handle);
+
+          if (presence)
+          {
+            GabblePresenceCapabilities save_caps = presence->caps;
+            DEBUG ("setting caps for %d (%s) to %d", handle, jid, caps);
+            gabble_presence_set_capabilities (presence, waiter->resource,caps,
+              waiter->serial);
+            DEBUG ("caps for %d (%s) now %d", handle, jid, presence->caps);
+            g_signal_emit (cache, signals[CAPABILITIES_UPDATE], 0,
+              waiter->handle, save_caps, presence->caps);
+          }
+
+          tmp = i;
+          i = i->next;
+
+          waiters = g_slist_delete_link (waiters, tmp);
+
+          g_hash_table_steal (priv->disco_pending, node);
+          g_hash_table_insert (priv->disco_pending, g_strdup (node), waiters);
+
+          disco_waiter_free (waiter);
+        }
+      else if (trust + disco_waiter_list_get_request_count (waiters) - 1
+          < CAPABILITY_BUNDLE_ENOUGH_TRUST)
+        {
+          /* if the possible trust, not counting this guy, is too low,
+           * we have been poisoned and reset our trust meters - disco
+           * anybody we still haven't to be able to get more trusted replies */
+
+          if (!waiter->disco_requested)
+            {
+              const gchar *jid;
+
+              jid = gabble_handle_inspect (priv->conn->handles,
+                TP_HANDLE_TYPE_CONTACT, waiter->handle);
+              full_jid = g_strdup_printf ("%s/%s", jid, waiter->resource);
+
+              gabble_disco_request (disco, GABBLE_DISCO_TYPE_INFO, full_jid,
+                  node, _caps_disco_cb, cache, G_OBJECT(cache), NULL);
+              waiter->disco_requested = TRUE;
+
+              g_free (full_jid);
+              full_jid = NULL;
+            }
+
+          i = i->next;
+        }
+      else
+        {
+          /* trust level still uncertain, don't do nothing */
+          i = i->next;
+        }
+    }
+
+  if (trust >= CAPABILITY_BUNDLE_ENOUGH_TRUST)
+    g_hash_table_remove (priv->disco_pending, node);
+
+OUT:
+
+  g_free (full_jid);
+}
+
+static void
+_process_caps_uri (GabblePresenceCache *cache,
+                   const gchar *from,
+                   const gchar *uri,
+                   GabbleHandle handle,
+                   const gchar *resource,
+                   guint serial)
+{
+  CapabilityInfo *info;
+  gpointer value;
+  GabblePresenceCachePrivate *priv;
+
+  priv = GABBLE_PRESENCE_CACHE_PRIV (cache);
+  info = capability_info_get (cache, uri, 0);
+
+  if (info->trust >= CAPABILITY_BUNDLE_ENOUGH_TRUST
+      || g_intset_is_member (info->guys, handle))
+    {
+      /* we already have enough trust for this node; apply the cached value to
+       * the (handle, resource) */
+
+      GabblePresence *presence = gabble_presence_cache_get (cache, handle);
+      DEBUG ("enough trust for URI %s, setting caps for %u (%s) to %u",
+          uri, handle, from, info->caps);
+
+      if (presence)
+        {
+          GabblePresenceCapabilities save_caps = presence->caps;
+          gabble_presence_set_capabilities (presence, resource, info->caps, serial);
+          g_signal_emit (cache, signals[CAPABILITIES_UPDATE], 0,
+              handle, save_caps, presence->caps);
+          DEBUG ("caps for %d (%s) now %d", handle, from, presence->caps);
+        }
+      else
+        {
+          DEBUG ("presence not found");
+        }
+    }
+  else
+    {
+      /* Append the (handle, resource) pair to the list of such pairs
+       * waiting for capabilities for this uri, and send a disco request
+       * if we don't have enough possible trust yet */
+
+      GSList *waiters;
+      DiscoWaiter *waiter;
+      guint possible_trust;
+
+      DEBUG ("not enough trust for URI %s", uri);
+      value = g_hash_table_lookup (priv->disco_pending, uri);
+
+      if (value)
+        g_hash_table_steal (priv->disco_pending, uri);
+
+      waiters = (GSList *) value;
+      waiter = disco_waiter_new (priv->conn->handles, handle, resource, serial);
+      waiters = g_slist_prepend (waiters, waiter);
+      g_hash_table_insert (priv->disco_pending, g_strdup (uri), waiters);
+
+      possible_trust = disco_waiter_list_get_request_count (waiters);
+
+      if (!value || info->trust + possible_trust < CAPABILITY_BUNDLE_ENOUGH_TRUST)
+        {
+          /* DISCO */
+          DEBUG ("only %u trust out of %u possible thus far, sending disco for URI %s",
+              info->trust + possible_trust, CAPABILITY_BUNDLE_ENOUGH_TRUST, uri);
+          gabble_disco_request (priv->conn->disco, GABBLE_DISCO_TYPE_INFO,
+              from, uri, _caps_disco_cb, cache, G_OBJECT (cache), NULL);
+          /* enough DISCO for you, buddy */
+          waiter->disco_requested = TRUE;
+        }
+    }
+}
+
+static void
+_process_caps (GabblePresenceCache *cache,
+               GabbleHandle handle,
+               const gchar *from,
+               LmMessageNode *lm_node)
+{
+  gchar *resource;
+  GSList *uris, *i;
+  GabblePresenceCachePrivate *priv;
+  guint serial;
+
+  priv = GABBLE_PRESENCE_CACHE_PRIV (cache);
+  serial = priv->caps_serial++;
+
+  gabble_decode_jid (from, NULL, NULL, &resource);
+
+  if (NULL == resource)
+    return;
+
+  uris = _extract_cap_bundles (lm_node);
+
+  for (i = uris; NULL != i; i = i->next)
+    {
+      _process_caps_uri (cache, from, (gchar *) i->data, handle, resource, serial);
+      g_free (i->data);
+    }
+
+  g_free (resource);
+  g_slist_free (uris);
+}
+
 static LmHandlerResult
 _parse_presence_message (GabblePresenceCache *cache,
                          GabbleHandle handle,
@@ -418,13 +882,7 @@ _parse_presence_message (GabblePresenceCache *cache,
   presence_node = message->node;
   g_assert (0 == strcmp (presence_node->name, "presence"));
 
-  gabble_handle_decode_jid (from, NULL, NULL, &resource);
-
-  if (resource == NULL)
-    {
-      HANDLER_DEBUG (presence_node, "ignoring presence with no resource");
-      return LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS;
-    }
+  gabble_decode_jid (from, NULL, NULL, &resource);
 
   presence = gabble_presence_cache_get (cache, handle);
 
@@ -454,20 +912,22 @@ _parse_presence_message (GabblePresenceCache *cache,
       gabble_presence_cache_update (cache, handle, resource, presence_id,
           status_message, priority);
 
+#if 0
       if (_presence_node_has_google_voice (presence_node))
         {
           presence = gabble_presence_cache_get (cache, handle);
           g_assert (NULL != presence);
-          g_debug ("%s: %s has voice-v1 support", G_STRFUNC, from);
+          DEBUG ("%s has voice-v1 support", from);
           gabble_presence_set_capabilities (presence, resource,
               PRESENCE_CAP_GOOGLE_VOICE);
         }
+#endif
 
       ret = LM_HANDLER_RESULT_REMOVE_MESSAGE;
-      goto OUT;
+      break;
 
     case LM_MESSAGE_SUB_TYPE_ERROR:
-      HANDLER_DEBUG (presence_node, "setting contact offline due to error");
+      NODE_DEBUG (presence_node, "setting contact offline due to error");
       /* fall through */
 
     case LM_MESSAGE_SUB_TYPE_UNAVAILABLE:
@@ -475,16 +935,17 @@ _parse_presence_message (GabblePresenceCache *cache,
           GABBLE_PRESENCE_OFFLINE, status_message, priority);
 
       ret = LM_HANDLER_RESULT_REMOVE_MESSAGE;
-      goto OUT;
+      break;
 
     default:
       break;
     }
 
-OUT:
   _grab_nickname (cache, handle, from, presence_node);
+  _process_caps (cache, handle, from, presence_node);
 
   g_free (resource);
+
   return ret;
 }
 
@@ -506,7 +967,9 @@ _parse_message_message (GabblePresenceCache *cache,
     }
 
   node = lm_message_get_node (message);
+
   _grab_nickname (cache, handle, from, node);
+  _process_caps (cache, handle, from, node);
 
   return LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS;
 }
@@ -538,7 +1001,7 @@ gabble_presence_cache_lm_message_cb (LmMessageHandler *handler,
 
   if (NULL == from)
     {
-      HANDLER_DEBUG (message->node, "presence stanza without from attribute, ignoring");
+      NODE_DEBUG (message->node, "message without from attribute, ignoring");
       return LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS;
     }
 
@@ -546,13 +1009,14 @@ gabble_presence_cache_lm_message_cb (LmMessageHandler *handler,
 
   if (0 == handle)
     {
-      HANDLER_DEBUG (message->node, "ignoring presence from malformed jid");
+      NODE_DEBUG (message->node, "ignoring message from malformed jid");
       return LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS;
     }
 
   if (handle == priv->conn->self_handle)
     {
-      HANDLER_DEBUG (message->node, "ignoring presence from ourselves on another resource");
+      NODE_DEBUG (message->node,
+        "ignoring message from ourselves on another resource");
       return LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS;
     }
 
@@ -608,10 +1072,9 @@ gabble_presence_cache_maybe_remove (
 
       jid = gabble_handle_inspect (priv->conn->handles, TP_HANDLE_TYPE_CONTACT,
           handle);
-      g_debug ("%s: discarding cached presence for unavailable jid %s",
-          G_STRFUNC, jid);
+      DEBUG ("discarding cached presence for unavailable jid %s", jid);
       g_hash_table_remove (priv->presence, GINT_TO_POINTER (handle));
-      gabble_handle_unref (priv->conn->handles, TP_HANDLE_TYPE_CONTACT, handle);
+      handle_set_remove (priv->presence_handles, handle);
     }
 }
 
@@ -625,7 +1088,7 @@ _cache_insert (
 
   presence = gabble_presence_new ();
   g_hash_table_insert (priv->presence, GINT_TO_POINTER (handle), presence);
-  gabble_handle_ref (priv->conn->handles, TP_HANDLE_TYPE_CONTACT, handle);
+  handle_set_add (priv->presence_handles, handle);
   return presence;
 }
 
@@ -644,8 +1107,8 @@ gabble_presence_cache_update (
 
   jid = gabble_handle_inspect (priv->conn->handles, TP_HANDLE_TYPE_CONTACT,
       handle);
-  g_debug ("%s: %s (%d) resource %s prio %d presence %d message \"%s\"",
-      G_STRFUNC, jid, handle, resource, priority, presence_id, status_message);
+  DEBUG ("%s (%d) resource %s prio %d presence %d message \"%s\"",
+      jid, handle, resource, priority, presence_id, status_message);
 
   presence = gabble_presence_cache_get (cache, handle);
 
@@ -659,3 +1122,13 @@ gabble_presence_cache_update (
   gabble_presence_cache_maybe_remove (cache, handle);
 }
 
+void gabble_presence_cache_add_bundle_caps (GabblePresenceCache *cache,
+    const gchar *node, GabblePresenceCapabilities new_caps)
+{
+  CapabilityInfo *info;
+
+  info = capability_info_get (cache, node, 0);
+  info->trust = CAPABILITY_BUNDLE_ENOUGH_TRUST;
+  info->caps |= new_caps;
+}
+
