@@ -23,13 +23,19 @@
 #include <stdlib.h>
 #include <string.h>
 
+#define DEBUG_FLAG GABBLE_DEBUG_ROOMLIST
+
+#include "debug.h"
 #include "disco.h"
 #include "gabble-connection.h"
 #include "handles.h"
+#include "handle-set.h"
 #include "telepathy-constants.h"
 #include "telepathy-interfaces.h"
 #include "telepathy-helpers.h"
 #include "tp-channel-iface.h"
+#include "namespaces.h"
+#include "util.h"
 
 #include "gabble-roomlist-channel.h"
 #include "gabble-roomlist-channel-glue.h"
@@ -44,8 +50,6 @@
 #define TP_TYPE_ROOM_LIST (dbus_g_type_get_collection ("GPtrArray", \
       TP_TYPE_ROOM_STRUCT))
 
-#define DISCO_PIPELINE_SIZE 10
-
 G_DEFINE_TYPE_WITH_CODE (GabbleRoomlistChannel, gabble_roomlist_channel,
     G_TYPE_OBJECT, G_IMPLEMENT_INTERFACE (TP_TYPE_CHANNEL_IFACE, NULL));
 
@@ -84,22 +88,22 @@ struct _GabbleRoomlistChannelPrivate
   gboolean closed;
   gboolean listing;
 
-  GPtrArray *disco_pipeline;
-  GHashTable *remaining_rooms;
+  gpointer disco_pipeline;
+  GabbleHandleSet *signalled_rooms;
 
   gboolean dispose_has_run;
 };
 
-#define GABBLE_ROOMLIST_CHANNEL_GET_PRIVATE(o)     (G_TYPE_INSTANCE_GET_PRIVATE ((o), GABBLE_TYPE_ROOMLIST_CHANNEL, GabbleRoomlistChannelPrivate))
+#define GABBLE_ROOMLIST_CHANNEL_GET_PRIVATE(obj) \
+    ((GabbleRoomlistChannelPrivate *)obj->priv)
 
 static void
-gabble_roomlist_channel_init (GabbleRoomlistChannel *obj)
+gabble_roomlist_channel_init (GabbleRoomlistChannel *self)
 {
-  GabbleRoomlistChannelPrivate *priv = GABBLE_ROOMLIST_CHANNEL_GET_PRIVATE (obj);
+  GabbleRoomlistChannelPrivate *priv = G_TYPE_INSTANCE_GET_PRIVATE (self,
+      GABBLE_TYPE_ROOMLIST_CHANNEL, GabbleRoomlistChannelPrivate);
 
-  priv->disco_pipeline = g_ptr_array_sized_new (DISCO_PIPELINE_SIZE);
-  priv->remaining_rooms = g_hash_table_new_full (g_str_hash, g_str_equal,
-      g_free, NULL);
+  self->priv = priv;
 }
 
 
@@ -163,6 +167,7 @@ gabble_roomlist_channel_set_property (GObject     *object,
 {
   GabbleRoomlistChannel *chan = GABBLE_ROOMLIST_CHANNEL (object);
   GabbleRoomlistChannelPrivate *priv = GABBLE_ROOMLIST_CHANNEL_GET_PRIVATE (chan);
+  GabbleHandleSet *new_signalled_rooms;
 
   switch (property_id) {
     case PROP_OBJECT_PATH:
@@ -175,6 +180,17 @@ gabble_roomlist_channel_set_property (GObject     *object,
       break;
     case PROP_CONNECTION:
       priv->conn = g_value_get_object (value);
+      new_signalled_rooms = handle_set_new (priv->conn->handles, TP_HANDLE_TYPE_ROOM);
+      if (priv->signalled_rooms != NULL)
+        {
+          const GIntSet *add;
+          GIntSet *tmp;
+          add = handle_set_peek (priv->signalled_rooms);
+          tmp = handle_set_update (new_signalled_rooms, add);
+          handle_set_destroy (priv->signalled_rooms);
+          g_intset_destroy (tmp);
+        }
+      priv->signalled_rooms = new_signalled_rooms;
       break;
     case PROP_CONFERENCE_SERVER:
       g_free (priv->conference_server);
@@ -238,7 +254,7 @@ gabble_roomlist_channel_class_init (GabbleRoomlistChannelClass *gabble_roomlist_
                   G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
                   0,
                   NULL, NULL,
-                  gabble_roomlist_channel_marshal_VOID__VOID,
+                  g_cclosure_marshal_VOID__VOID,
                   G_TYPE_NONE, 0);
 
   signals[GOT_ROOMS] =
@@ -247,7 +263,7 @@ gabble_roomlist_channel_class_init (GabbleRoomlistChannelClass *gabble_roomlist_
                   G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
                   0,
                   NULL, NULL,
-                  gabble_roomlist_channel_marshal_VOID__BOXED,
+                  g_cclosure_marshal_VOID__BOXED,
                   G_TYPE_NONE, 1, (dbus_g_type_get_collection ("GPtrArray", (dbus_g_type_get_struct ("GValueArray", G_TYPE_UINT, G_TYPE_STRING, (dbus_g_type_get_map ("GHashTable", G_TYPE_STRING, G_TYPE_VALUE)), G_TYPE_INVALID)))));
 
   signals[LISTING_ROOMS] =
@@ -256,30 +272,12 @@ gabble_roomlist_channel_class_init (GabbleRoomlistChannelClass *gabble_roomlist_
                   G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
                   0,
                   NULL, NULL,
-                  gabble_roomlist_channel_marshal_VOID__BOOLEAN,
+                  g_cclosure_marshal_VOID__BOOLEAN,
                   G_TYPE_NONE, 1, G_TYPE_BOOLEAN);
 
   dbus_g_object_type_install_info (G_TYPE_FROM_CLASS (gabble_roomlist_channel_class), &dbus_glib_gabble_roomlist_channel_object_info);
 }
 
-static void
-room_list_flush_disco_pipeline_one (gpointer item, gpointer data)
-{
-  GabbleDiscoRequest *request = (GabbleDiscoRequest *) item;
-  GabbleDisco *disco = GABBLE_DISCO (data);
-
-  gabble_disco_cancel_request (disco, request);
-}
-
-void
-room_list_flush_disco_pipeline (GabbleRoomlistChannel *self)
-{
-  GabbleRoomlistChannelPrivate *priv = GABBLE_ROOMLIST_CHANNEL_GET_PRIVATE (self);
-
-  g_ptr_array_foreach (priv->disco_pipeline, (GFunc)
-      room_list_flush_disco_pipeline_one, priv->conn->disco);
-}
-
 void
 gabble_roomlist_channel_dispose (GObject *object)
 {
@@ -303,7 +301,11 @@ gabble_roomlist_channel_dispose (GObject *object)
       priv->closed = TRUE;
     }
 
-  room_list_flush_disco_pipeline (self);
+  if (priv->disco_pipeline != NULL)
+    {
+      gabble_disco_pipeline_destroy (priv->disco_pipeline);
+      priv->disco_pipeline = NULL;
+    }
 
   if (G_OBJECT_CLASS (gabble_roomlist_channel_parent_class)->dispose)
     G_OBJECT_CLASS (gabble_roomlist_channel_parent_class)->dispose (object);
@@ -320,8 +322,8 @@ gabble_roomlist_channel_finalize (GObject *object)
   g_free (priv->object_path);
   g_free (priv->conference_server);
 
-  g_ptr_array_free (priv->disco_pipeline, TRUE);
-  g_hash_table_destroy (priv->remaining_rooms);
+  if (priv->signalled_rooms != NULL)
+    handle_set_destroy (priv->signalled_rooms);
 
   G_OBJECT_CLASS (gabble_roomlist_channel_parent_class)->finalize (object);
 }
@@ -342,56 +344,6 @@ _gabble_roomlist_channel_new (GabbleConnection *conn,
                     "conference-server", conference_server, NULL));
 }
 
-static void room_info_cb (GabbleDisco *, GabbleDiscoRequest *, const gchar *,
-    const gchar *, LmMessageNode *, GError *, gpointer);
-
-static gboolean
-return_true (gpointer key, gpointer value, gpointer data)
-{
-  return TRUE;
-}
-
-static void
-room_list_fill_disco_pipeline (GabbleRoomlistChannel *chan)
-{
-  GabbleRoomlistChannelPrivate *priv =
-    GABBLE_ROOMLIST_CHANNEL_GET_PRIVATE (chan);
-
-  if (priv->closed)
-    {
-      g_debug ("%s: not refilling pipeline, channel is closed", G_STRFUNC);
-    }
-  else
-    {
-      /* send disco requests for the JIDs in the remaining_rooms hash table
-       * until there are DISCO_PIPELINE_SIZE requests in progress */
-      while (priv->disco_pipeline->len < DISCO_PIPELINE_SIZE)
-        {
-          gchar *jid;
-          GabbleDiscoRequest *request;
-
-          jid = (gchar *) g_hash_table_find (priv->remaining_rooms,
-              return_true, NULL);
-          if (NULL == jid)
-            break;
-
-          request = gabble_disco_request (priv->conn->disco,
-              GABBLE_DISCO_TYPE_INFO, jid, NULL, room_info_cb, chan,
-              G_OBJECT(chan), NULL);
-
-          g_ptr_array_add (priv->disco_pipeline, request);
-
-          /* frees jid */
-          g_hash_table_remove (priv->remaining_rooms, jid);
-        }
-
-      if (0 == priv->disco_pipeline->len)
-        {
-          priv->listing = FALSE;
-          g_signal_emit (chan, signals [LISTING_ROOMS], 0, FALSE);
-        }
-    }
-}
 
 /**
  * destroy_value:
@@ -407,24 +359,17 @@ destroy_value (GValue *value)
 }
 
 static void
-room_info_cb (GabbleDisco *disco,
-              GabbleDiscoRequest *request,
-              const gchar *jid,
-              const gchar *node,
-              LmMessageNode *result,
-              GError *error,
-              gpointer user_data)
+room_info_cb (gpointer pipeline, GabbleDiscoItem *item, gpointer user_data)
 {
   GabbleRoomlistChannel *chan = user_data;
   GabbleRoomlistChannelPrivate *priv;
-  LmMessageNode *identity, *feature, *field, *value_node;
-  const char *category, *type, *var, *name, *namespace, *value;
+  const char *jid, *category, *type, *var, *name;
   GabbleHandle handle;
   GHashTable *keys;
   GValue room = {0,};
   GPtrArray *rooms ;
   GValue *tmp;
-  gboolean is_muc;
+  gpointer k, v;
 
   #define INSERT_KEY(hash, name, type, type2, value) \
     do {\
@@ -437,229 +382,133 @@ room_info_cb (GabbleDisco *disco,
   g_assert (GABBLE_IS_ROOMLIST_CHANNEL (chan));
   priv = GABBLE_ROOMLIST_CHANNEL_GET_PRIVATE (chan);
 
-  g_ptr_array_remove_fast (priv->disco_pipeline, request);
-
-  if (error)
-    {
-      g_debug ("%s: got error %s", G_STRFUNC, error->message);
-      goto done;
-    }
-
-  identity = lm_message_node_get_child (result, "identity");
-  if (NULL == identity)
-    goto done;
-
-  name = lm_message_node_get_attribute (identity, "name");
-  if (NULL == name)
-    goto done;
-
-  category = lm_message_node_get_attribute (identity, "category");
-  if (NULL == category)
-    goto done;
-
-  type = lm_message_node_get_attribute (identity, "type");
-  if (NULL == type)
-    goto done;
+  jid = item->jid;
+  name = item->name;
+  category = item->category;
+  type = item->type;
 
   if (0 != strcmp (category, "conference") ||
       0 != strcmp (type, "text"))
-    goto done;
+    return;
 
-  g_debug ("%s: got room identity, name=%s, category=%s, type=%s", G_STRFUNC,
-      name, category, type);
+  if (!g_hash_table_lookup_extended (item->features, "http://jabber.org/protocol/muc", &k, &v))
+    {
+      /* not muc */
+      return;
+    }
+
+  DEBUG ("got room identity, name=%s, category=%s, type=%s", name, category, type);
 
   keys = g_hash_table_new_full (g_str_hash, g_str_equal, NULL,
                                 (GDestroyNotify) destroy_value);
 
   INSERT_KEY (keys, "name", G_TYPE_STRING, string, name);
 
-  is_muc = FALSE;
+  if (g_hash_table_lookup_extended (item->features, "muc_membersonly", &k, &v))
+    INSERT_KEY (keys, "invite-only", G_TYPE_BOOLEAN, boolean, TRUE);
+  if (g_hash_table_lookup_extended (item->features, "muc_open", &k, &v))
+    INSERT_KEY (keys, "invite-only", G_TYPE_BOOLEAN, boolean, FALSE);
+  if (g_hash_table_lookup_extended (item->features, "muc_passwordprotected", &k, &v))
+    INSERT_KEY (keys, "password", G_TYPE_BOOLEAN, boolean, TRUE);
+  if (g_hash_table_lookup_extended (item->features, "muc_unsecure", &k, &v))
+    INSERT_KEY (keys, "password", G_TYPE_BOOLEAN, boolean, FALSE);
+  if (g_hash_table_lookup_extended (item->features, "muc_unsecured", &k, &v))
+    INSERT_KEY (keys, "password", G_TYPE_BOOLEAN, boolean, FALSE);
+  if (g_hash_table_lookup_extended (item->features, "muc_hidden", &k, &v))
+    INSERT_KEY (keys, "hidden", G_TYPE_BOOLEAN, boolean, TRUE);
+  if (g_hash_table_lookup_extended (item->features, "muc_public", &k, &v))
+    INSERT_KEY (keys, "hidden", G_TYPE_BOOLEAN, boolean, FALSE);
+  if (g_hash_table_lookup_extended (item->features, "muc_membersonly", &k, &v))
+    INSERT_KEY (keys, "members-only", G_TYPE_BOOLEAN, boolean, TRUE);
+  if (g_hash_table_lookup_extended (item->features, "muc_open", &k, &v))
+    INSERT_KEY (keys, "members-only", G_TYPE_BOOLEAN, boolean, FALSE);
+  if (g_hash_table_lookup_extended (item->features, "muc_moderated", &k, &v))
+    INSERT_KEY (keys, "moderated", G_TYPE_BOOLEAN, boolean, TRUE);
+  if (g_hash_table_lookup_extended (item->features, "muc_unmoderated", &k, &v))
+    INSERT_KEY (keys, "moderated", G_TYPE_BOOLEAN, boolean, FALSE);
+  if (g_hash_table_lookup_extended (item->features, "muc_nonanonymous", &k, &v))
+    INSERT_KEY (keys, "anonymous", G_TYPE_BOOLEAN, boolean, FALSE);
+  if (g_hash_table_lookup_extended (item->features, "muc_anonymous", &k, &v))
+    INSERT_KEY (keys, "anonymous", G_TYPE_BOOLEAN, boolean, TRUE);
+  if (g_hash_table_lookup_extended (item->features, "muc_semianonymous", &k, &v))
+    INSERT_KEY (keys, "anonymous", G_TYPE_BOOLEAN, boolean, FALSE);
+  if (g_hash_table_lookup_extended (item->features, "muc_persistent", &k, &v))
+    INSERT_KEY (keys, "persistent", G_TYPE_BOOLEAN, boolean, TRUE);
+  if (g_hash_table_lookup_extended (item->features, "muc_temporary", &k, &v))
+    INSERT_KEY (keys, "persistent", G_TYPE_BOOLEAN, boolean, FALSE);
 
-  for (feature = result->children; feature; feature = feature->next)
-    {
-      if (0 == strcmp (feature->name, "feature"))
-        {
-          var = lm_message_node_get_attribute (feature, "var");
-          if (!var)
-            continue;
-          if (0 == strcmp (var, "http://jabber.org/protocol/muc"))
-            is_muc = TRUE;
-          else if (0 == strcmp (var, "muc_membersonly"))
-            INSERT_KEY (keys, "invite-only", G_TYPE_BOOLEAN, boolean, TRUE);
-          else if (0 == strcmp (var, "muc_open"))
-            INSERT_KEY (keys, "invite-only", G_TYPE_BOOLEAN, boolean, FALSE);
-          else if (0 == strcmp (var, "muc_passwordprotected"))
-            INSERT_KEY (keys, "password", G_TYPE_BOOLEAN, boolean, TRUE);
-          else if (0 == strcmp (var, "muc_unsecure"))
-            INSERT_KEY (keys, "password", G_TYPE_BOOLEAN, boolean, FALSE);
-          else if (0 == strcmp (var, "muc_unsecured"))
-            INSERT_KEY (keys, "password", G_TYPE_BOOLEAN, boolean, FALSE);
-          else if (0 == strcmp (var, "muc_hidden"))
-            INSERT_KEY (keys, "hidden", G_TYPE_BOOLEAN, boolean, TRUE);
-          else if (0 == strcmp (var, "muc_public"))
-            INSERT_KEY (keys, "hidden", G_TYPE_BOOLEAN, boolean, FALSE);
-          else if (0 == strcmp (var, "muc_membersonly"))
-            INSERT_KEY (keys, "members-only", G_TYPE_BOOLEAN, boolean, TRUE);
-          else if (0 == strcmp (var, "muc_open"))
-            INSERT_KEY (keys, "members-only", G_TYPE_BOOLEAN, boolean, FALSE);
-          else if (0 == strcmp (var, "muc_moderated"))
-            INSERT_KEY (keys, "moderated", G_TYPE_BOOLEAN, boolean, TRUE);
-          else if (0 == strcmp (var, "muc_unmoderated"))
-            INSERT_KEY (keys, "moderated", G_TYPE_BOOLEAN, boolean, FALSE);
-          else if (0 == strcmp (var, "muc_nonanonymous"))
-            INSERT_KEY (keys, "anonymous", G_TYPE_BOOLEAN, boolean, FALSE);
-          else if (0 == strcmp (var, "muc_anonymous"))
-            INSERT_KEY (keys, "anonymous", G_TYPE_BOOLEAN, boolean, TRUE);
-          else if (0 == strcmp (var, "muc_semianonymous"))
-            INSERT_KEY (keys, "anonymous", G_TYPE_BOOLEAN, boolean, FALSE);
-          else if (0 == strcmp (var, "muc_persistent"))
-            INSERT_KEY (keys, "persistent", G_TYPE_BOOLEAN, boolean, TRUE);
-          else if (0 == strcmp (var, "muc_temporary"))
-            INSERT_KEY (keys, "persistent", G_TYPE_BOOLEAN, boolean, FALSE);
-          else
-            HANDLER_DEBUG (feature, "got unknown feature");
-        }
-      else if (0 == strcmp (feature->name, "x"))
-        {
-          namespace = lm_message_node_get_attribute (feature, "xmlns");
-          if (namespace && 0 == strcmp (namespace, "jabber:x:data"))
-            {
-              for (field = feature->children;
-                   field; field = field->next)
-                {
-                  if (0 != strcmp (field->name, "field"))
-                    continue;
+  var = g_hash_table_lookup (item->features, "muc#roominfo_description");
+  if (var != NULL)
+    INSERT_KEY (keys, "description", G_TYPE_STRING, string, var);
 
-                  var = lm_message_node_get_attribute (field, "var");
-                  if (NULL == var)
-                    continue;
+  var = g_hash_table_lookup (item->features, "muc#roominfo_occupants");
+  if (var != NULL)
+    INSERT_KEY (keys, "members", G_TYPE_UINT, uint,
+                (guint) g_ascii_strtoull (var, NULL, 10));
 
-                  value_node = lm_message_node_get_child (field, "value");
-                  if (NULL == value_node)
-                    continue;
+  var = g_hash_table_lookup (item->features, "muc#roominfo_lang");
+  if (var != NULL)
+    INSERT_KEY (keys, "language", G_TYPE_STRING, string, var);
 
-                  value = lm_message_node_get_value (value_node);
-                  if (NULL == value)
-                    continue;
+  DEBUG ("emitting new room signal for %s", jid);
 
-                  if (0 == strcmp (var, "muc#roominfo_description"))
-                    {
-                      INSERT_KEY (keys, "description",
-                                  G_TYPE_STRING, string, value);
-                    }
-                  else if (0 == strcmp (var, "muc#roominfo_occupants"))
-                    {
-                      INSERT_KEY (keys, "members", G_TYPE_UINT, uint,
-                        (guint) g_ascii_strtoull (value, NULL, 10));
-                    }
-                  else if (0 == strcmp (var, "muc#roominfo_lang"))
-                    {
-                      INSERT_KEY (keys, "language", G_TYPE_STRING,
-                                  string, value);
-                    }
-                }
-            }
-        }
-    }
+  handle = gabble_handle_for_room (priv->conn->handles, jid);
 
-  if (is_muc)
-    {
-      g_debug ("%s: emitting new room signal for %s", G_STRFUNC, jid);
+  handle_set_add (priv->signalled_rooms, handle);
 
-      handle = gabble_handle_for_room (priv->conn->handles, jid);
+  g_value_init (&room, TP_TYPE_ROOM_STRUCT);
+  g_value_take_boxed (&room,
+      dbus_g_type_specialized_construct (TP_TYPE_ROOM_STRUCT));
 
-      g_value_init (&room, TP_TYPE_ROOM_STRUCT);
-      g_value_take_boxed (&room,
-          dbus_g_type_specialized_construct (TP_TYPE_ROOM_STRUCT));
+  dbus_g_type_struct_set (&room,
+      0, handle,
+      1, "org.freedesktop.Telepathy.Channel.Type.Text",
+      2, keys,
+      G_MAXUINT);
 
-      dbus_g_type_struct_set (&room,
-          0, handle,
-          1, "org.freedesktop.Telepathy.Channel.Type.Text",
-          2, keys,
-          G_MAXUINT);
-
-      rooms = g_ptr_array_sized_new (1);
-      g_ptr_array_add (rooms, g_value_get_boxed (&room));
-      g_signal_emit (chan, signals[GOT_ROOMS], 0, rooms);
-      g_ptr_array_free (rooms, TRUE);
-      g_value_unset (&room);
-    }
+  rooms = g_ptr_array_sized_new (1);
+  g_ptr_array_add (rooms, g_value_get_boxed (&room));
+  g_signal_emit (chan, signals[GOT_ROOMS], 0, rooms);
+  g_ptr_array_free (rooms, TRUE);
+  g_value_unset (&room);
 
   g_hash_table_destroy (keys);
-
-done:
-  room_list_fill_disco_pipeline (chan);
-
-  return;
 }
 
 static void
-rooms_cb (GabbleDisco *disco,
-          GabbleDiscoRequest *request,
-          const gchar *jid,
-          const gchar *node,
-          LmMessageNode *result,
-          GError *error,
-          gpointer user_data)
+rooms_end_cb (gpointer data, gpointer user_data)
 {
-  LmMessageNode *iter;
   GabbleRoomlistChannel *chan = user_data;
-  GabbleRoomlistChannelPrivate *priv;
-  const char *item_jid;
-  gpointer key, value;
+  GabbleRoomlistChannelPrivate *priv = GABBLE_ROOMLIST_CHANNEL_GET_PRIVATE (chan);
 
-  g_assert (GABBLE_IS_ROOMLIST_CHANNEL (chan));
-
-  priv = GABBLE_ROOMLIST_CHANNEL_GET_PRIVATE (chan);
-
-  if (error)
-    {
-      g_debug ("%s: got error %s", G_STRFUNC, error->message);
-      goto out;
-    }
-
-  iter = result->children;
-
-  for (; iter; iter = iter->next)
-    {
-      if (0 != strcmp (iter->name, "item"))
-        continue;
-
-      item_jid = lm_message_node_get_attribute (iter, "jid");
-
-      if (NULL != item_jid &&
-          !g_hash_table_lookup_extended (priv->remaining_rooms, item_jid, &key, &value))
-        {
-          gchar *tmp = g_strdup (item_jid);
-          g_hash_table_insert (priv->remaining_rooms, tmp, tmp);
-        }
-    }
-
-out:
-  room_list_fill_disco_pipeline (chan);
+  priv->listing = FALSE;
+  g_signal_emit (chan, signals[LISTING_ROOMS], 0, FALSE);
 }
 
-/************************* DBUS Method definitions **************************/
+
+/************************* D-Bus Method definitions **************************/
 
 /**
  * gabble_roomlist_channel_close
  *
- * Implements DBus method Close
+ * Implements D-Bus method Close
  * on interface org.freedesktop.Telepathy.Channel
  *
  * @error: Used to return a pointer to a GError detailing any error
- *         that occured, DBus will throw the error only if this
- *         function returns false.
+ *         that occurred, D-Bus will throw the error only if this
+ *         function returns FALSE.
  *
  * Returns: TRUE if successful, FALSE if an error was thrown.
  */
-gboolean gabble_roomlist_channel_close (GabbleRoomlistChannel *obj, GError **error)
+gboolean
+gabble_roomlist_channel_close (GabbleRoomlistChannel *self,
+                               GError **error)
 {
-  g_assert (GABBLE_IS_ROOMLIST_CHANNEL (obj));
+  g_assert (GABBLE_IS_ROOMLIST_CHANNEL (self));
 
-  g_debug ("%s called on %p", G_STRFUNC, obj);
+  DEBUG ("called on %p", self);
 
-  g_object_run_dispose (G_OBJECT (obj));
+  g_object_run_dispose (G_OBJECT (self));
 
   return TRUE;
 }
@@ -668,16 +517,19 @@ gboolean gabble_roomlist_channel_close (GabbleRoomlistChannel *obj, GError **err
 /**
  * gabble_roomlist_channel_get_channel_type
  *
- * Implements DBus method GetChannelType
+ * Implements D-Bus method GetChannelType
  * on interface org.freedesktop.Telepathy.Channel
  *
  * @error: Used to return a pointer to a GError detailing any error
- *         that occured, DBus will throw the error only if this
- *         function returns false.
+ *         that occurred, D-Bus will throw the error only if this
+ *         function returns FALSE.
  *
  * Returns: TRUE if successful, FALSE if an error was thrown.
  */
-gboolean gabble_roomlist_channel_get_channel_type (GabbleRoomlistChannel *obj, gchar ** ret, GError **error)
+gboolean
+gabble_roomlist_channel_get_channel_type (GabbleRoomlistChannel *self,
+                                          gchar **ret,
+                                          GError **error)
 {
   *ret = g_strdup (TP_IFACE_CHANNEL_TYPE_ROOM_LIST);
   return TRUE;
@@ -687,18 +539,22 @@ gboolean gabble_roomlist_channel_get_channel_type (GabbleRoomlistChannel *obj, g
 /**
  * gabble_roomlist_channel_get_handle
  *
- * Implements DBus method GetHandle
+ * Implements D-Bus method GetHandle
  * on interface org.freedesktop.Telepathy.Channel
  *
  * @error: Used to return a pointer to a GError detailing any error
- *         that occured, DBus will throw the error only if this
- *         function returns false.
+ *         that occurred, D-Bus will throw the error only if this
+ *         function returns FALSE.
  *
  * Returns: TRUE if successful, FALSE if an error was thrown.
  */
-gboolean gabble_roomlist_channel_get_handle (GabbleRoomlistChannel *obj, guint* ret, guint* ret1, GError **error)
+gboolean
+gabble_roomlist_channel_get_handle (GabbleRoomlistChannel *self,
+                                    guint *ret,
+                                    guint *ret1,
+                                    GError **error)
 {
-  g_assert (GABBLE_IS_ROOMLIST_CHANNEL (obj));
+  g_assert (GABBLE_IS_ROOMLIST_CHANNEL (self));
 
   *ret = 0;
   *ret1 = 0;
@@ -710,16 +566,19 @@ gboolean gabble_roomlist_channel_get_handle (GabbleRoomlistChannel *obj, guint*
 /**
  * gabble_roomlist_channel_get_interfaces
  *
- * Implements DBus method GetInterfaces
+ * Implements D-Bus method GetInterfaces
  * on interface org.freedesktop.Telepathy.Channel
  *
  * @error: Used to return a pointer to a GError detailing any error
- *         that occured, DBus will throw the error only if this
- *         function returns false.
+ *         that occurred, D-Bus will throw the error only if this
+ *         function returns FALSE.
  *
  * Returns: TRUE if successful, FALSE if an error was thrown.
  */
-gboolean gabble_roomlist_channel_get_interfaces (GabbleRoomlistChannel *obj, gchar *** ret, GError **error)
+gboolean
+gabble_roomlist_channel_get_interfaces (GabbleRoomlistChannel *self,
+                                        gchar ***ret,
+                                        GError **error)
 {
   const char *interfaces[] = { NULL };
 
@@ -732,22 +591,25 @@ gboolean gabble_roomlist_channel_get_interfaces (GabbleRoomlistChannel *obj, gch
 /**
  * gabble_roomlist_channel_get_listing_rooms
  *
- * Implements DBus method GetListingRooms
+ * Implements D-Bus method GetListingRooms
  * on interface org.freedesktop.Telepathy.Channel.Type.RoomList
  *
  * @error: Used to return a pointer to a GError detailing any error
- *         that occured, DBus will throw the error only if this
- *         function returns false.
+ *         that occurred, D-Bus will throw the error only if this
+ *         function returns FALSE.
  *
  * Returns: TRUE if successful, FALSE if an error was thrown.
  */
-gboolean gabble_roomlist_channel_get_listing_rooms (GabbleRoomlistChannel *obj, gboolean* ret, GError **error)
+gboolean
+gabble_roomlist_channel_get_listing_rooms (GabbleRoomlistChannel *self,
+                                           gboolean *ret,
+                                           GError **error)
 {
   GabbleRoomlistChannelPrivate *priv;
 
-  g_assert (GABBLE_IS_ROOMLIST_CHANNEL (obj));
+  g_assert (GABBLE_IS_ROOMLIST_CHANNEL (self));
 
-  priv = GABBLE_ROOMLIST_CHANNEL_GET_PRIVATE (obj);
+  priv = GABBLE_ROOMLIST_CHANNEL_GET_PRIVATE (self);
   *ret = priv->listing;
   return TRUE;
 }
@@ -756,28 +618,34 @@ gboolean gabble_roomlist_channel_get_listing_rooms (GabbleRoomlistChannel *obj,
 /**
  * gabble_roomlist_channel_list_rooms
  *
- * Implements DBus method ListRooms
+ * Implements D-Bus method ListRooms
  * on interface org.freedesktop.Telepathy.Channel.Type.RoomList
  *
  * @error: Used to return a pointer to a GError detailing any error
- *         that occured, DBus will throw the error only if this
- *         function returns false.
+ *         that occurred, D-Bus will throw the error only if this
+ *         function returns FALSE.
  *
  * Returns: TRUE if successful, FALSE if an error was thrown.
  */
-gboolean gabble_roomlist_channel_list_rooms (GabbleRoomlistChannel *obj, GError **error)
+gboolean
+gabble_roomlist_channel_list_rooms (GabbleRoomlistChannel *self,
+                                    GError **error)
 {
   GabbleRoomlistChannelPrivate *priv;
 
-  g_assert (GABBLE_IS_ROOMLIST_CHANNEL (obj));
+  g_assert (GABBLE_IS_ROOMLIST_CHANNEL (self));
 
-  priv = GABBLE_ROOMLIST_CHANNEL_GET_PRIVATE (obj);
+  priv = GABBLE_ROOMLIST_CHANNEL_GET_PRIVATE (self);
 
   priv->listing = TRUE;
-  g_signal_emit (obj, signals[LISTING_ROOMS], 0, TRUE);
-  gabble_disco_request (priv->conn->disco, GABBLE_DISCO_TYPE_ITEMS,
-                        priv->conference_server, NULL,
-                        rooms_cb, obj, G_OBJECT(obj), NULL);
+  g_signal_emit (self, signals[LISTING_ROOMS], 0, TRUE);
+
+  if (priv->disco_pipeline == NULL)
+    priv->disco_pipeline = gabble_disco_pipeline_init (priv->conn->disco,
+        room_info_cb, rooms_end_cb, self);
+
+  gabble_disco_pipeline_run (priv->disco_pipeline, priv->conference_server);
+
   return TRUE;
 }
 
