@@ -22,6 +22,9 @@
 #include <stdio.h>
 #include <stdlib.h>
 
+#define DEBUG_FLAG GABBLE_DEBUG_ROSTER
+
+#include "debug.h"
 #include "gabble-connection.h"
 #include "gintset.h"
 #include "group-mixin.h"
@@ -74,12 +77,16 @@ struct _GabbleRosterChannelPrivate
   gboolean dispose_has_run;
 };
 
-#define GABBLE_ROSTER_CHANNEL_GET_PRIVATE(o)     (G_TYPE_INSTANCE_GET_PRIVATE ((o), GABBLE_TYPE_ROSTER_CHANNEL, GabbleRosterChannelPrivate))
+#define GABBLE_ROSTER_CHANNEL_GET_PRIVATE(obj) \
+    ((GabbleRosterChannelPrivate *)obj->priv)
 
 static void
-gabble_roster_channel_init (GabbleRosterChannel *obj)
+gabble_roster_channel_init (GabbleRosterChannel *self)
 {
-  /* GabbleRosterChannelPrivate *priv = GABBLE_ROSTER_CHANNEL_GET_PRIVATE (obj); */
+  GabbleRosterChannelPrivate *priv = G_TYPE_INSTANCE_GET_PRIVATE (self,
+      GABBLE_TYPE_ROSTER_CHANNEL, GabbleRosterChannelPrivate);
+
+  self->priv = priv;
 
   /* allocate any data required by the object here */
 }
@@ -113,7 +120,7 @@ gabble_roster_channel_constructor (GType type, guint n_props,
   gabble_group_mixin_init (obj, G_STRUCT_OFFSET (GabbleRosterChannel, group),
                            handles, self_handle);
 
-  if (gabble_handle_for_list_publish (handles) == priv->handle)
+  if (GABBLE_LIST_HANDLE_PUBLISH == priv->handle)
     {
       gabble_group_mixin_change_flags (obj,
           TP_CHANNEL_GROUP_FLAG_CAN_REMOVE |
@@ -121,7 +128,7 @@ gabble_roster_channel_constructor (GType type, guint n_props,
           TP_CHANNEL_GROUP_FLAG_MESSAGE_REMOVE,
           0);
     }
-  else if (gabble_handle_for_list_subscribe (handles) == priv->handle)
+  else if (GABBLE_LIST_HANDLE_SUBSCRIBE == priv->handle)
     {
       gabble_group_mixin_change_flags (obj,
           TP_CHANNEL_GROUP_FLAG_CAN_ADD |
@@ -132,12 +139,19 @@ gabble_roster_channel_constructor (GType type, guint n_props,
           TP_CHANNEL_GROUP_FLAG_MESSAGE_RESCIND,
           0);
     }
-  else if (gabble_handle_for_list_known (handles) == priv->handle)
+  else if (GABBLE_LIST_HANDLE_KNOWN == priv->handle)
     {
       gabble_group_mixin_change_flags (obj,
           TP_CHANNEL_GROUP_FLAG_CAN_REMOVE,
           0);
     }
+  else if (GABBLE_LIST_HANDLE_DENY == priv->handle)
+    {
+      gabble_group_mixin_change_flags (obj,
+          TP_CHANNEL_GROUP_FLAG_CAN_ADD |
+          TP_CHANNEL_GROUP_FLAG_CAN_REMOVE,
+          0);
+    }
   else
     {
       g_assert_not_reached ();
@@ -246,7 +260,7 @@ gabble_roster_channel_class_init (GabbleRosterChannelClass *gabble_roster_channe
                   G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
                   0,
                   NULL, NULL,
-                  gabble_roster_channel_marshal_VOID__VOID,
+                  g_cclosure_marshal_VOID__VOID,
                   G_TYPE_NONE, 0);
 
   gabble_group_mixin_class_init (object_class,
@@ -344,29 +358,34 @@ _gabble_roster_channel_add_member_cb (GObject *obj,
   GabbleHandleRepo *repo;
   gboolean ret = FALSE;
 
-  g_assert (GABBLE_IS_ROSTER_CHANNEL (obj));
+  priv = GABBLE_ROSTER_CHANNEL_GET_PRIVATE (GABBLE_ROSTER_CHANNEL (obj));
 
-  priv = GABBLE_ROSTER_CHANNEL_GET_PRIVATE (obj);
   repo = priv->conn->handles;
 
-  g_debug ("%s: called on %s with handle %u (%s) \"%s\"", G_STRFUNC,
-      gabble_handle_inspect (repo, TP_HANDLE_TYPE_LIST, priv->handle), handle,
+  DEBUG ("called on %s with handle %u (%s) \"%s\"", gabble_handle_inspect (repo, TP_HANDLE_TYPE_LIST, priv->handle), handle,
       gabble_handle_inspect (repo, TP_HANDLE_TYPE_CONTACT, handle), message);
 
   /* publish list */
-  if (gabble_handle_for_list_publish (repo) == priv->handle)
+  if (GABBLE_LIST_HANDLE_PUBLISH == priv->handle)
     {
       /* send <presence type="subscribed"> */
       ret = _gabble_roster_channel_send_presence (GABBLE_ROSTER_CHANNEL (obj),
           LM_MESSAGE_SUB_TYPE_SUBSCRIBED, handle, message, error);
     }
   /* subscribe list */
-  else if (gabble_handle_for_list_subscribe (repo) == priv->handle)
+  else if (GABBLE_LIST_HANDLE_SUBSCRIBE == priv->handle)
     {
       /* send <presence type="subscribe"> */
       ret = _gabble_roster_channel_send_presence (GABBLE_ROSTER_CHANNEL (obj),
           LM_MESSAGE_SUB_TYPE_SUBSCRIBE, handle, message, error);
     }
+  /* deny list */
+  else if (GABBLE_LIST_HANDLE_DENY == priv->handle)
+    {
+      /* block contact */
+      ret = gabble_roster_handle_set_blocked (priv->conn->roster, handle, TRUE,
+          error);
+    }
   else
     {
       g_assert_not_reached ();
@@ -391,34 +410,55 @@ _gabble_roster_channel_remove_member_cb (GObject *obj,
   GabbleHandleRepo *repo;
   gboolean ret = FALSE;
 
-  g_assert (GABBLE_IS_ROSTER_CHANNEL (obj));
+  priv = GABBLE_ROSTER_CHANNEL_GET_PRIVATE (GABBLE_ROSTER_CHANNEL (obj));
 
-  priv = GABBLE_ROSTER_CHANNEL_GET_PRIVATE (obj);
   repo = priv->conn->handles;
 
-  g_debug ("%s: called on %s with handle %u (%s) \"%s\"", G_STRFUNC,
-      gabble_handle_inspect (repo, TP_HANDLE_TYPE_LIST, priv->handle), handle,
+  DEBUG ("called on %s with handle %u (%s) \"%s\"", gabble_handle_inspect (repo, TP_HANDLE_TYPE_LIST, priv->handle), handle,
       gabble_handle_inspect (repo, TP_HANDLE_TYPE_CONTACT, handle), message);
 
   /* publish list */
-  if (gabble_handle_for_list_publish (repo) == priv->handle)
+  if (GABBLE_LIST_HANDLE_PUBLISH == priv->handle)
     {
       /* send <presence type="unsubscribed"> */
       ret = _gabble_roster_channel_send_presence (GABBLE_ROSTER_CHANNEL (obj),
           LM_MESSAGE_SUB_TYPE_UNSUBSCRIBED, handle, message, error);
+
+      /* remove it from local_pending here, because roster callback doesn't
+         know if it can (subscription='none' is used both during request and
+         when it's rejected) */
+      if (handle_set_is_member (GABBLE_ROSTER_CHANNEL (obj)->group.local_pending, handle))
+        {
+          GIntSet *rem = g_intset_new ();
+          GIntSet *empty = g_intset_new();
+
+          g_intset_add (rem, handle);
+          gabble_group_mixin_change_members (obj, "", empty, rem, empty, empty, 0, 0);
+
+          g_intset_destroy (empty);
+          g_intset_destroy (rem);
+        }
     }
   /* subscribe list */
-  else if (gabble_handle_for_list_subscribe (repo) == priv->handle)
+  else if (GABBLE_LIST_HANDLE_SUBSCRIBE == priv->handle)
     {
       /* send <presence type="unsubscribe"> */
       ret = _gabble_roster_channel_send_presence (GABBLE_ROSTER_CHANNEL (obj),
           LM_MESSAGE_SUB_TYPE_UNSUBSCRIBE, handle, message, error);
     }
-  else if (gabble_handle_for_list_known (repo) == priv->handle)
+  /* known list */
+  else if (GABBLE_LIST_HANDLE_KNOWN == priv->handle)
     {
       /* send roster subscription=remove IQ */
       ret = gabble_roster_handle_remove (priv->conn->roster, handle, error);
     }
+  /* deny list */
+  else if (GABBLE_LIST_HANDLE_DENY == priv->handle)
+    {
+      /* unblock contact */
+      ret = gabble_roster_handle_set_blocked (priv->conn->roster, handle, FALSE,
+          error);
+    }
   else
     {
       g_assert_not_reached ();
@@ -431,37 +471,44 @@ _gabble_roster_channel_remove_member_cb (GObject *obj,
 /**
  * gabble_roster_channel_add_members
  *
- * Implements DBus method AddMembers
+ * Implements D-Bus method AddMembers
  * on interface org.freedesktop.Telepathy.Channel.Interface.Group
  *
  * @error: Used to return a pointer to a GError detailing any error
- *         that occured, DBus will throw the error only if this
- *         function returns false.
+ *         that occurred, D-Bus will throw the error only if this
+ *         function returns FALSE.
  *
  * Returns: TRUE if successful, FALSE if an error was thrown.
  */
-gboolean gabble_roster_channel_add_members (GabbleRosterChannel *obj, const GArray * contacts, const gchar * message, GError **error)
+gboolean
+gabble_roster_channel_add_members (GabbleRosterChannel *self,
+                                   const GArray *contacts,
+                                   const gchar *message,
+                                   GError **error)
 {
-  return gabble_group_mixin_add_members (G_OBJECT (obj), contacts, message, error);
+  return gabble_group_mixin_add_members (G_OBJECT (self), contacts, message,
+      error);
 }
 
 
 /**
  * gabble_roster_channel_close
  *
- * Implements DBus method Close
+ * Implements D-Bus method Close
  * on interface org.freedesktop.Telepathy.Channel
  *
  * @error: Used to return a pointer to a GError detailing any error
- *         that occured, DBus will throw the error only if this
- *         function returns false.
+ *         that occurred, D-Bus will throw the error only if this
+ *         function returns FALSE.
  *
  * Returns: TRUE if successful, FALSE if an error was thrown.
  */
-gboolean gabble_roster_channel_close (GabbleRosterChannel *obj, GError **error)
+gboolean
+gabble_roster_channel_close (GabbleRosterChannel *self,
+                             GError **error)
 {
-  *error = g_error_new (TELEPATHY_ERRORS, NotImplemented,
-                        "you may not close contact list channels");
+  g_set_error (error, TELEPATHY_ERRORS, NotImplemented,
+      "you may not close contact list channels");
 
   return FALSE;
 }
@@ -470,34 +517,43 @@ gboolean gabble_roster_channel_close (GabbleRosterChannel *obj, GError **error)
 /**
  * gabble_roster_channel_get_all_members
  *
- * Implements DBus method GetAllMembers
+ * Implements D-Bus method GetAllMembers
  * on interface org.freedesktop.Telepathy.Channel.Interface.Group
  *
  * @error: Used to return a pointer to a GError detailing any error
- *         that occured, DBus will throw the error only if this
- *         function returns false.
+ *         that occurred, D-Bus will throw the error only if this
+ *         function returns FALSE.
  *
  * Returns: TRUE if successful, FALSE if an error was thrown.
  */
-gboolean gabble_roster_channel_get_all_members (GabbleRosterChannel *obj, GArray ** ret, GArray ** ret1, GArray ** ret2, GError **error)
+gboolean
+gabble_roster_channel_get_all_members (GabbleRosterChannel *self,
+                                       GArray **ret,
+                                       GArray **ret1,
+                                       GArray **ret2,
+                                       GError **error)
 {
-  return gabble_group_mixin_get_all_members (G_OBJECT (obj), ret, ret1, ret2, error);
+  return gabble_group_mixin_get_all_members (G_OBJECT (self), ret, ret1, ret2,
+      error);
 }
 
 
 /**
  * gabble_roster_channel_get_channel_type
  *
- * Implements DBus method GetChannelType
+ * Implements D-Bus method GetChannelType
  * on interface org.freedesktop.Telepathy.Channel
  *
  * @error: Used to return a pointer to a GError detailing any error
- *         that occured, DBus will throw the error only if this
- *         function returns false.
+ *         that occurred, D-Bus will throw the error only if this
+ *         function returns FALSE.
  *
  * Returns: TRUE if successful, FALSE if an error was thrown.
  */
-gboolean gabble_roster_channel_get_channel_type (GabbleRosterChannel *obj, gchar ** ret, GError **error)
+gboolean
+gabble_roster_channel_get_channel_type (GabbleRosterChannel *self,
+                                        gchar **ret,
+                                        GError **error)
 {
   *ret = g_strdup (TP_IFACE_CHANNEL_TYPE_CONTACT_LIST);
 
@@ -508,40 +564,47 @@ gboolean gabble_roster_channel_get_channel_type (GabbleRosterChannel *obj, gchar
 /**
  * gabble_roster_channel_get_group_flags
  *
- * Implements DBus method GetGroupFlags
+ * Implements D-Bus method GetGroupFlags
  * on interface org.freedesktop.Telepathy.Channel.Interface.Group
  *
  * @error: Used to return a pointer to a GError detailing any error
- *         that occured, DBus will throw the error only if this
- *         function returns false.
+ *         that occurred, D-Bus will throw the error only if this
+ *         function returns FALSE.
  *
  * Returns: TRUE if successful, FALSE if an error was thrown.
  */
-gboolean gabble_roster_channel_get_group_flags (GabbleRosterChannel *obj, guint* ret, GError **error)
+gboolean
+gabble_roster_channel_get_group_flags (GabbleRosterChannel *self,
+                                       guint *ret,
+                                       GError **error)
 {
-  return gabble_group_mixin_get_group_flags (G_OBJECT (obj), ret, error);
+  return gabble_group_mixin_get_group_flags (G_OBJECT (self), ret, error);
 }
 
 
 /**
  * gabble_roster_channel_get_handle
  *
- * Implements DBus method GetHandle
+ * Implements D-Bus method GetHandle
  * on interface org.freedesktop.Telepathy.Channel
  *
  * @error: Used to return a pointer to a GError detailing any error
- *         that occured, DBus will throw the error only if this
- *         function returns false.
+ *         that occurred, D-Bus will throw the error only if this
+ *         function returns FALSE.
  *
  * Returns: TRUE if successful, FALSE if an error was thrown.
  */
-gboolean gabble_roster_channel_get_handle (GabbleRosterChannel *obj, guint* ret, guint* ret1, GError **error)
+gboolean
+gabble_roster_channel_get_handle (GabbleRosterChannel *self,
+                                  guint *ret,
+                                  guint *ret1,
+                                  GError **error)
 {
   GabbleRosterChannelPrivate *priv;
 
-  g_assert (GABBLE_IS_ROSTER_CHANNEL (obj));
+  g_assert (GABBLE_IS_ROSTER_CHANNEL (self));
 
-  priv = GABBLE_ROSTER_CHANNEL_GET_PRIVATE (obj);
+  priv = GABBLE_ROSTER_CHANNEL_GET_PRIVATE (self);
 
   *ret = TP_HANDLE_TYPE_LIST;
   *ret1 = priv->handle;
@@ -553,34 +616,42 @@ gboolean gabble_roster_channel_get_handle (GabbleRosterChannel *obj, guint* ret,
 /**
  * gabble_roster_channel_get_handle_owners
  *
- * Implements DBus method GetHandleOwners
+ * Implements D-Bus method GetHandleOwners
  * on interface org.freedesktop.Telepathy.Channel.Interface.Group
  *
  * @error: Used to return a pointer to a GError detailing any error
- *         that occured, DBus will throw the error only if this
- *         function returns false.
+ *         that occurred, D-Bus will throw the error only if this
+ *         function returns FALSE.
  *
  * Returns: TRUE if successful, FALSE if an error was thrown.
  */
-gboolean gabble_roster_channel_get_handle_owners (GabbleRosterChannel *obj, const GArray * handles, GArray ** ret, GError **error)
+gboolean
+gabble_roster_channel_get_handle_owners (GabbleRosterChannel *self,
+                                         const GArray *handles,
+                                         GArray **ret,
+                                         GError **error)
 {
-  return gabble_group_mixin_get_handle_owners (G_OBJECT (obj), handles, ret, error);
+  return gabble_group_mixin_get_handle_owners (G_OBJECT (self), handles, ret,
+      error);
 }
 
 
 /**
  * gabble_roster_channel_get_interfaces
  *
- * Implements DBus method GetInterfaces
+ * Implements D-Bus method GetInterfaces
  * on interface org.freedesktop.Telepathy.Channel
  *
  * @error: Used to return a pointer to a GError detailing any error
- *         that occured, DBus will throw the error only if this
- *         function returns false.
+ *         that occurred, D-Bus will throw the error only if this
+ *         function returns FALSE.
  *
  * Returns: TRUE if successful, FALSE if an error was thrown.
  */
-gboolean gabble_roster_channel_get_interfaces (GabbleRosterChannel *obj, gchar *** ret, GError **error)
+gboolean
+gabble_roster_channel_get_interfaces (GabbleRosterChannel *self,
+                                      gchar ***ret,
+                                      GError **error)
 {
   const char *interfaces[] = { TP_IFACE_CHANNEL_INTERFACE_GROUP, NULL };
 
@@ -593,89 +664,108 @@ gboolean gabble_roster_channel_get_interfaces (GabbleRosterChannel *obj, gchar *
 /**
  * gabble_roster_channel_get_local_pending_members
  *
- * Implements DBus method GetLocalPendingMembers
+ * Implements D-Bus method GetLocalPendingMembers
  * on interface org.freedesktop.Telepathy.Channel.Interface.Group
  *
  * @error: Used to return a pointer to a GError detailing any error
- *         that occured, DBus will throw the error only if this
- *         function returns false.
+ *         that occurred, D-Bus will throw the error only if this
+ *         function returns FALSE.
  *
  * Returns: TRUE if successful, FALSE if an error was thrown.
  */
-gboolean gabble_roster_channel_get_local_pending_members (GabbleRosterChannel *obj, GArray ** ret, GError **error)
+gboolean
+gabble_roster_channel_get_local_pending_members (GabbleRosterChannel *self,
+                                                 GArray **ret,
+                                                 GError **error)
 {
-  return gabble_group_mixin_get_local_pending_members (G_OBJECT (obj), ret, error);
+  return gabble_group_mixin_get_local_pending_members (G_OBJECT (self), ret,
+      error);
 }
 
 
 /**
  * gabble_roster_channel_get_members
  *
- * Implements DBus method GetMembers
+ * Implements D-Bus method GetMembers
  * on interface org.freedesktop.Telepathy.Channel.Interface.Group
  *
  * @error: Used to return a pointer to a GError detailing any error
- *         that occured, DBus will throw the error only if this
- *         function returns false.
+ *         that occurred, D-Bus will throw the error only if this
+ *         function returns FALSE.
  *
  * Returns: TRUE if successful, FALSE if an error was thrown.
  */
-gboolean gabble_roster_channel_get_members (GabbleRosterChannel *obj, GArray ** ret, GError **error)
+gboolean
+gabble_roster_channel_get_members (GabbleRosterChannel *self,
+                                   GArray **ret,
+                                   GError **error)
 {
-  return gabble_group_mixin_get_members (G_OBJECT (obj), ret, error);
+  return gabble_group_mixin_get_members (G_OBJECT (self), ret, error);
 }
 
 
 /**
  * gabble_roster_channel_get_remote_pending_members
  *
- * Implements DBus method GetRemotePendingMembers
+ * Implements D-Bus method GetRemotePendingMembers
  * on interface org.freedesktop.Telepathy.Channel.Interface.Group
  *
  * @error: Used to return a pointer to a GError detailing any error
- *         that occured, DBus will throw the error only if this
- *         function returns false.
+ *         that occurred, D-Bus will throw the error only if this
+ *         function returns FALSE.
  *
  * Returns: TRUE if successful, FALSE if an error was thrown.
  */
-gboolean gabble_roster_channel_get_remote_pending_members (GabbleRosterChannel *obj, GArray ** ret, GError **error)
+gboolean
+gabble_roster_channel_get_remote_pending_members (GabbleRosterChannel *self,
+                                                  GArray **ret,
+                                                  GError **error)
 {
-  return gabble_group_mixin_get_remote_pending_members (G_OBJECT (obj), ret, error);
+  return gabble_group_mixin_get_remote_pending_members (G_OBJECT (self), ret,
+      error);
 }
 
 
 /**
  * gabble_roster_channel_get_self_handle
  *
- * Implements DBus method GetSelfHandle
+ * Implements D-Bus method GetSelfHandle
  * on interface org.freedesktop.Telepathy.Channel.Interface.Group
  *
  * @error: Used to return a pointer to a GError detailing any error
- *         that occured, DBus will throw the error only if this
- *         function returns false.
+ *         that occurred, D-Bus will throw the error only if this
+ *         function returns FALSE.
  *
  * Returns: TRUE if successful, FALSE if an error was thrown.
  */
-gboolean gabble_roster_channel_get_self_handle (GabbleRosterChannel *obj, guint* ret, GError **error)
+gboolean
+gabble_roster_channel_get_self_handle (GabbleRosterChannel *self,
+                                       guint *ret,
+                                       GError **error)
 {
-  return gabble_group_mixin_get_self_handle (G_OBJECT (obj), ret, error);
+  return gabble_group_mixin_get_self_handle (G_OBJECT (self), ret, error);
 }
 
 
 /**
  * gabble_roster_channel_remove_members
  *
- * Implements DBus method RemoveMembers
+ * Implements D-Bus method RemoveMembers
  * on interface org.freedesktop.Telepathy.Channel.Interface.Group
  *
  * @error: Used to return a pointer to a GError detailing any error
- *         that occured, DBus will throw the error only if this
- *         function returns false.
+ *         that occurred, D-Bus will throw the error only if this
+ *         function returns FALSE.
  *
  * Returns: TRUE if successful, FALSE if an error was thrown.
  */
-gboolean gabble_roster_channel_remove_members (GabbleRosterChannel *obj, const GArray * contacts, const gchar * message, GError **error)
+gboolean
+gabble_roster_channel_remove_members (GabbleRosterChannel *self,
+                                      const GArray *contacts,
+                                      const gchar *message,
+                                      GError **error)
 {
-  return gabble_group_mixin_remove_members (G_OBJECT (obj), contacts, message, error);
+  return gabble_group_mixin_remove_members (G_OBJECT (self), contacts, message,
+      error);
 }
 
