@@ -30,7 +30,13 @@ typedef struct _GabbleHandleRepo GabbleHandleRepo;
 typedef struct _GabblePresenceCache GabblePresenceCache;
 typedef struct _GabbleRoster GabbleRoster;
 typedef struct _GabbleRosterChannel GabbleRosterChannel;
-typedef GQuark GabbleHandle;
+typedef struct _GabbleVCardManager GabbleVCardManager;
+typedef guint32 GabbleHandle;
+
+typedef enum {
+    INITIATOR_LOCAL,
+    INITIATOR_REMOTE,
+} JingleInitiator;
 
 G_END_DECLS
 
