@@ -27,10 +27,35 @@
 
 #include "telepathy-errors.h"
 
+#define DEBUG_FLAG GABBLE_DEBUG_GROUPS
+
+#include "debug.h"
 #include "group-mixin.h"
 #include "group-mixin-signals-marshal.h"
 
+static const char *group_change_reason_str(guint reason)
+{
+  switch (reason)
+    {
+    case TP_CHANNEL_GROUP_CHANGE_REASON_NONE:
+      return "unspecified reason";
+    case TP_CHANNEL_GROUP_CHANGE_REASON_OFFLINE:
+      return "offline";
+    case TP_CHANNEL_GROUP_CHANGE_REASON_KICKED:
+      return "kicked";
+    case TP_CHANNEL_GROUP_CHANGE_REASON_BUSY:
+      return "busy";
+    case TP_CHANNEL_GROUP_CHANGE_REASON_INVITED:
+      return "invited";
+    case TP_CHANNEL_GROUP_CHANGE_REASON_BANNED:
+      return "banned";
+    default:
+      return "(unknown reason code)";
+    }
+}
+
 struct _GabbleGroupMixinPrivate {
+    GabbleHandleSet *actors;
     GHashTable *handle_owners;
 };
 
@@ -86,7 +111,7 @@ void gabble_group_mixin_class_init (GObjectClass *obj_cls,
                   G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
                   0,
                   NULL, NULL,
-                  group_mixin_marshal_VOID__INT_INT,
+                  group_mixin_marshal_VOID__UINT_UINT,
                   G_TYPE_NONE, 2, G_TYPE_UINT, G_TYPE_UINT);
 
   mixin_cls->members_changed_signal_id =
@@ -95,8 +120,8 @@ void gabble_group_mixin_class_init (GObjectClass *obj_cls,
                   G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
                   0,
                   NULL, NULL,
-                  group_mixin_marshal_VOID__STRING_BOXED_BOXED_BOXED_BOXED,
-                  G_TYPE_NONE, 5, G_TYPE_STRING, DBUS_TYPE_G_UINT_ARRAY, DBUS_TYPE_G_UINT_ARRAY, DBUS_TYPE_G_UINT_ARRAY, DBUS_TYPE_G_UINT_ARRAY);
+                  group_mixin_marshal_VOID__STRING_BOXED_BOXED_BOXED_BOXED_UINT_UINT,
+                  G_TYPE_NONE, 7, G_TYPE_STRING, DBUS_TYPE_G_UINT_ARRAY, DBUS_TYPE_G_UINT_ARRAY, DBUS_TYPE_G_UINT_ARRAY, DBUS_TYPE_G_UINT_ARRAY, G_TYPE_UINT, G_TYPE_UINT);
 }
 
 void gabble_group_mixin_init (GObject *obj,
@@ -125,6 +150,7 @@ void gabble_group_mixin_init (GObject *obj,
 
   mixin->priv = g_new0 (GabbleGroupMixinPrivate, 1);
   mixin->priv->handle_owners = g_hash_table_new (g_direct_hash, g_direct_equal);
+  mixin->priv->actors = handle_set_new (handle_repo, TP_HANDLE_TYPE_CONTACT);
 }
 
 static void
@@ -144,6 +170,8 @@ void gabble_group_mixin_finalize (GObject *obj)
 {
   GabbleGroupMixin *mixin = GABBLE_GROUP_MIXIN (obj);
 
+  handle_set_destroy (mixin->priv->actors);
+
   g_hash_table_foreach (mixin->priv->handle_owners,
                         handle_owners_foreach_unref,
                         mixin);
@@ -210,10 +238,10 @@ gabble_group_mixin_add_members (GObject *obj, const GArray *contacts, const gcha
       if ((mixin->group_flags & TP_CHANNEL_GROUP_FLAG_CAN_ADD) == 0 &&
           !handle_set_is_member (mixin->local_pending, handle))
         {
-          g_debug ("%s: handle %u cannot be added to members without GROUP_FLAG_CAN_ADD",
-              G_STRFUNC, handle);
+          DEBUG ("handle %u cannot be added to members without GROUP_FLAG_CAN_ADD",
+              handle);
 
-          *error = g_error_new (TELEPATHY_ERRORS, PermissionDenied,
+          g_set_error (error, TELEPATHY_ERRORS, PermissionDenied,
               "handle %u cannot be added to members without GROUP_FLAG_CAN_ADD",
               handle);
 
@@ -228,7 +256,7 @@ gabble_group_mixin_add_members (GObject *obj, const GArray *contacts, const gcha
 
       if (handle_set_is_member (mixin->members, handle))
         {
-          g_debug ("%s: handle %u is already a member, skipping", G_STRFUNC, handle);
+          DEBUG ("handle %u is already a member, skipping", handle);
 
           continue;
         }
@@ -267,10 +295,10 @@ gabble_group_mixin_remove_members (GObject *obj, const GArray *contacts, const g
         {
           if ((mixin->group_flags & TP_CHANNEL_GROUP_FLAG_CAN_REMOVE) == 0)
             {
-              g_debug ("%s: handle %u cannot be removed from members without GROUP_FLAG_CAN_REMOVE",
-                  G_STRFUNC, handle);
+              DEBUG ("handle %u cannot be removed from members without GROUP_FLAG_CAN_REMOVE",
+                  handle);
 
-              *error = g_error_new (TELEPATHY_ERRORS, PermissionDenied,
+              g_set_error (error, TELEPATHY_ERRORS, PermissionDenied,
                   "handle %u cannot be removed from members without GROUP_FLAG_CAN_REMOVE",
                   handle);
 
@@ -281,10 +309,10 @@ gabble_group_mixin_remove_members (GObject *obj, const GArray *contacts, const g
         {
           if ((mixin->group_flags & TP_CHANNEL_GROUP_FLAG_CAN_RESCIND) == 0)
             {
-              g_debug ("%s: handle %u cannot be removed from remote pending without GROUP_FLAG_CAN_RESCIND",
-                  G_STRFUNC, handle);
+              DEBUG ("handle %u cannot be removed from remote pending without GROUP_FLAG_CAN_RESCIND",
+                  handle);
 
-              *error = g_error_new (TELEPATHY_ERRORS, PermissionDenied,
+              g_set_error (error, TELEPATHY_ERRORS, PermissionDenied,
                   "handle %u cannot be removed from remote pending without GROUP_FLAG_CAN_RESCIND",
                   handle);
 
@@ -293,10 +321,10 @@ gabble_group_mixin_remove_members (GObject *obj, const GArray *contacts, const g
         }
       else if (!handle_set_is_member (mixin->local_pending, handle))
         {
-          g_debug ("%s: handle %u is not a current or pending member",
-                   G_STRFUNC, handle);
+          DEBUG ("handle %u is not a current or pending member",
+                   handle);
 
-          *error = g_error_new (TELEPATHY_ERRORS, NotAvailable,
+          g_set_error (error, TELEPATHY_ERRORS, NotAvailable,
               "handle %u is not a current or pending member", handle);
 
           return FALSE;
@@ -372,7 +400,7 @@ gabble_group_mixin_get_handle_owners (GObject *obj,
   if ((mixin->group_flags &
         TP_CHANNEL_GROUP_FLAG_CHANNEL_SPECIFIC_HANDLES) == 0)
     {
-      *error = g_error_new (TELEPATHY_ERRORS, NotAvailable,
+      g_set_error (error, TELEPATHY_ERRORS, NotAvailable,
           "channel doesn't have channel specific handles");
 
       return FALSE;
@@ -393,7 +421,7 @@ gabble_group_mixin_get_handle_owners (GObject *obj,
 
       if (!handle_set_is_member (mixin->members, local_handle))
         {
-          *error = g_error_new (TELEPATHY_ERRORS, InvalidArgument,
+          g_set_error (error, TELEPATHY_ERRORS, InvalidArgument,
               "handle %u is not a member", local_handle);
 
           g_array_free (*ret, TRUE);
@@ -467,22 +495,25 @@ gabble_group_mixin_change_flags (GObject *obj,
     {
       gchar *str_added, *str_removed, *str_flags;
 
-      str_added = group_flags_to_string (added);
-      str_removed = group_flags_to_string (removed);
-      str_flags = group_flags_to_string (mixin->group_flags);
+      if (DEBUGGING)
+        {
+          str_added = group_flags_to_string (added);
+          str_removed = group_flags_to_string (removed);
+          str_flags = group_flags_to_string (mixin->group_flags);
 
-      printf (ANSI_BOLD_ON ANSI_FG_WHITE
-              "%s: emitting group flags changed\n"
-              "  added    : %s\n"
-              "  removed  : %s\n"
-              "  flags now: %s\n" ANSI_RESET,
-              G_STRFUNC, str_added, str_removed, str_flags);
+          printf (ANSI_BOLD_ON ANSI_FG_WHITE
+                  "%s: emitting group flags changed\n"
+                  "  added    : %s\n"
+                  "  removed  : %s\n"
+                  "  flags now: %s\n" ANSI_RESET,
+                  G_STRFUNC, str_added, str_removed, str_flags);
 
-      fflush (stdout);
+          fflush (stdout);
 
-      g_free (str_added);
-      g_free (str_removed);
-      g_free (str_flags);
+          g_free (str_added);
+          g_free (str_removed);
+          g_free (str_flags);
+        }
 
       g_signal_emit(obj, mixin_cls->group_flags_changed_signal_id, 0, added, removed);
     }
@@ -528,7 +559,9 @@ gabble_group_mixin_change_members (GObject *obj,
                                    GIntSet *add,
                                    GIntSet *remove,
                                    GIntSet *local_pending,
-                                   GIntSet *remote_pending)
+                                   GIntSet *remote_pending,
+                                   GabbleHandle actor,
+                                   guint reason)
 {
   GabbleGroupMixin *mixin = GABBLE_GROUP_MIXIN (obj);
   GabbleGroupMixinClass *mixin_cls = GABBLE_GROUP_MIXIN_CLASS (G_OBJECT_GET_CLASS (obj));
@@ -610,34 +643,43 @@ gabble_group_mixin_change_members (GObject *obj,
       /* remove any handle owner mappings */
       remove_handle_owners_if_exist (obj, arr_remove);
 
-      /* debug start */
-      add_str = member_array_to_string (mixin->handle_repo, arr_add);
-      rem_str = member_array_to_string (mixin->handle_repo, arr_remove);
-      local_str = member_array_to_string (mixin->handle_repo, arr_local);
-      remote_str = member_array_to_string (mixin->handle_repo, arr_remote);
+      if (DEBUGGING)
+        {
+          add_str = member_array_to_string (mixin->handle_repo, arr_add);
+          rem_str = member_array_to_string (mixin->handle_repo, arr_remove);
+          local_str = member_array_to_string (mixin->handle_repo, arr_local);
+          remote_str = member_array_to_string (mixin->handle_repo, arr_remote);
 
-      printf (ANSI_BOLD_ON ANSI_FG_CYAN
-              "%s: emitting members changed\n"
-              "  message       : \"%s\"\n"
-              "  added         : %s\n"
-              "  removed       : %s\n"
-              "  local_pending : %s\n"
-              "  remote_pending: %s\n" ANSI_RESET,
-              G_STRFUNC, message, add_str, rem_str, local_str, remote_str);
+          printf (ANSI_BOLD_ON ANSI_FG_CYAN
+                  "%s: emitting members changed\n"
+                  "  message       : \"%s\"\n"
+                  "  added         : %s\n"
+                  "  removed       : %s\n"
+                  "  local_pending : %s\n"
+                  "  remote_pending: %s\n"
+                  "  actor         : %u\n"
+                  "  reason        : %u: %s\n" ANSI_RESET,
+                  G_STRFUNC, message, add_str, rem_str, local_str, remote_str,
+                  actor, reason, group_change_reason_str(reason));
 
-      fflush (stdout);
+          fflush (stdout);
 
-      g_free (add_str);
-      g_free (rem_str);
-      g_free (local_str);
-      g_free (remote_str);
-      /* debug end */
+          g_free (add_str);
+          g_free (rem_str);
+          g_free (local_str);
+          g_free (remote_str);
+        }
 
+      if (actor)
+        {
+          handle_set_add (mixin->priv->actors, actor);
+        }
       /* emit signal */
       g_signal_emit(obj, mixin_cls->members_changed_signal_id, 0,
                     message,
                     arr_add, arr_remove,
-                    arr_local, arr_remote);
+                    arr_local, arr_remote,
+                    actor, reason);
 
       /* free arrays */
       g_array_free (arr_add, TRUE);
@@ -649,7 +691,7 @@ gabble_group_mixin_change_members (GObject *obj,
     }
   else
     {
-      g_debug ("%s: not emitting signal, nothing changed", G_STRFUNC);
+      DEBUG ("not emitting signal, nothing changed");
 
       ret = FALSE;
     }
