@@ -30,16 +30,21 @@
 
 #include <loudmouth/loudmouth.h>
 
+#define DEBUG_FLAG GABBLE_DEBUG_MUC
+
+#include "debug.h"
 #include "disco.h"
 #include "gabble-connection.h"
 #include "gabble-presence-cache.h"
 #include "gabble-muc-channel.h"
+#include "gabble-roomlist-channel.h"
 #include "handles.h"
 #include "muc-factory.h"
 #include "namespaces.h"
 #include "telepathy-interfaces.h"
 #include "text-mixin.h"
 #include "tp-channel-factory-iface.h"
+#include "util.h"
 
 static void gabble_muc_factory_iface_init (gpointer g_iface, gpointer iface_data);
 
@@ -62,6 +67,9 @@ struct _GabbleMucFactoryPrivate
   LmMessageHandler *presence_cb;
 
   GHashTable *channels;
+  GabbleRoomlistChannel *roomlist_channel;
+
+  GHashTable *disco_requests;
 
   gboolean dispose_has_run;
 };
@@ -78,6 +86,9 @@ gabble_muc_factory_init (GabbleMucFactory *fac)
   priv->channels = g_hash_table_new_full (g_direct_hash, g_direct_equal,
                                           NULL, g_object_unref);
 
+  priv->disco_requests = g_hash_table_new_full (g_direct_hash, g_direct_equal,
+                                                NULL, NULL);
+
   priv->message_cb = NULL;
   priv->presence_cb = NULL;
 
@@ -99,6 +110,14 @@ gabble_muc_factory_constructor (GType type, guint n_props,
   return obj;
 }
 
+static void
+cancel_disco_request (gpointer key, gpointer value, gpointer user_data)
+{
+  GabbleDisco *disco = GABBLE_DISCO (user_data);
+  GabbleDiscoRequest *request = (GabbleDiscoRequest *) key;
+
+  gabble_disco_cancel_request (disco, request);
+}
 
 static void
 gabble_muc_factory_dispose (GObject *object)
@@ -109,11 +128,14 @@ gabble_muc_factory_dispose (GObject *object)
   if (priv->dispose_has_run)
     return;
 
-  g_debug ("%s: dispose called", G_STRFUNC);
+  DEBUG ("dispose called");
   priv->dispose_has_run = TRUE;
 
   tp_channel_factory_iface_close_all (TP_CHANNEL_FACTORY_IFACE (object));
 
+  g_hash_table_foreach (priv->disco_requests, cancel_disco_request, priv->conn->disco);
+  g_hash_table_destroy (priv->disco_requests);
+
   if (G_OBJECT_CLASS (gabble_muc_factory_parent_class)->dispose)
     G_OBJECT_CLASS (gabble_muc_factory_parent_class)->dispose (object);
 }
@@ -181,19 +203,6 @@ gabble_muc_factory_class_init (GabbleMucFactoryClass *gabble_muc_factory_class)
   g_object_class_install_property (object_class, PROP_CONNECTION, param_spec);
 }
 
-static LmMessageNode *
-_get_muc_node (LmMessageNode *toplevel_node)
-{
-  LmMessageNode *node;
-
-  for (node = toplevel_node->children; node; node = node->next)
-    if (strcmp (node->name, "x") == 0)
-      if (_lm_message_node_has_namespace (node, NS_MUC_USER))
-        return node;
-
-  return NULL;
-}
-
 
 static GabbleMucChannel *
 get_muc_from_jid (GabbleMucFactory *fac, const gchar *jid)
@@ -226,12 +235,11 @@ muc_channel_closed_cb (GabbleMucChannel *chan, gpointer user_data)
   GabbleMucFactoryPrivate *priv = GABBLE_MUC_FACTORY_GET_PRIVATE (fac);
   GabbleHandle room_handle;
 
-  if (priv->channels)
+  if (priv->channels != NULL)
     {
       g_object_get (chan, "handle", &room_handle, NULL);
 
-      g_debug ("%s: removing MUC channel with handle %d", G_STRFUNC,
-          room_handle);
+      DEBUG ("removing MUC channel with handle %d", room_handle);
 
       g_hash_table_remove (priv->channels, GINT_TO_POINTER (room_handle));
     }
@@ -243,7 +251,7 @@ muc_ready_cb (GabbleMucChannel *chan,
 {
   GabbleMucFactory *fac = GABBLE_MUC_FACTORY (data);
 
-  g_debug ("%s: chan=%p", G_STRFUNC, chan);
+  DEBUG ("chan=%p", chan);
 
   g_signal_emit_by_name (fac, "new-channel", chan);
 }
@@ -255,8 +263,7 @@ muc_join_error_cb (GabbleMucChannel *chan,
 {
   GabbleMucFactory *fac = GABBLE_MUC_FACTORY (data);
 
-  g_debug ("%s: error->code=%u, error->message=\"%s\"", G_STRFUNC,
-           error->code, error->message);
+  DEBUG ("error->code=%u, error->message=\"%s\"", error->code, error->message);
 
   g_signal_emit_by_name (fac, "channel-error", chan, error);
 }
@@ -265,7 +272,7 @@ muc_join_error_cb (GabbleMucChannel *chan,
  * new_muc_channel
  */
 static GabbleMucChannel *
-new_muc_channel (GabbleMucFactory *fac, GabbleHandle handle)
+new_muc_channel (GabbleMucFactory *fac, GabbleHandle handle, gboolean invite_self)
 {
   GabbleMucFactoryPrivate *priv = GABBLE_MUC_FACTORY_GET_PRIVATE (fac);
   GabbleMucChannel *chan;
@@ -275,12 +282,13 @@ new_muc_channel (GabbleMucFactory *fac, GabbleHandle handle)
 
   object_path = g_strdup_printf ("%s/MucChannel%u", priv->conn->object_path, handle);
 
-  g_debug ("new_muc_channel: creating new chan, object path %s", object_path);
+  DEBUG ("creating new chan, object path %s", object_path);
 
   chan = g_object_new (GABBLE_TYPE_MUC_CHANNEL,
                        "connection", priv->conn,
                        "object-path", object_path,
                        "handle", handle,
+                       "invite-self", invite_self,
                        NULL);
 
   g_signal_connect (chan, "closed", (GCallback) muc_channel_closed_cb, fac);
@@ -297,6 +305,74 @@ new_muc_channel (GabbleMucFactory *fac, GabbleHandle handle)
 }
 
 
+struct DiscoInviteData {
+    GabbleMucFactory *factory;
+    gchar *reason;
+    GabbleHandle inviter;
+};
+
+/**
+ * obsolete_invite_disco_cb:
+ *
+ * Callback for disco request we fired upon encountering obsolete disco.
+ * If the object is in fact MUC room, create a channel for it.
+ */
+static void
+obsolete_invite_disco_cb (GabbleDisco *self,
+                          GabbleDiscoRequest *request,
+                          const gchar *jid,
+                          const gchar *node,
+                          LmMessageNode *query_result,
+                          GError* error,
+                          gpointer user_data)
+{
+  struct DiscoInviteData *data = (struct DiscoInviteData *) user_data;
+
+  GabbleMucFactory *fac = GABBLE_MUC_FACTORY (data->factory);
+  GabbleMucFactoryPrivate *priv = GABBLE_MUC_FACTORY_GET_PRIVATE (fac);
+  LmMessageNode *identity;
+  const char *category, *type;
+  GabbleHandle handle;
+
+  g_hash_table_remove (priv->disco_requests, request);
+
+  identity = lm_message_node_get_child (query_result, "identity");
+  if (NULL == identity)
+    return;
+
+  category = lm_message_node_get_attribute (identity, "category");
+  if (NULL == category)
+    return;
+
+  type = lm_message_node_get_attribute (identity, "type");
+  if (NULL == type)
+    return;
+
+  if (0 != strcmp (category, "conference") ||
+      0 != strcmp (type, "text"))
+    {
+      DEBUG ("obsolete invite request specified invalid jid '%s', ignoring", jid);
+    }
+
+  /* OK, it's MUC after all, create a new channel */
+  handle = gabble_handle_for_room (priv->conn->handles, jid);
+
+  if (g_hash_table_lookup (priv->channels, GINT_TO_POINTER (handle)) == NULL)
+    {
+      GabbleMucChannel *chan;
+      chan = new_muc_channel (fac, handle, FALSE);
+      _gabble_muc_channel_handle_invited (chan, data->inviter, data->reason);
+    }
+  else
+    {
+      DEBUG ("ignoring invite to a room '%s' we're already in", jid);
+    }
+
+  g_free (data->reason);
+  g_free (data);
+}
+
+
 /**
  * muc_factory_message_cb:
  *
@@ -316,20 +392,23 @@ muc_factory_message_cb (LmMessageHandler *handler,
   time_t stamp;
   TpChannelTextMessageType msgtype;
   LmMessageNode *node;
+  TpHandleType handle_type;
   GabbleHandle room_handle, handle;
   GabbleMucChannel *chan;
   GabbleTextMixinSendError send_error;
 
-  if (!gabble_text_mixin_parse_incoming_message (message, &from, &stamp, &msgtype, &body, &body_offset, &send_error))
+  if (!gabble_text_mixin_parse_incoming_message (message, &from, &stamp,
+        &msgtype, &body, &body_offset, &send_error))
     return LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS;
 
   /* does it have a muc subnode? */
-  node = _get_muc_node (message->node);
-  if (node)
+  node = lm_message_node_get_child_with_namespace (message->node, "x",
+      NS_MUC_USER);
+  if (node != NULL)
     {
       /* and an invitation? */
       node = lm_message_node_get_child (node, "invite");
-      if (node)
+      if (node != NULL)
         {
           LmMessageNode *reason_node;
           const gchar *invite_from, *reason;
@@ -337,7 +416,7 @@ muc_factory_message_cb (LmMessageHandler *handler,
 
           if (send_error != CHANNEL_TEXT_SEND_NO_ERROR)
             {
-              HANDLER_DEBUG (message->node, "got a MUC invitation message "
+              NODE_DEBUG (message->node, "got a MUC invitation message "
                              "with a send error; ignoring");
 
               return LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS;
@@ -346,7 +425,7 @@ muc_factory_message_cb (LmMessageHandler *handler,
           invite_from = lm_message_node_get_attribute (node, "from");
           if (invite_from == NULL)
             {
-              HANDLER_DEBUG (message->node, "got a MUC invitation message "
+              NODE_DEBUG (message->node, "got a MUC invitation message "
                              "without a from field on the invite node, "
                              "ignoring");
 
@@ -357,14 +436,14 @@ muc_factory_message_cb (LmMessageHandler *handler,
                                                       invite_from, FALSE);
 
           reason_node = lm_message_node_get_child (node, "reason");
-          if (reason_node)
+          if (reason_node != NULL)
             {
               reason = lm_message_node_get_value (reason_node);
             }
           else
             {
               reason = "";
-              HANDLER_DEBUG (message->node, "no MUC invite reason specified");
+              NODE_DEBUG (message->node, "no MUC invite reason specified");
             }
 
           /* create the channel */
@@ -372,17 +451,62 @@ muc_factory_message_cb (LmMessageHandler *handler,
 
           if (g_hash_table_lookup (priv->channels, GINT_TO_POINTER (handle)) == NULL)
             {
-              chan = new_muc_channel (fac, handle);
+              chan = new_muc_channel (fac, handle, FALSE);
               _gabble_muc_channel_handle_invited (chan, inviter_handle, reason);
             }
           else
             {
-              HANDLER_DEBUG (message->node, "ignoring invite to a room we're already in");
+              NODE_DEBUG (message->node, "ignoring invite to a room we're already in");
             }
 
           return LM_HANDLER_RESULT_REMOVE_MESSAGE;
         }
     }
+  else
+    {
+      GabbleHandle inviter_handle;
+      GabbleDiscoRequest *request;
+      const gchar *reason;
+      struct DiscoInviteData *disco_udata;
+
+      /* check for obsolete invite method */
+      for (node = message->node->children; node != NULL; node = node->next)
+        if (strcmp (node->name, "x") == 0)
+          if (lm_message_node_has_namespace (node, NS_X_CONFERENCE, NULL))
+            break;
+
+      if (node == NULL)
+        goto HANDLE_MESSAGE;
+
+      /* the room JID is in x */
+      from = lm_message_node_get_attribute (node, "jid");
+      if (from == NULL)
+        return LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS;
+
+      /* the inviter JID is in "from" */
+      inviter_handle = gabble_handle_for_contact (priv->conn->handles,
+                                                  from, FALSE);
+
+      /* reason is the body */
+      reason = body;
+
+      disco_udata = g_new0 (struct DiscoInviteData, 1);
+      disco_udata->factory = fac;
+      disco_udata->reason = g_strdup (reason);
+      disco_udata->inviter = inviter_handle;
+
+      NODE_DEBUG (message->node, "received obsolete invite method");
+
+      request = gabble_disco_request (priv->conn->disco, GABBLE_DISCO_TYPE_INFO,
+          from, NULL, obsolete_invite_disco_cb, disco_udata, G_OBJECT (fac), NULL);
+
+      if (request != NULL)
+        g_hash_table_insert (priv->disco_requests, request, NULL);
+
+      return LM_HANDLER_RESULT_REMOVE_MESSAGE;
+  }
+
+HANDLE_MESSAGE:
 
   /* check if a room with the jid exists */
   if (!gabble_handle_for_room_exists (priv->conn->handles, from, TRUE))
@@ -395,7 +519,7 @@ muc_factory_message_cb (LmMessageHandler *handler,
   /* find the MUC channel */
   chan = g_hash_table_lookup (priv->channels, GUINT_TO_POINTER (room_handle));
 
-  if (!chan)
+  if (chan == NULL)
     {
       g_warning ("%s: ignoring groupchat message from known handle with "
                  "no MUC channel", G_STRFUNC);
@@ -407,10 +531,12 @@ muc_factory_message_cb (LmMessageHandler *handler,
    * itself or one of its members */
   if (gabble_handle_for_room_exists (priv->conn->handles, from, FALSE))
     {
+      handle_type = TP_HANDLE_TYPE_ROOM;
       handle = room_handle;
     }
   else
     {
+      handle_type = TP_HANDLE_TYPE_CONTACT;
       handle = gabble_handle_for_contact (priv->conn->handles, from, TRUE);
     }
 
@@ -421,7 +547,7 @@ muc_factory_message_cb (LmMessageHandler *handler,
       return LM_HANDLER_RESULT_REMOVE_MESSAGE;
     }
 
-  if (_gabble_muc_channel_receive (chan, msgtype, handle, stamp,
+  if (_gabble_muc_channel_receive (chan, msgtype, handle_type, handle, stamp,
                                    body_offset, message))
     return LM_HANDLER_RESULT_REMOVE_MESSAGE;
 
@@ -457,7 +583,7 @@ muc_factory_presence_cb (LmMessageHandler *handler,
 
   if (from == NULL)
     {
-      HANDLER_DEBUG (msg->node, "presence stanza without from attribute, ignoring");
+      NODE_DEBUG (msg->node, "presence stanza without from attribute, ignoring");
       return LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS;
     }
 
@@ -474,10 +600,10 @@ muc_factory_presence_cb (LmMessageHandler *handler,
       return LM_HANDLER_RESULT_REMOVE_MESSAGE;
     }
 
-  x_node = _get_muc_node (msg->node);
+  x_node = lm_message_node_get_child_with_namespace (msg->node, "x", NS_MUC_USER);
 
   /* is it a MUC member presence? */
-  if (x_node)
+  if (x_node != NULL)
     {
       if (muc_chan != NULL)
         {
@@ -486,7 +612,7 @@ muc_factory_presence_cb (LmMessageHandler *handler,
           handle = gabble_handle_for_contact (priv->conn->handles, from, TRUE);
           if (handle == 0)
             {
-              HANDLER_DEBUG (msg->node, "discarding MUC presence from malformed jid");
+              NODE_DEBUG (msg->node, "discarding MUC presence from malformed jid");
               return LM_HANDLER_RESULT_REMOVE_MESSAGE;
             }
 
@@ -495,7 +621,7 @@ muc_factory_presence_cb (LmMessageHandler *handler,
         }
       else
         {
-          HANDLER_DEBUG (msg->node, "discarding unexpected MUC member presence");
+          NODE_DEBUG (msg->node, "discarding unexpected MUC member presence");
 
           return LM_HANDLER_RESULT_REMOVE_MESSAGE;
         }
@@ -505,20 +631,73 @@ muc_factory_presence_cb (LmMessageHandler *handler,
 
 }
 
+static void
+roomlist_channel_closed_cb (GabbleRoomlistChannel *chan, gpointer data)
+{
+  GabbleMucFactory *fac = GABBLE_MUC_FACTORY (data);
+  GabbleMucFactoryPrivate *priv = GABBLE_MUC_FACTORY_GET_PRIVATE (fac);
+
+  if (priv->roomlist_channel != NULL)
+    {
+      g_object_unref (priv->roomlist_channel);
+      priv->roomlist_channel = NULL;
+    }
+}
+
+static gboolean
+make_roomlist_channel (GabbleMucFactory *fac)
+{
+  GabbleMucFactoryPrivate *priv = GABBLE_MUC_FACTORY_GET_PRIVATE (fac);
+
+  if (priv->roomlist_channel == NULL)
+    {
+      const gchar *server;
+      gchar *object_path;
+
+      server = _gabble_connection_find_conference_server (priv->conn);
+
+      if (server == NULL)
+        return FALSE;
+
+      object_path = g_strdup_printf ("%s/RoomlistChannel", priv->conn->object_path);
+
+      priv->roomlist_channel = _gabble_roomlist_channel_new (priv->conn,
+          object_path, server);
+
+      g_signal_connect (priv->roomlist_channel, "closed",
+                        (GCallback) roomlist_channel_closed_cb, fac);
+
+      g_signal_emit_by_name (fac, "new-channel", priv->roomlist_channel);
+
+      g_free (object_path);
+    }
+
+  return TRUE;
+}
+
+
+
 static void
 gabble_muc_factory_iface_close_all (TpChannelFactoryIface *iface)
 {
   GabbleMucFactory *fac = GABBLE_MUC_FACTORY (iface);
   GabbleMucFactoryPrivate *priv = GABBLE_MUC_FACTORY_GET_PRIVATE (fac);
 
-  g_debug ("%s: closing channels", G_STRFUNC);
+  DEBUG ("closing channels");
 
-  if (priv->channels)
+  if (priv->channels != NULL)
     {
       GHashTable *tmp = priv->channels;
       priv->channels = NULL;
       g_hash_table_destroy (tmp);
     }
+
+  if (priv->roomlist_channel != NULL)
+    {
+      GObject *tmp = G_OBJECT (priv->roomlist_channel);
+      priv->roomlist_channel = NULL;
+      g_object_unref (tmp);
+    }
 }
 
 static void
@@ -527,7 +706,7 @@ gabble_muc_factory_iface_connecting (TpChannelFactoryIface *iface)
   GabbleMucFactory *fac = GABBLE_MUC_FACTORY (iface);
   GabbleMucFactoryPrivate *priv = GABBLE_MUC_FACTORY_GET_PRIVATE (fac);
 
-  g_debug ("%s: adding callbacks", G_STRFUNC);
+  DEBUG ("adding callbacks");
 
   g_assert (priv->message_cb == NULL);
   g_assert (priv->presence_cb == NULL);
@@ -556,7 +735,7 @@ gabble_muc_factory_iface_disconnected (TpChannelFactoryIface *iface)
   GabbleMucFactory *fac = GABBLE_MUC_FACTORY (iface);
   GabbleMucFactoryPrivate *priv = GABBLE_MUC_FACTORY_GET_PRIVATE (fac);
 
-  g_debug ("%s: removing callbacks", G_STRFUNC);
+  DEBUG ("removing callbacks");
 
   g_assert (priv->message_cb != NULL);
   g_assert (priv->presence_cb != NULL);
@@ -598,6 +777,9 @@ gabble_muc_factory_iface_foreach (TpChannelFactoryIface *iface, TpChannelFunc fo
   data.foreach = foreach;
 
   g_hash_table_foreach (priv->channels, _foreach_slave, &data);
+
+  if (priv->roomlist_channel != NULL)
+    foreach (TP_CHANNEL_IFACE (priv->roomlist_channel), user_data);
 }
 
 static TpChannelFactoryRequestStatus
@@ -605,14 +787,24 @@ gabble_muc_factory_iface_request (TpChannelFactoryIface *iface,
                                   const gchar *chan_type,
                                   TpHandleType handle_type,
                                   guint handle,
-                                  TpChannelIface **ret)
+                                  TpChannelIface **ret,
+                                  GError **error)
 {
   GabbleMucFactory *fac = GABBLE_MUC_FACTORY (iface);
   GabbleMucFactoryPrivate *priv = GABBLE_MUC_FACTORY_GET_PRIVATE (fac);
   GabbleMucChannel *chan;
-  GArray *members;
-  GError *error;
-  gboolean retval;
+
+  if (!g_strdiff (chan_type, TP_IFACE_CHANNEL_TYPE_ROOM_LIST))
+    {
+      /* FIXME - delay if services aren't discovered yet? */
+      if (!make_roomlist_channel (fac))
+        {
+          DEBUG ("no conference server available for roomlist request");
+          return TP_CHANNEL_FACTORY_REQUEST_STATUS_NOT_AVAILABLE;
+        }
+      *ret = TP_CHANNEL_IFACE (priv->roomlist_channel);
+      return TP_CHANNEL_FACTORY_REQUEST_STATUS_DONE;
+    }
 
   if (strcmp (chan_type, TP_IFACE_CHANNEL_TYPE_TEXT))
     return TP_CHANNEL_FACTORY_REQUEST_STATUS_NOT_IMPLEMENTED;
@@ -626,33 +818,19 @@ gabble_muc_factory_iface_request (TpChannelFactoryIface *iface,
   chan = g_hash_table_lookup (priv->channels, GINT_TO_POINTER (handle));
   if (!chan)
     {
-      chan = new_muc_channel (fac, handle);
-
-      members = g_array_sized_new (FALSE, FALSE, sizeof (GabbleHandle), 1);
-      g_array_append_val (members, priv->conn->self_handle);
-
-      retval = gabble_group_mixin_add_members (G_OBJECT (chan), members, "", &error);
-
-      g_array_free (members, TRUE);
-
-      if (!retval)
-        {
-          GError *close_err;
-
-          if (!gabble_muc_channel_close (GABBLE_MUC_CHANNEL (chan), &close_err))
-            {
-              g_error_free (close_err);
-            }
-
-          g_debug ("%s: error while adding self to group mixin", G_STRFUNC);
-          return TP_CHANNEL_FACTORY_REQUEST_STATUS_NOT_AVAILABLE;
-        }
-
+      chan = new_muc_channel (fac, handle, TRUE);
       return TP_CHANNEL_FACTORY_REQUEST_STATUS_QUEUED;
     }
 
-  *ret = TP_CHANNEL_IFACE (chan);
-  return TP_CHANNEL_FACTORY_REQUEST_STATUS_DONE;
+  if (_gabble_muc_channel_is_ready (chan))
+    {
+      *ret = TP_CHANNEL_IFACE (chan);
+      return TP_CHANNEL_FACTORY_REQUEST_STATUS_DONE;
+    }
+  else
+    {
+      return TP_CHANNEL_FACTORY_REQUEST_STATUS_QUEUED;
+    }
 }
 
 static void
