@@ -22,25 +22,36 @@
 #define __GABBLE_NAMESPACES__H__
 
 #define NS_CAPS                 "http://jabber.org/protocol/caps"
-#define NS_DATA                 "jabber:x:data"
 #define NS_DISCO_INFO           "http://jabber.org/protocol/disco#info"
 #define NS_DISCO_ITEMS          "http://jabber.org/protocol/disco#items"
 #define NS_GABBLE_CAPS          "http://telepathy.freedesktop.org/caps"
 #define NS_GOOGLE_CAPS          "http://www.google.com/xmpp/client/caps"
+#define NS_GOOGLE_FEAT_SESSION  "http://www.google.com/xmpp/protocol/session"
+#define NS_GOOGLE_FEAT_VOICE    "http://www.google.com/xmpp/protocol/voice/v1"
 #define NS_GOOGLE_JINGLE_INFO   "google:jingleinfo"
 #define NS_GOOGLE_ROSTER        "google:roster"
 #define NS_GOOGLE_SESSION       "http://www.google.com/session"
 #define NS_GOOGLE_SESSION_PHONE "http://www.google.com/session/phone"
-#define NS_GOOGLE_VOICE         "http://www.google.com/xmpp/protocol/voice/v1"
-#define NS_JINGLE_AUDIO         "http://jabber.org/protocol/jingle/media/audio"
+#define NS_GOOGLE_TRANSPORT_P2P "http://www.google.com/transport/p2p"
 #define NS_JINGLE               "http://jabber.org/protocol/jingle"
+#define NS_JINGLE_DESCRIPTION_AUDIO "http://jabber.org/protocol/jingle/description/audio"
+#define NS_JINGLE_DESCRIPTION_VIDEO "http://jabber.org/protocol/jingle/description/video"
+#define NS_JINGLE_ERRORS            "http://jabber.org/protocol/jingle#errors"
+#define NS_JINGLE_TRANSPORT_ICE     "http://jabber.org/protocol/jingle/transport/ice"
 #define NS_MUC                  "http://jabber.org/protocol/muc"
 #define NS_MUC_USER             "http://jabber.org/protocol/muc#user"
 #define NS_MUC_ADMIN            "http://jabber.org/protocol/muc#admin"
 #define NS_MUC_OWNER            "http://jabber.org/protocol/muc#owner"
 #define NS_NICK                 "http://jabber.org/protocol/nick"
+#define NS_NOKIA_IV             "http://xis.nokia.fi/xmpp/privilege"
 #define NS_PRESENCE_INVISIBLE   "presence-invisible"
 #define NS_PRIVACY              "jabber:iq:privacy"
+#define NS_REGISTER             "jabber:iq:register"
 #define NS_ROSTER               "jabber:iq:roster"
+#define NS_VCARD_TEMP           "vcard-temp"
+#define NS_X_DATA               "jabber:x:data"
+#define NS_X_DELAY              "jabber:x:delay"
+#define NS_X_CONFERENCE         "jabber:x:conference"
+#define NS_XMPP_STANZAS         "urn:ietf:params:xml:ns:xmpp-stanzas"
 
 #endif /* __GABBLE_NAMESPACES__H__ */
