@@ -24,7 +24,10 @@
 #include <stdio.h>
 #include <string.h>
 
+#define DEBUG_FLAG GABBLE_DEBUG_PROPERTIES
+
 #include "ansi.h"
+#include "debug.h"
 #include "properties-mixin.h"
 #include "properties-mixin-signals-marshal.h"
 #include "telepathy-errors.h"
@@ -98,7 +101,7 @@ void gabble_properties_mixin_class_init (GObjectClass *obj_cls,
                   G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
                   0,
                   NULL, NULL,
-                  properties_mixin_marshal_VOID__BOXED,
+                  g_cclosure_marshal_VOID__BOXED,
                   G_TYPE_NONE, 1, (dbus_g_type_get_collection ("GPtrArray", (dbus_g_type_get_struct ("GValueArray", G_TYPE_UINT, G_TYPE_VALUE, G_TYPE_INVALID)))));
 
   mixin_cls->property_flags_changed_signal_id =
@@ -107,7 +110,7 @@ void gabble_properties_mixin_class_init (GObjectClass *obj_cls,
                   G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
                   0,
                   NULL, NULL,
-                  properties_mixin_marshal_VOID__BOXED,
+                  g_cclosure_marshal_VOID__BOXED,
                   G_TYPE_NONE, 1, (dbus_g_type_get_collection ("GPtrArray", (dbus_g_type_get_struct ("GValueArray", G_TYPE_UINT, G_TYPE_UINT, G_TYPE_INVALID)))));
 }
 
@@ -200,6 +203,7 @@ gabble_properties_mixin_list_properties (GObject *obj, GPtrArray **ret, GError *
           break;
         default:
           g_assert_not_reached ();
+          continue;
       };
 
       g_value_init (&val, TP_TYPE_PROPERTY_INFO_STRUCT);
@@ -235,17 +239,17 @@ gabble_properties_mixin_get_properties (GObject *obj, const GArray *properties,
       /* Valid? */
       if (prop_id >= mixin_cls->num_props)
         {
-          *error = g_error_new (TELEPATHY_ERRORS, InvalidArgument,
-                                "invalid property identifier %d", prop_id);
+          g_set_error (error, TELEPATHY_ERRORS, InvalidArgument,
+              "invalid property identifier %d", prop_id);
 
           return FALSE;
         }
 
       /* Permitted? */
-      if (!(mixin->properties[prop_id].flags & TP_PROPERTY_FLAG_READ))
+      if (!gabble_properties_mixin_is_readable (obj, prop_id))
         {
-          *error = g_error_new (TELEPATHY_ERRORS, PermissionDenied,
-                                "permission denied for property identifier %d", prop_id);
+          g_set_error (error, TELEPATHY_ERRORS, PermissionDenied,
+              "permission denied for property identifier %d", prop_id);
 
           return FALSE;
         }
@@ -275,14 +279,15 @@ gabble_properties_mixin_get_properties (GObject *obj, const GArray *properties,
   return TRUE;
 }
 
-gboolean
-gabble_properties_mixin_set_properties (GObject *obj, const GPtrArray *properties, DBusGMethodInvocation *context)
+void
+gabble_properties_mixin_set_properties (GObject *obj,
+                                        const GPtrArray *properties,
+                                        DBusGMethodInvocation *context)
 {
   GabblePropertiesMixin *mixin = GABBLE_PROPERTIES_MIXIN (obj);
   GabblePropertiesMixinClass *mixin_cls = GABBLE_PROPERTIES_MIXIN_CLASS (
                                             G_OBJECT_GET_CLASS (obj));
   GabblePropertiesContext *ctx = &mixin->priv->context;
-  gboolean result;
   GError *error;
   guint i;
 
@@ -295,7 +300,6 @@ gabble_properties_mixin_set_properties (GObject *obj, const GPtrArray *propertie
     }
 
   ctx->dbus_ctx = context;
-  result = TRUE;
   error = NULL;
 
   /* Check input property identifiers */
@@ -328,7 +332,7 @@ gabble_properties_mixin_set_properties (GObject *obj, const GPtrArray *propertie
       ctx->values[prop_id] = prop_val;
 
       /* Permitted? */
-      if (!(mixin->properties[prop_id].flags & TP_PROPERTY_FLAG_WRITE))
+      if (!gabble_properties_mixin_is_writable (obj, prop_id))
         {
           error = g_error_new (TELEPATHY_ERRORS, PermissionDenied,
                                "permission denied for property identifier %d", prop_id);
@@ -349,20 +353,16 @@ gabble_properties_mixin_set_properties (GObject *obj, const GPtrArray *propertie
   if (mixin_cls->set_properties)
     {
       if (mixin_cls->set_properties (obj, ctx, &error))
-        goto OUT;
+        return;
     }
   else
     {
       gabble_properties_context_return (ctx, NULL);
-      goto OUT;
+      return;
     }
 
 ERROR:
   gabble_properties_context_return (ctx, error);
-  result = FALSE;
-
-OUT:
-  return result;
 }
 
 gboolean
@@ -439,9 +439,9 @@ gabble_properties_context_return (GabblePropertiesContext *ctx, GError *error)
 {
   GObject *obj = ctx->mixin->priv->object;
   GArray *changed_props_val, *changed_props_flags;
-  int i;
+  guint i;
 
-  g_debug ("%s: %s", G_STRFUNC, (error) ? "failure" : "success");
+  DEBUG ("%s", (error) ? "failure" : "success");
 
   changed_props_val = changed_props_flags = NULL;
 
@@ -631,7 +631,7 @@ gabble_properties_mixin_change_flags (GObject *obj, guint prop_id,
 
   if (props)
     {
-      gint i;
+      guint i;
 
       if (*props == NULL)
         {
@@ -675,9 +675,10 @@ gabble_properties_mixin_emit_changed (GObject *obj, GArray **props)
 
   prop_arr = g_ptr_array_sized_new ((*props)->len);
 
-  printf (ANSI_BOLD_ON ANSI_FG_CYAN
-          "%s: emitting properties changed for propert%s:\n",
-          G_STRFUNC, ((*props)->len > 1) ? "ies" : "y");
+  if (DEBUGGING)
+    printf (ANSI_BOLD_ON ANSI_FG_CYAN
+            "%s: emitting properties changed for propert%s:\n",
+            G_STRFUNC, ((*props)->len > 1) ? "ies" : "y");
 
   for (i = 0; i < (*props)->len; i++)
     {
@@ -695,11 +696,15 @@ gabble_properties_mixin_emit_changed (GObject *obj, GArray **props)
 
       g_ptr_array_add (prop_arr, g_value_get_boxed (&prop_val));
 
-      printf ("  %s\n", mixin_cls->signatures[prop_id].name);
+      if (DEBUGGING)
+        printf ("  %s\n", mixin_cls->signatures[prop_id].name);
     }
 
-  printf (ANSI_RESET);
-  fflush (stdout);
+  if (DEBUGGING)
+    {
+      printf (ANSI_RESET);
+      fflush (stdout);
+    }
 
   g_signal_emit (obj, mixin_cls->properties_changed_signal_id, 0, prop_arr);
 
@@ -729,16 +734,16 @@ gabble_properties_mixin_emit_flags (GObject *obj, GArray **props)
 
   prop_arr = g_ptr_array_sized_new ((*props)->len);
 
-  printf (ANSI_BOLD_ON ANSI_FG_WHITE
-          "%s: emitting properties flags changed for propert%s:\n",
-          G_STRFUNC, ((*props)->len > 1) ? "ies" : "y");
+  if (DEBUGGING)
+    printf (ANSI_BOLD_ON ANSI_FG_WHITE
+            "%s: emitting properties flags changed for propert%s:\n",
+            G_STRFUNC, ((*props)->len > 1) ? "ies" : "y");
 
   for (i = 0; i < (*props)->len; i++)
     {
       GValue prop_val = { 0, };
       guint prop_id = g_array_index (*props, guint, i);
       guint prop_flags;
-      gchar *str_flags;
 
       prop_flags = mixin->properties[prop_id].flags;
 
@@ -753,16 +758,22 @@ gabble_properties_mixin_emit_flags (GObject *obj, GArray **props)
 
       g_ptr_array_add (prop_arr, g_value_get_boxed (&prop_val));
 
-      str_flags = property_flags_to_string (prop_flags);
+      if (DEBUGGING)
+        {
+          gchar *str_flags = property_flags_to_string (prop_flags);
 
-      printf ("  %s's flags now: %s\n",
-              mixin_cls->signatures[prop_id].name, str_flags);
+          printf ("  %s's flags now: %s\n",
+                  mixin_cls->signatures[prop_id].name, str_flags);
 
-      g_free (str_flags);
+          g_free (str_flags);
+        }
     }
 
-  printf (ANSI_RESET);
-  fflush (stdout);
+  if (DEBUGGING)
+    {
+      printf (ANSI_RESET);
+      fflush (stdout);
+    }
 
   g_signal_emit (obj, mixin_cls->property_flags_changed_signal_id, 0, prop_arr);
 
@@ -774,3 +785,29 @@ gabble_properties_mixin_emit_flags (GObject *obj, GArray **props)
   *props = NULL;
 }
 
+gboolean
+gabble_properties_mixin_is_readable (GObject *obj, guint prop_id)
+{
+  GabblePropertiesMixin *mixin = GABBLE_PROPERTIES_MIXIN (obj);
+  GabblePropertiesMixinClass *mixin_cls = GABBLE_PROPERTIES_MIXIN_CLASS (
+                                            G_OBJECT_GET_CLASS (obj));
+
+  if (prop_id >= mixin_cls->num_props)
+    return FALSE;
+
+  return ((mixin->properties[prop_id].flags & TP_PROPERTY_FLAG_READ) != 0);
+}
+
+gboolean
+gabble_properties_mixin_is_writable (GObject *obj, guint prop_id)
+{
+  GabblePropertiesMixin *mixin = GABBLE_PROPERTIES_MIXIN (obj);
+  GabblePropertiesMixinClass *mixin_cls = GABBLE_PROPERTIES_MIXIN_CLASS (
+                                            G_OBJECT_GET_CLASS (obj));
+
+  if (prop_id >= mixin_cls->num_props)
+    return FALSE;
+
+  return ((mixin->properties[prop_id].flags & TP_PROPERTY_FLAG_WRITE) != 0);
+}
+
