@@ -27,12 +27,17 @@
 #include "telepathy-interfaces.h"
 #include "tp-channel-factory-iface.h"
 
+#define DEBUG_FLAG GABBLE_DEBUG_ROSTER
+
+#include "debug.h"
 #include "gabble-connection.h"
 #include "gabble-roster-channel.h"
 #include "namespaces.h"
 #include "roster.h"
 #include "util.h"
 
+#define GOOGLE_ROSTER_VERSION "2"
+
 /* Properties */
 enum
 {
@@ -64,11 +69,20 @@ struct _GabbleRosterPrivate
   gboolean dispose_has_run;
 };
 
+typedef enum
+{
+  GOOGLE_ITEM_TYPE_NORMAL = 0,
+  GOOGLE_ITEM_TYPE_BLOCKED,
+  GOOGLE_ITEM_TYPE_HIDDEN,
+  GOOGLE_ITEM_TYPE_PINNED
+} GoogleItemType;
+
 typedef struct _GabbleRosterItem GabbleRosterItem;
 struct _GabbleRosterItem
 {
   GabbleRosterSubscription subscription;
   gboolean ask_subscribe;
+  GoogleItemType google_type;
   gchar *name;
   gchar **groups;
 };
@@ -169,7 +183,7 @@ gabble_roster_dispose (GObject *object)
   if (priv->dispose_has_run)
     return;
 
-  g_debug ("%s: dispose called", G_STRFUNC);
+  DEBUG ("dispose called");
 
   priv->dispose_has_run = TRUE;
 
@@ -183,14 +197,24 @@ gabble_roster_dispose (GObject *object)
     G_OBJECT_CLASS (gabble_roster_parent_class)->dispose (object);
 }
 
+static void
+item_handle_unref_foreach (gpointer key, gpointer data, gpointer user_data)
+{
+  GabbleHandle handle = (GabbleHandle) key;
+  GabbleRosterPrivate *priv = (GabbleRosterPrivate *) user_data;
+
+  gabble_handle_unref (priv->conn->handles, TP_HANDLE_TYPE_CONTACT, handle);
+}
+
 void
 gabble_roster_finalize (GObject *object)
 {
   GabbleRoster *self = GABBLE_ROSTER (object);
   GabbleRosterPrivate *priv = GABBLE_ROSTER_GET_PRIVATE (self);
 
-  g_debug ("%s called with %p", G_STRFUNC, object);
+  DEBUG ("called with %p", object);
 
+  g_hash_table_foreach (priv->items, item_handle_unref_foreach, priv);
   g_hash_table_destroy (priv->items);
 
   G_OBJECT_CLASS (gabble_roster_parent_class)->finalize (object);
@@ -286,7 +310,7 @@ _parse_item_subscription (LmMessageNode *item_node)
     return GABBLE_ROSTER_SUBSCRIPTION_REMOVE;
   else
     {
-      HANDLER_DEBUG (item_node, "got unexpected subscription value");
+       NODE_DEBUG (item_node, "got unexpected subscription value");
       return GABBLE_ROSTER_SUBSCRIPTION_NONE;
     }
 }
@@ -317,6 +341,94 @@ _parse_item_groups (LmMessageNode *item_node)
   return (gchar **) g_ptr_array_free (strv, FALSE);
 }
 
+static const gchar *
+_google_item_type_to_string (GoogleItemType google_type)
+{
+  switch (google_type)
+    {
+      case GOOGLE_ITEM_TYPE_NORMAL:
+        return NULL;
+      case GOOGLE_ITEM_TYPE_BLOCKED:
+        return "B";
+      case GOOGLE_ITEM_TYPE_HIDDEN:
+        return "H";
+      case GOOGLE_ITEM_TYPE_PINNED:
+        return "P";
+    }
+
+  g_assert_not_reached ();
+
+  return NULL;
+}
+
+static GoogleItemType
+_parse_google_item_type (LmMessageNode *item_node)
+{
+  const gchar *google_type;
+
+  g_assert (item_node != NULL);
+
+  google_type = lm_message_node_get_attribute (item_node, "gr:t");
+
+  if (NULL == google_type)
+    return GOOGLE_ITEM_TYPE_NORMAL;
+  else if (!g_strdiff (google_type, "B"))
+    return GOOGLE_ITEM_TYPE_BLOCKED;
+  else if (!g_strdiff (google_type, "H"))
+    return GOOGLE_ITEM_TYPE_HIDDEN;
+  else if (!g_strdiff (google_type, "P"))
+    return GOOGLE_ITEM_TYPE_PINNED;
+
+  NODE_DEBUG (item_node, "got unexpected google contact type value");
+
+  return GOOGLE_ITEM_TYPE_NORMAL;
+}
+
+static gboolean
+_google_roster_item_should_keep (LmMessageNode *item_node,
+                                 GabbleRosterItem *item)
+{
+  const gchar *attr;
+
+  /* skip automatically subscribed Google roster items */
+  attr = lm_message_node_get_attribute (item_node, "gr:autosub");
+
+  if (!g_strdiff (attr, "true"))
+    return FALSE;
+
+  /* skip email addresses the user has invited */
+  attr = lm_message_node_get_attribute (item_node, "gr:inv");
+
+  if (!g_strdiff (attr, "A"))
+    return FALSE;
+
+  /* skip email addresses that replied to an invite */
+  attr = lm_message_node_get_attribute (item_node, "gr:alias-for");
+
+  if (attr != NULL)
+    return FALSE;
+
+  /* skip hidden items */
+  if (item->google_type == GOOGLE_ITEM_TYPE_HIDDEN)
+    return FALSE;
+
+  /* allow items that have rejected a subscription */
+  attr = lm_message_node_get_attribute (item_node, "gr:rejected");
+
+  if (!g_strdiff (attr, "true"))
+    return TRUE;
+
+  /* allow items that we've requested a subscription from */
+  if (item->ask_subscribe)
+    return TRUE;
+
+  if (item->subscription != GABBLE_ROSTER_SUBSCRIPTION_NONE)
+    return TRUE;
+
+  /* discard anything else */
+  return FALSE;
+}
+
 static GabbleRosterItem *
 _gabble_roster_item_get (GabbleRoster *roster,
                          GabbleHandle handle)
@@ -359,7 +471,8 @@ _gabble_roster_item_remove (GabbleRoster *roster,
 static GabbleRosterItem *
 _gabble_roster_item_update (GabbleRoster *roster,
                             GabbleHandle handle,
-                            LmMessageNode *node)
+                            LmMessageNode *node,
+                            gboolean google_roster_mode)
 {
   GabbleRosterPrivate *priv = GABBLE_ROSTER_GET_PRIVATE (roster);
   GabbleRosterItem *item;
@@ -381,14 +494,30 @@ _gabble_roster_item_update (GabbleRoster *roster,
   else
     item->ask_subscribe = FALSE;
 
-  name = lm_message_node_get_attribute (node, "name");
+  if (google_roster_mode)
+    {
+      item->google_type = _parse_google_item_type (node);
+
+      /* discard roster item if strange */
+      if (!_google_roster_item_should_keep (node, item))
+        {
+          DEBUG ("Google roster: discarding odd contact %d (%s)", handle,
+              lm_message_node_get_attribute (node, "jid"));
+          item->subscription = GABBLE_ROSTER_SUBSCRIPTION_REMOVE;
+        }
+    }
+
+  if (item->subscription == GABBLE_ROSTER_SUBSCRIPTION_REMOVE)
+    name = NULL;
+  else
+    name = lm_message_node_get_attribute (node, "name");
+
   if (g_strdiff (item->name, name))
     {
       g_free (item->name);
       item->name = g_strdup (name);
 
-      g_debug ("%s: name for handle %d changed to %s", G_STRFUNC, handle,
-          name);
+      DEBUG ("name for handle %d changed to %s", handle, name);
       g_signal_emit (G_OBJECT (roster), signals[NICKNAME_UPDATE], 0, handle);
     }
 
@@ -399,6 +528,7 @@ _gabble_roster_item_update (GabbleRoster *roster,
 }
 
 
+#ifdef ENABLE_DEBUG
 static gchar *
 _gabble_roster_item_dump (GabbleRosterItem *item)
 {
@@ -413,6 +543,10 @@ _gabble_roster_item_dump (GabbleRosterItem *item)
   if (item->ask_subscribe)
     g_string_append (str, ", ask: subscribe");
 
+  if (item->google_type != GOOGLE_ITEM_TYPE_NORMAL)
+    g_string_append_printf (str, ", google_type: %s",
+        _google_item_type_to_string (item->google_type));
+
   if (item->name)
     g_string_append_printf (str, ", name: %s", item->name);
 
@@ -430,6 +564,44 @@ _gabble_roster_item_dump (GabbleRosterItem *item)
 
   return g_string_free (str, FALSE);
 }
+#endif /* ENABLE_DEBUG */
+
+
+static LmMessage *
+_gabble_roster_message_new (GabbleRoster *roster,
+                            LmMessageSubType sub_type,
+                            LmMessageNode **query_return)
+{
+  GabbleRosterPrivate *priv = GABBLE_ROSTER_GET_PRIVATE (roster);
+  LmMessage *message;
+  LmMessageNode *query_node;
+
+  g_assert (roster != NULL);
+  g_assert (GABBLE_IS_ROSTER (roster));
+
+  message = lm_message_new_with_sub_type (NULL,
+                                          LM_MESSAGE_TYPE_IQ,
+                                          sub_type);
+
+  query_node = lm_message_node_add_child (message->node, "query", NULL);
+
+  if (NULL != query_return)
+    *query_return = query_node;
+
+  lm_message_node_set_attribute (query_node, "xmlns", NS_ROSTER);
+
+  if (priv->conn->features & GABBLE_CONNECTION_FEATURES_GOOGLE_ROSTER)
+    {
+      lm_message_node_set_attributes (query_node,
+          "xmlns:gr", NS_GOOGLE_ROSTER,
+          "gr:ext", GOOGLE_ROSTER_VERSION,
+          "gr:include", "all",
+          NULL);
+    }
+
+  return message;
+}
+
 
 static LmMessage *
 _gabble_roster_item_to_message (GabbleRoster *roster,
@@ -449,13 +621,8 @@ _gabble_roster_item_to_message (GabbleRoster *roster,
 
   item = _gabble_roster_item_get (roster, handle);
 
-  message = lm_message_new_with_sub_type (NULL,
-                                          LM_MESSAGE_TYPE_IQ,
-                                          LM_MESSAGE_SUB_TYPE_SET);
-
-  query_node = lm_message_node_add_child (message->node, "query", NULL);
-
-  lm_message_node_set_attribute (query_node, "xmlns", NS_ROSTER);
+  message = _gabble_roster_message_new (roster, LM_MESSAGE_SUB_TYPE_SET,
+      &query_node);
 
   item_node = lm_message_node_add_child (query_node, "item", NULL);
 
@@ -475,6 +642,11 @@ _gabble_roster_item_to_message (GabbleRoster *roster,
   if (item->subscription == GABBLE_ROSTER_SUBSCRIPTION_REMOVE)
     goto DONE;
 
+  if ((priv->conn->features & GABBLE_CONNECTION_FEATURES_GOOGLE_ROSTER) &&
+      item->google_type != GOOGLE_ITEM_TYPE_NORMAL)
+    lm_message_node_set_attribute (item_node, "gr:t",
+        _google_item_type_to_string (item->google_type));
+
   if (item->ask_subscribe)
     lm_message_node_set_attribute (item_node, "ask", "subscribe");
 
@@ -515,22 +687,22 @@ _gabble_roster_create_channel (GabbleRoster *roster,
                        "handle", handle,
                        NULL);
 
-  g_debug ("%s: created %s", G_STRFUNC, object_path);
+  DEBUG ("created %s", object_path);
   g_free (object_path);
 
   g_hash_table_insert (priv->channels, GINT_TO_POINTER (handle), chan);
 
   if (priv->roster_received)
     {
-      g_debug ("%s: roster already received, emitting signal for %s list channel",
-          G_STRFUNC, name);
+      DEBUG ("roster already received, emitting signal for %s list channel",
+          name);
 
       g_signal_emit_by_name (roster, "new-channel", chan);
     }
   else
     {
-      g_debug ("%s: roster not yet received, not emitting signal for %s list channel",
-          G_STRFUNC, name);
+      DEBUG ("roster not yet received, not emitting signal for %s list channel",
+          name);
     }
 
   return chan;
@@ -560,13 +732,15 @@ _gabble_roster_emit_one (gpointer key,
                          gpointer data)
 {
   GabbleRoster *roster = GABBLE_ROSTER (data);
-  GabbleRosterPrivate *priv = GABBLE_ROSTER_GET_PRIVATE (roster);
   GabbleRosterChannel *chan = GABBLE_ROSTER_CHANNEL (value);
+#ifdef ENABLE_DEBUG
+  GabbleRosterPrivate *priv = GABBLE_ROSTER_GET_PRIVATE (roster);
   GabbleHandle handle = GPOINTER_TO_INT (key);
   const gchar *name = gabble_handle_inspect (priv->conn->handles, TP_HANDLE_TYPE_LIST, handle);
 
-  g_debug ("%s: roster now received, emitting signal signal for %s list channel",
-      G_STRFUNC, name);
+  DEBUG ("roster now received, emitting signal signal for %s list channel",
+      name);
+#endif
 
   g_signal_emit_by_name (roster, "new-channel", chan);
 }
@@ -604,6 +778,7 @@ gabble_roster_iq_cb (LmMessageHandler *handler,
   LmMessageNode *iq_node, *query_node;
   LmMessageSubType sub_type;
   const gchar *from;
+  gboolean google_roster = FALSE;
 
   g_assert (lmconn == priv->conn->lmconn);
 
@@ -611,9 +786,10 @@ gabble_roster_iq_cb (LmMessageHandler *handler,
     return LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS;
 
   iq_node = lm_message_get_node (message);
-  query_node = lm_message_node_get_child (iq_node, "query");
+  query_node = lm_message_node_get_child_with_namespace (iq_node, "query",
+      NS_ROSTER);
 
-  if (!query_node || !_lm_message_node_has_namespace (query_node, NS_ROSTER))
+  if (query_node == NULL)
     return LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS;
 
   from = lm_message_node_get_attribute (message->node, "from");
@@ -627,12 +803,22 @@ gabble_roster_iq_cb (LmMessageHandler *handler,
 
       if (sender != priv->conn->self_handle)
         {
-          HANDLER_DEBUG (iq_node, "discarding roster IQ which is not from "
+           NODE_DEBUG (iq_node, "discarding roster IQ which is not from "
               "ourselves or the server");
           return LM_HANDLER_RESULT_REMOVE_MESSAGE;
         }
     }
 
+  if (priv->conn->features & GABBLE_CONNECTION_FEATURES_GOOGLE_ROSTER)
+    {
+      const char *gr_ext;
+
+      gr_ext = lm_message_node_get_attribute (query_node, "gr:ext");
+
+      if (!g_strdiff (gr_ext, GOOGLE_ROSTER_VERSION))
+        google_roster = TRUE;
+    }
+
   sub_type = lm_message_get_sub_type (message);
 
   /* if this is a result, it's from our initial query. if it's a set,
@@ -642,9 +828,12 @@ gabble_roster_iq_cb (LmMessageHandler *handler,
       LmMessageNode *item_node;
       GIntSet *empty, *pub_add, *pub_rem,
               *sub_add, *sub_rem, *sub_rp,
-              *known_add, *known_rem;
+              *known_add, *known_rem,
+              *deny_add, *deny_rem;
+      GArray *removed;
       GabbleHandle handle;
       GabbleRosterChannel *chan;
+      guint i;
 
     case LM_MESSAGE_SUB_TYPE_RESULT:
     case LM_MESSAGE_SUB_TYPE_SET:
@@ -658,6 +847,22 @@ gabble_roster_iq_cb (LmMessageHandler *handler,
       sub_rp = g_intset_new ();
       known_add = g_intset_new ();
       known_rem = g_intset_new ();
+      removed = g_array_new (FALSE, FALSE, sizeof (GabbleHandle));
+
+      if (google_roster)
+        {
+          deny_add = g_intset_new ();
+          deny_rem = g_intset_new ();
+        }
+      else
+        {
+          deny_add = NULL;
+          deny_rem = NULL;
+        }
+
+      /* get the publish channel first because we need it when processing */
+      handle = GABBLE_LIST_HANDLE_PUBLISH;
+      chan = _gabble_roster_get_channel (roster, handle);
 
       /* iterate every sub-node, which we expect to be <item>s */
       for (item_node = query_node->children;
@@ -669,90 +874,160 @@ gabble_roster_iq_cb (LmMessageHandler *handler,
 
           if (strcmp (item_node->name, "item"))
             {
-              HANDLER_DEBUG (item_node, "query sub-node is not item, skipping");
+               NODE_DEBUG (item_node, "query sub-node is not item, skipping");
               continue;
             }
 
           jid = lm_message_node_get_attribute (item_node, "jid");
           if (!jid)
             {
-              HANDLER_DEBUG (item_node, "item node has no jid, skipping");
+               NODE_DEBUG (item_node, "item node has no jid, skipping");
               continue;
             }
 
           handle = gabble_handle_for_contact (priv->conn->handles, jid, FALSE);
           if (handle == 0)
             {
-              HANDLER_DEBUG (item_node, "item jid is malformed, skipping");
+               NODE_DEBUG (item_node, "item jid is malformed, skipping");
               continue;
             }
 
-          item = _gabble_roster_item_update (roster, handle, item_node);
+          item = _gabble_roster_item_update (roster, handle, item_node,
+              google_roster);
+
+#ifdef ENABLE_DEBUG
+          if (DEBUGGING)
             {
               gchar *dump = _gabble_roster_item_dump (item);
-              g_debug ("%s: jid: %s, %s", G_STRFUNC, jid, dump);
+              DEBUG ("jid: %s, %s", jid, dump);
               g_free (dump);
             }
+#endif
 
+          /* handle publish list changes */
           switch (item->subscription)
             {
-            case GABBLE_ROSTER_SUBSCRIPTION_NONE:
-              g_intset_add (pub_rem, handle);
-              if (item->ask_subscribe)
-                g_intset_add (sub_rp, handle);
-              else
-                g_intset_add (sub_rem, handle);
-              g_intset_add (known_add, handle);
-              break;
-            case GABBLE_ROSTER_SUBSCRIPTION_TO:
-              g_intset_add (pub_rem, handle);
-              g_intset_add (sub_add, handle);
-              g_intset_add (known_add, handle);
-              break;
             case GABBLE_ROSTER_SUBSCRIPTION_FROM:
-              g_intset_add (pub_add, handle);
-              if (item->ask_subscribe)
-                g_intset_add (sub_rp, handle);
-              else
-                g_intset_add (sub_rem, handle);
-              g_intset_add (known_add, handle);
-              break;
             case GABBLE_ROSTER_SUBSCRIPTION_BOTH:
               g_intset_add (pub_add, handle);
-              g_intset_add (sub_add, handle);
-              g_intset_add (known_add, handle);
               break;
+            case GABBLE_ROSTER_SUBSCRIPTION_NONE:
+            case GABBLE_ROSTER_SUBSCRIPTION_TO:
             case GABBLE_ROSTER_SUBSCRIPTION_REMOVE:
-              g_intset_add (pub_rem, handle);
-              g_intset_add (sub_rem, handle);
-              g_intset_add (known_rem, handle);
-              _gabble_roster_item_remove (roster, handle);
+              /* publish channel is a bit odd, the roster item doesn't tell us
+               * if someone is awaiting our approval - we get this via presence
+               * type=subscribe, so we have to not remove them if they're
+               * already local_pending in our publish channel */
+              if (!handle_set_is_member (chan->group.local_pending, handle))
+                {
+                  g_intset_add (pub_rem, handle);
+                }
               break;
             default:
               g_assert_not_reached ();
             }
+
+          /* handle subscribe list changes */
+          switch (item->subscription)
+            {
+            case GABBLE_ROSTER_SUBSCRIPTION_TO:
+            case GABBLE_ROSTER_SUBSCRIPTION_BOTH:
+              g_intset_add (sub_add, handle);
+              break;
+            case GABBLE_ROSTER_SUBSCRIPTION_NONE:
+            case GABBLE_ROSTER_SUBSCRIPTION_FROM:
+              if (item->ask_subscribe)
+                g_intset_add (sub_rp, handle);
+              else
+                g_intset_add (sub_rem, handle);
+              break;
+            case GABBLE_ROSTER_SUBSCRIPTION_REMOVE:
+              g_intset_add (sub_rem, handle);
+              break;
+            default:
+              g_assert_not_reached ();
+            }
+
+          /* handle known list changes */
+          switch (item->subscription)
+            {
+            case GABBLE_ROSTER_SUBSCRIPTION_NONE:
+            case GABBLE_ROSTER_SUBSCRIPTION_TO:
+            case GABBLE_ROSTER_SUBSCRIPTION_FROM:
+            case GABBLE_ROSTER_SUBSCRIPTION_BOTH:
+              g_intset_add (known_add, handle);
+              break;
+            case GABBLE_ROSTER_SUBSCRIPTION_REMOVE:
+              g_intset_add (known_rem, handle);
+              break;
+            default:
+              g_assert_not_reached ();
+            }
+
+          /* handle deny list changes */
+          if (google_roster)
+            {
+              switch (item->subscription)
+                {
+                case GABBLE_ROSTER_SUBSCRIPTION_NONE:
+                case GABBLE_ROSTER_SUBSCRIPTION_TO:
+                case GABBLE_ROSTER_SUBSCRIPTION_FROM:
+                case GABBLE_ROSTER_SUBSCRIPTION_BOTH:
+                  if (item->google_type == GOOGLE_ITEM_TYPE_BLOCKED)
+                    g_intset_add (deny_add, handle);
+                  else
+                    g_intset_add (deny_rem, handle);
+                  break;
+                case GABBLE_ROSTER_SUBSCRIPTION_REMOVE:
+                  g_intset_add (deny_rem, handle);
+                  break;
+                default:
+                  g_assert_not_reached ();
+                }
+            }
+
+          /* delay removing items from roster until signals have been emitted;
+           * otherwise handles go out of scope! */
+          if (GABBLE_ROSTER_SUBSCRIPTION_REMOVE == item->subscription)
+            g_array_append_val (removed, handle);
         }
 
-      handle = gabble_handle_for_list_publish (priv->conn->handles);
+      /* chan was initialised to the publish channel before the for loop */
+
+      DEBUG ("calling change members on publish channel");
+      gabble_group_mixin_change_members (G_OBJECT (chan),
+            "", pub_add, pub_rem, empty, empty, 0, 0);
+
+      handle = GABBLE_LIST_HANDLE_SUBSCRIBE;
       chan = _gabble_roster_get_channel (roster, handle);
 
-      g_debug ("%s: calling change members on publish channel", G_STRFUNC);
+      DEBUG ("calling change members on subscribe channel");
       gabble_group_mixin_change_members (G_OBJECT (chan),
-            "", pub_add, pub_rem, empty, empty);
+            "", sub_add, sub_rem, empty, sub_rp, 0, 0);
 
-      handle = gabble_handle_for_list_subscribe (priv->conn->handles);
+      handle = GABBLE_LIST_HANDLE_KNOWN;
       chan = _gabble_roster_get_channel (roster, handle);
 
-      g_debug ("%s: calling change members on subscribe channel", G_STRFUNC);
+      DEBUG ("calling change members on known channel");
       gabble_group_mixin_change_members (G_OBJECT (chan),
-            "", sub_add, sub_rem, empty, sub_rp);
+            "", known_add, known_rem, empty, empty, 0, 0);
 
-      handle = gabble_handle_for_list_known (priv->conn->handles);
-      chan = _gabble_roster_get_channel (roster, handle);
+      if (google_roster)
+        {
+          handle = GABBLE_LIST_HANDLE_DENY;
+          chan = _gabble_roster_get_channel (roster, handle);
 
-      g_debug ("%s: calling change members on known channel", G_STRFUNC);
-      gabble_group_mixin_change_members (G_OBJECT (chan),
-            "", known_add, known_rem, empty, empty);
+          DEBUG ("calling change members on deny channel");
+          gabble_group_mixin_change_members (G_OBJECT (chan),
+              "", deny_add, deny_rem, empty, empty, 0, 0);
+
+          g_intset_destroy (deny_add);
+          g_intset_destroy (deny_rem);
+        }
+
+      for (i = 0; i < removed->len; i++)
+          _gabble_roster_item_remove (roster,
+              g_array_index (removed, GabbleHandle, i));
 
       g_intset_destroy (empty);
       g_intset_destroy (pub_add);
@@ -762,9 +1037,10 @@ gabble_roster_iq_cb (LmMessageHandler *handler,
       g_intset_destroy (sub_rp);
       g_intset_destroy (known_add);
       g_intset_destroy (known_rem);
+      g_array_free (removed, TRUE);
       break;
     default:
-      HANDLER_DEBUG (iq_node, "unhandled roster IQ");
+       NODE_DEBUG (iq_node, "unhandled roster IQ");
       return LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS;
     }
 
@@ -798,8 +1074,7 @@ _gabble_roster_send_presence_ack (GabbleRoster *roster,
 
   if (!changed)
     {
-      g_debug ("%s: not sending ack to avoid loop with buggy server",
-          G_STRFUNC);
+      DEBUG ("not sending ack to avoid loop with buggy server");
       return;
     }
 
@@ -866,7 +1141,7 @@ gabble_roster_presence_cb (LmMessageHandler *handler,
 
   if (from == NULL)
     {
-      HANDLER_DEBUG (pres_node, "presence stanza without from attribute, ignoring");
+       NODE_DEBUG (pres_node, "presence stanza without from attribute, ignoring");
       return LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS;
     }
 
@@ -876,13 +1151,13 @@ gabble_roster_presence_cb (LmMessageHandler *handler,
 
   if (handle == 0)
     {
-      HANDLER_DEBUG (pres_node, "ignoring presence from malformed jid");
+       NODE_DEBUG (pres_node, "ignoring presence from malformed jid");
       return LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS;
     }
 
   if (handle == priv->conn->self_handle)
     {
-      HANDLER_DEBUG (pres_node, "ignoring presence from ourselves on another resource");
+       NODE_DEBUG (pres_node, "ignoring presence from ourselves on another resource");
       return LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS;
     }
 
@@ -895,34 +1170,34 @@ gabble_roster_presence_cb (LmMessageHandler *handler,
   switch (sub_type)
     {
     case LM_MESSAGE_SUB_TYPE_SUBSCRIBE:
-      g_debug ("%s: making %s (handle %u) local pending on the publish channel",
-          G_STRFUNC, from, handle);
+      DEBUG ("making %s (handle %u) local pending on the publish channel",
+          from, handle);
 
       empty = g_intset_new ();
       tmp = g_intset_new ();
       g_intset_add (tmp, handle);
 
-      handle = gabble_handle_for_list_publish (priv->conn->handles);
+      handle = GABBLE_LIST_HANDLE_PUBLISH;
       chan = _gabble_roster_get_channel (roster, handle);
       gabble_group_mixin_change_members (G_OBJECT (chan), status_message,
-          empty, empty, tmp, empty);
+          empty, empty, tmp, empty, 0, 0);
 
       g_intset_destroy (empty);
       g_intset_destroy (tmp);
 
       return LM_HANDLER_RESULT_REMOVE_MESSAGE;
     case LM_MESSAGE_SUB_TYPE_UNSUBSCRIBE:
-      g_debug ("%s: removing %s (handle %u) from the publish channel",
-          G_STRFUNC, from, handle);
+      DEBUG ("removing %s (handle %u) from the publish channel",
+          from, handle);
 
       empty = g_intset_new ();
       tmp = g_intset_new ();
       g_intset_add (tmp, handle);
 
-      handle = gabble_handle_for_list_publish (priv->conn->handles);
+      handle = GABBLE_LIST_HANDLE_PUBLISH;
       chan = _gabble_roster_get_channel (roster, handle);
       changed = gabble_group_mixin_change_members (G_OBJECT (chan),
-          status_message, empty, tmp, empty, empty);
+          status_message, empty, tmp, empty, empty, 0, 0);
 
       _gabble_roster_send_presence_ack (roster, from, sub_type, changed);
 
@@ -931,17 +1206,17 @@ gabble_roster_presence_cb (LmMessageHandler *handler,
 
       return LM_HANDLER_RESULT_REMOVE_MESSAGE;
     case LM_MESSAGE_SUB_TYPE_SUBSCRIBED:
-      g_debug ("%s: adding %s (handle %u) to the subscribe channel",
-          G_STRFUNC, from, handle);
+      DEBUG ("adding %s (handle %u) to the subscribe channel",
+          from, handle);
 
       empty = g_intset_new ();
       tmp = g_intset_new ();
       g_intset_add (tmp, handle);
 
-      handle = gabble_handle_for_list_subscribe (priv->conn->handles);
+      handle = GABBLE_LIST_HANDLE_SUBSCRIBE;
       chan = _gabble_roster_get_channel (roster, handle);
       changed = gabble_group_mixin_change_members (G_OBJECT (chan),
-          status_message, tmp, empty, empty, empty);
+          status_message, tmp, empty, empty, empty, 0, 0);
 
       _gabble_roster_send_presence_ack (roster, from, sub_type, changed);
 
@@ -950,17 +1225,17 @@ gabble_roster_presence_cb (LmMessageHandler *handler,
 
       return LM_HANDLER_RESULT_REMOVE_MESSAGE;
     case LM_MESSAGE_SUB_TYPE_UNSUBSCRIBED:
-      g_debug ("%s: removing %s (handle %u) from the subscribe channel",
-          G_STRFUNC, from, handle);
+      DEBUG ("removing %s (handle %u) from the subscribe channel",
+          from, handle);
 
       empty = g_intset_new ();
       tmp = g_intset_new ();
       g_intset_add (tmp, handle);
 
-      handle = gabble_handle_for_list_subscribe (priv->conn->handles);
+      handle = GABBLE_LIST_HANDLE_SUBSCRIBE;
       chan = _gabble_roster_get_channel (roster, handle);
       changed = gabble_group_mixin_change_members (G_OBJECT (chan),
-          status_message, empty, tmp, empty, empty);
+          status_message, empty, tmp, empty, empty, 0, 0);
 
       _gabble_roster_send_presence_ack (roster, from, sub_type, changed);
 
@@ -979,7 +1254,7 @@ gabble_roster_factory_iface_close_all (TpChannelFactoryIface *iface)
   GabbleRoster *roster = GABBLE_ROSTER (iface);
   GabbleRosterPrivate *priv = GABBLE_ROSTER_GET_PRIVATE (roster);
 
-  g_debug ("%s: closing channels", G_STRFUNC);
+  DEBUG ("closing channels");
 
   if (priv->channels)
     {
@@ -994,7 +1269,7 @@ gabble_roster_factory_iface_connecting (TpChannelFactoryIface *iface)
   GabbleRoster *roster = GABBLE_ROSTER (iface);
   GabbleRosterPrivate *priv = GABBLE_ROSTER_GET_PRIVATE (roster);
 
-  g_debug ("%s: adding callbacks", G_STRFUNC);
+  DEBUG ("adding callbacks");
 
   g_assert (priv->iq_cb == NULL);
   g_assert (priv->presence_cb == NULL);
@@ -1020,18 +1295,10 @@ gabble_roster_factory_iface_connected (TpChannelFactoryIface *iface)
   GabbleRoster *roster = GABBLE_ROSTER (iface);
   GabbleRosterPrivate *priv = GABBLE_ROSTER_GET_PRIVATE (roster);
   LmMessage *message;
-  LmMessageNode *msgnode;
 
-  g_debug ("%s: requesting roster", G_STRFUNC);
+  DEBUG ("requesting roster");
 
-  message = lm_message_new_with_sub_type (NULL,
-                                          LM_MESSAGE_TYPE_IQ,
-                                          LM_MESSAGE_SUB_TYPE_GET);
-
-  msgnode = lm_message_node_add_child (lm_message_get_node (message),
-                                       "query", NULL);
-
-  lm_message_node_set_attribute (msgnode, "xmlns", NS_ROSTER);
+  message = _gabble_roster_message_new (roster, LM_MESSAGE_SUB_TYPE_GET, NULL);
 
   _gabble_connection_send (priv->conn, message, NULL);
 
@@ -1044,7 +1311,7 @@ gabble_roster_factory_iface_disconnected (TpChannelFactoryIface *iface)
   GabbleRoster *roster = GABBLE_ROSTER (iface);
   GabbleRosterPrivate *priv = GABBLE_ROSTER_GET_PRIVATE (roster);
 
-  g_debug ("%s: removing callbacks", G_STRFUNC);
+  DEBUG ("removing callbacks");
 
   g_assert (priv->iq_cb != NULL);
   g_assert (priv->presence_cb != NULL);
@@ -1099,7 +1366,8 @@ gabble_roster_factory_iface_request (TpChannelFactoryIface *iface,
                                      const gchar *chan_type,
                                      TpHandleType handle_type,
                                      guint handle,
-                                     TpChannelIface **ret)
+                                     TpChannelIface **ret,
+                                     GError **error)
 {
   GabbleRoster *roster = GABBLE_ROSTER (iface);
   GabbleRosterPrivate *priv = GABBLE_ROSTER_GET_PRIVATE (roster);
@@ -1116,6 +1384,11 @@ gabble_roster_factory_iface_request (TpChannelFactoryIface *iface,
                                NULL))
     return TP_CHANNEL_FACTORY_REQUEST_STATUS_INVALID_HANDLE;
 
+  /* disallow "deny" channels if we don't have google:roster support */
+  if (handle == GABBLE_LIST_HANDLE_DENY &&
+      !(priv->conn->features & GABBLE_CONNECTION_FEATURES_GOOGLE_ROSTER))
+    return TP_CHANNEL_FACTORY_REQUEST_STATUS_NOT_AVAILABLE;
+
   if (priv->roster_received)
     {
       GabbleRosterChannel *chan;
@@ -1175,6 +1448,63 @@ gabble_roster_handle_get_subscription (GabbleRoster *roster,
   return item->subscription;
 }
 
+gboolean
+gabble_roster_handle_set_blocked (GabbleRoster *roster,
+                                  GabbleHandle handle,
+                                  gboolean blocked,
+                                  GError **error)
+{
+  GabbleRosterPrivate *priv = GABBLE_ROSTER_GET_PRIVATE (roster);
+  GabbleRosterItem *item;
+  GoogleItemType orig_type;
+  LmMessage *message;
+  gboolean ret;
+
+  g_return_val_if_fail (roster != NULL, FALSE);
+  g_return_val_if_fail (GABBLE_IS_ROSTER (roster), FALSE);
+  g_return_val_if_fail (gabble_handle_is_valid (priv->conn->handles,
+      TP_HANDLE_TYPE_CONTACT, handle, NULL), FALSE);
+  g_return_val_if_fail (priv->conn->features &
+      GABBLE_CONNECTION_FEATURES_GOOGLE_ROSTER, FALSE);
+
+  item = _gabble_roster_item_get (roster, handle);
+  orig_type = item->google_type;
+
+  if (blocked == (orig_type == GOOGLE_ITEM_TYPE_BLOCKED))
+    return TRUE;
+
+  /* temporarily set the desired block state and generate a message */
+  if (blocked)
+    item->google_type = GOOGLE_ITEM_TYPE_BLOCKED;
+  else
+    item->google_type = GOOGLE_ITEM_TYPE_NORMAL;
+  message = _gabble_roster_item_to_message (roster, handle, NULL);
+  item->google_type = orig_type;
+
+  ret = _gabble_connection_send (priv->conn, message, error);
+
+  lm_message_unref (message);
+
+  return ret;
+}
+
+gboolean
+gabble_roster_handle_has_entry (GabbleRoster *roster,
+                                GabbleHandle handle)
+{
+  GabbleRosterPrivate *priv = GABBLE_ROSTER_GET_PRIVATE (roster);
+  GabbleRosterItem *item;
+
+  g_return_val_if_fail (roster != NULL, FALSE);
+  g_return_val_if_fail (GABBLE_IS_ROSTER (roster), FALSE);
+  g_return_val_if_fail (gabble_handle_is_valid (priv->conn->handles,
+      TP_HANDLE_TYPE_CONTACT, handle, NULL), FALSE);
+
+  item = g_hash_table_lookup (priv->items, GINT_TO_POINTER (handle));
+
+  return (NULL != item);
+}
+
 const gchar *
 gabble_roster_handle_get_name (GabbleRoster *roster,
                                GabbleHandle handle)
