@@ -33,7 +33,11 @@
 #include "telepathy-constants.h"
 #include "telepathy-errors.h"
 
+#define DEBUG_FLAG GABBLE_DEBUG_IM
+
+#include "debug.h"
 #include "gabble-connection.h"
+#include "namespaces.h"
 #include "roster.h"
 #include "util.h"
 
@@ -45,6 +49,7 @@
       G_TYPE_UINT, \
       G_TYPE_UINT, \
       G_TYPE_UINT, \
+      G_TYPE_UINT, \
       G_TYPE_STRING, \
       G_TYPE_INVALID))
 
@@ -103,7 +108,7 @@ static void gabble_allocator_free (GabbleAllocator *alloc, gpointer thing)
 
 /* pending message */
 #define MAX_PENDING_MESSAGES 256
-#define MAX_MESSAGE_SIZE 8*1024 - 1
+#define MAX_MESSAGE_SIZE 1024 - 1
 
 typedef struct _GabblePendingMessage GabblePendingMessage;
 struct _GabblePendingMessage
@@ -113,6 +118,7 @@ struct _GabblePendingMessage
   GabbleHandle sender;
   TpChannelTextMessageType type;
   char *text;
+  guint flags;
 };
 
 /**
@@ -163,7 +169,7 @@ gabble_text_mixin_class_init (GObjectClass *obj_cls, glong offset)
                 G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
                 0,
                 NULL, NULL,
-                text_mixin_marshal_VOID__VOID,
+                g_cclosure_marshal_VOID__VOID,
                 G_TYPE_NONE, 0);
 
   mixin_cls->received_signal_id = g_signal_new ("received",
@@ -171,15 +177,15 @@ gabble_text_mixin_class_init (GObjectClass *obj_cls, glong offset)
                 G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
                 0,
                 NULL, NULL,
-                text_mixin_marshal_VOID__INT_INT_INT_INT_STRING,
-                G_TYPE_NONE, 5, G_TYPE_UINT, G_TYPE_UINT, G_TYPE_UINT, G_TYPE_UINT, G_TYPE_STRING);
+                text_mixin_marshal_VOID__UINT_UINT_UINT_UINT_UINT_STRING,
+                G_TYPE_NONE, 6, G_TYPE_UINT, G_TYPE_UINT, G_TYPE_UINT, G_TYPE_UINT, G_TYPE_UINT, G_TYPE_STRING);
 
   mixin_cls->send_error_signal_id = g_signal_new ("send-error",
                 G_OBJECT_CLASS_TYPE (obj_cls),
                 G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
                 0,
                 NULL, NULL,
-                text_mixin_marshal_VOID__INT_INT_INT_STRING,
+                text_mixin_marshal_VOID__UINT_UINT_UINT_STRING,
                 G_TYPE_NONE, 4, G_TYPE_UINT, G_TYPE_UINT, G_TYPE_UINT, G_TYPE_STRING);
 
   mixin_cls->sent_signal_id = g_signal_new ("sent",
@@ -187,7 +193,7 @@ gabble_text_mixin_class_init (GObjectClass *obj_cls, glong offset)
                 G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
                 0,
                 NULL, NULL,
-                text_mixin_marshal_VOID__INT_INT_STRING,
+                text_mixin_marshal_VOID__UINT_UINT_STRING,
                 G_TYPE_NONE, 3, G_TYPE_UINT, G_TYPE_UINT, G_TYPE_STRING);
 }
 
@@ -305,7 +311,7 @@ gboolean gabble_text_mixin_receive (GObject *obj,
 
   if (msg == NULL)
     {
-      g_debug ("%s: no more pending messages available, giving up", G_STRFUNC);
+      DEBUG ("no more pending messages available, giving up");
 
       if (!mixin->message_lost)
         {
@@ -320,9 +326,9 @@ gboolean gabble_text_mixin_receive (GObject *obj,
 
   if (len > MAX_MESSAGE_SIZE)
     {
-      g_debug ("%s: message exceeds maximum size, truncating", G_STRFUNC);
+      DEBUG ("message exceeds maximum size, truncating");
 
-      /* TODO: add CHANNEL_TEXT_MESSAGE_FLAG_TRUNCATED flag*/
+      msg->flags |= TP_CHANNEL_TEXT_MESSAGE_FLAG_TRUNCATED;
 
       end = g_utf8_find_prev_char (text, text+MAX_MESSAGE_SIZE);
       if (end)
@@ -335,7 +341,7 @@ gboolean gabble_text_mixin_receive (GObject *obj,
 
   if (msg->text == NULL)
     {
-      g_debug ("%s: unable to allocate message, giving up", G_STRFUNC);
+      DEBUG ("unable to allocate message, giving up");
 
       if (!mixin->message_lost)
         {
@@ -348,7 +354,6 @@ gboolean gabble_text_mixin_receive (GObject *obj,
       return FALSE;
     }
 
-  /* TODO: UTF-8 truncation */
   g_strlcpy (msg->text, text, len + 1);
 
   msg->id = mixin->recv_id++;
@@ -364,9 +369,10 @@ gboolean gabble_text_mixin_receive (GObject *obj,
                  msg->timestamp,
                  msg->sender,
                  msg->type,
+                 msg->flags,
                  msg->text);
 
-  g_debug ("%s: queued message %u", G_STRFUNC, msg->id);
+  DEBUG ("queued message %u", msg->id);
 
   mixin->message_lost = FALSE;
 
@@ -384,62 +390,75 @@ compare_pending_message (gconstpointer haystack,
 }
 
 /**
- * gabble_text_mixin_acknowledge_pending_message
+ * gabble_text_mixin_acknowledge_pending_messages
  *
- * Implements DBus method AcknowledgePendingMessage
+ * Implements D-Bus method AcknowledgePendingMessages
  * on interface org.freedesktop.Telepathy.Channel.Type.Text
  *
  * @error: Used to return a pointer to a GError detailing any error
- *         that occured, DBus will throw the error only if this
+ *         that occurred, D-Bus will throw the error only if this
  *         function returns false.
  *
  * Returns: TRUE if successful, FALSE if an error was thrown.
  */
-gboolean gabble_text_mixin_acknowledge_pending_message (GObject *obj, guint id, GError **error)
+gboolean gabble_text_mixin_acknowledge_pending_messages (GObject *obj, const GArray * ids, GError **error)
 {
   GabbleTextMixin *mixin = GABBLE_TEXT_MIXIN (obj);
-  GList *node;
+  GList **nodes;
   GabblePendingMessage *msg;
+  guint i;
 
-  node = g_queue_find_custom (mixin->pending,
-                              GINT_TO_POINTER (id),
-                              compare_pending_message);
+  nodes = g_new(GList *, ids->len);
 
-  if (node == NULL)
+  for (i = 0; i < ids->len; i++)
     {
-      g_debug ("%s: invalid message id %u", G_STRFUNC, id);
+      guint id = g_array_index(ids, guint, i);
 
-      *error = g_error_new (TELEPATHY_ERRORS, InvalidArgument,
-                            "invalid message id %u", id);
+      nodes[i] = g_queue_find_custom (mixin->pending,
+                                      GINT_TO_POINTER (id),
+                                      compare_pending_message);
 
-      return FALSE;
+      if (nodes[i] == NULL)
+        {
+          DEBUG ("invalid message id %u", id);
+
+          g_set_error (error, TELEPATHY_ERRORS, InvalidArgument,
+              "invalid message id %u", id);
+
+          g_free(nodes);
+          return FALSE;
+        }
     }
 
-  msg = (GabblePendingMessage *) node->data;
+  for (i = 0; i < ids->len; i++)
+    {
+      msg = (GabblePendingMessage *) nodes[i]->data;
 
-  g_debug ("%s: acknowleding message id %u", G_STRFUNC, id);
+      DEBUG ("acknowleding message id %u", msg->id);
 
-  g_queue_remove (mixin->pending, msg);
+      g_queue_remove (mixin->pending, msg);
 
-  gabble_handle_unref (mixin->handle_repo, TP_HANDLE_TYPE_CONTACT, msg->sender);
-  _gabble_pending_free (msg);
+      gabble_handle_unref (mixin->handle_repo, TP_HANDLE_TYPE_CONTACT, msg->sender);
+      _gabble_pending_free (msg);
+    }
 
+  g_free(nodes);
   return TRUE;
 }
 
 /**
  * gabble_text_mixin_list_pending_messages
  *
- * Implements DBus method ListPendingMessages
+ * Implements D-Bus method ListPendingMessages
  * on interface org.freedesktop.Telepathy.Channel.Type.Text
  *
  * @error: Used to return a pointer to a GError detailing any error
- *         that occured, DBus will throw the error only if this
+ *         that occurred, D-Bus will throw the error only if this
  *         function returns false.
  *
  * Returns: TRUE if successful, FALSE if an error was thrown.
  */
-gboolean gabble_text_mixin_list_pending_messages (GObject *obj, GPtrArray ** ret, GError **error)
+gboolean gabble_text_mixin_list_pending_messages (GObject *obj, gboolean clear, GPtrArray ** ret, GError **error)
 {
   GabbleTextMixin *mixin = GABBLE_TEXT_MIXIN (obj);
   guint count;
@@ -449,9 +468,11 @@ gboolean gabble_text_mixin_list_pending_messages (GObject *obj, GPtrArray ** ret
   count = g_queue_get_length (mixin->pending);
   messages = g_ptr_array_sized_new (count);
 
-  for (cur = g_queue_peek_head_link(mixin->pending);
+  for (cur = (clear ? g_queue_pop_head_link(mixin->pending)
+                    : g_queue_peek_head_link(mixin->pending));
        cur != NULL;
-       cur = cur->next)
+       cur = (clear ? g_queue_pop_head_link(mixin->pending)
+                    : cur->next))
     {
       GabblePendingMessage *msg = (GabblePendingMessage *) cur->data;
       GValue val = { 0, };
@@ -464,7 +485,8 @@ gboolean gabble_text_mixin_list_pending_messages (GObject *obj, GPtrArray ** ret
           1, msg->timestamp,
           2, msg->sender,
           3, msg->type,
-          4, msg->text,
+          4, msg->flags,
+          5, msg->text,
           G_MAXUINT);
 
       g_ptr_array_add (messages, g_value_get_boxed (&val));
@@ -478,31 +500,31 @@ gboolean gabble_text_mixin_list_pending_messages (GObject *obj, GPtrArray ** ret
 /**
  * gabble_text_mixin_send
  *
- * Implements DBus method Send
+ * Implements D-Bus method Send
  * on interface org.freedesktop.Telepathy.Channel.Type.Text
  *
  * @error: Used to return a pointer to a GError detailing any error
- *         that occured, DBus will throw the error only if this
+ *         that occurred, D-Bus will throw the error only if this
  *         function returns false.
  *
  * Returns: TRUE if successful, FALSE if an error was thrown.
  */
 gboolean gabble_text_mixin_send (GObject *obj, guint type, guint subtype,
                                  const char *recipient, const gchar *text,
-                                 GabbleConnection *conn, GError **error)
+                                 GabbleConnection *conn, gboolean emit_signal,
+                                 GError **error)
 {
   GabbleTextMixin *mixin = GABBLE_TEXT_MIXIN (obj);
-  GabbleTextMixinClass *mixin_cls = GABBLE_TEXT_MIXIN_CLASS (G_OBJECT_GET_CLASS (obj));
   LmMessage *msg;
   gboolean result;
   time_t timestamp;
 
   if (type > TP_CHANNEL_TEXT_MESSAGE_TYPE_NOTICE)
     {
-      g_debug ("%s: invalid message type %u", G_STRFUNC, type);
+      DEBUG ("invalid message type %u", type);
 
-      *error = g_error_new (TELEPATHY_ERRORS, InvalidArgument,
-                            "invalid message type: %u", type);
+      g_set_error (error, TELEPATHY_ERRORS, InvalidArgument,
+          "invalid message type: %u", type);
 
       return FALSE;
     }
@@ -547,14 +569,29 @@ gboolean gabble_text_mixin_send (GObject *obj, guint type, guint subtype,
   if (!result)
     return FALSE;
 
-  timestamp = time (NULL);
+  if (emit_signal)
+    {
+      timestamp = time (NULL);
+
+      gabble_text_mixin_emit_sent (obj, timestamp, type, text);
+    }
+
+  return TRUE;
+}
+
+void
+gabble_text_mixin_emit_sent (GObject *obj,
+                             time_t timestamp,
+                             guint type,
+                             const char *text)
+{
+  GabbleTextMixinClass *mixin_cls = GABBLE_TEXT_MIXIN_CLASS (G_OBJECT_GET_CLASS
+      (obj));
 
   g_signal_emit (obj, mixin_cls->sent_signal_id, 0,
                  timestamp,
                  type,
                  text);
-
-  return TRUE;
 }
 
 gboolean
@@ -610,9 +647,8 @@ gabble_text_mixin_parse_incoming_message (LmMessage *message,
       if (error_node)
         {
           GabbleXmppError err = gabble_xmpp_error_from_node (error_node);
-          g_debug ("%s: got xmpp error: %s: %s", G_STRFUNC,
-                   gabble_xmpp_error_string (err),
-                   gabble_xmpp_error_description (err));
+          DEBUG ("got xmpp error: %s: %s", gabble_xmpp_error_string (err),
+                 gabble_xmpp_error_description (err));
 
           /* these are based on descriptions of errors, and some testing */
           switch (err)
@@ -636,6 +672,10 @@ gabble_text_mixin_parse_incoming_message (LmMessage *message,
                 *send_error = CHANNEL_TEXT_SEND_ERROR_TOO_LONG;
                 break;
 
+              case XMPP_ERROR_FEATURE_NOT_IMPLEMENTED:
+                *send_error = CHANNEL_TEXT_SEND_ERROR_NOT_IMPLEMENTED;
+                break;
+
               default:
                 *send_error = CHANNEL_TEXT_SEND_ERROR_UNKNOWN;
             }
@@ -649,47 +689,41 @@ gabble_text_mixin_parse_incoming_message (LmMessage *message,
   *from = lm_message_node_get_attribute (message->node, "from");
   if (*from == NULL)
     {
-      HANDLER_DEBUG (message->node, "got a message without a from field");
+      NODE_DEBUG (message->node, "got a message without a from field");
       return FALSE;
     }
 
   type = lm_message_node_get_attribute (message->node, "type");
 
   /*
-   * Parse timestamp of delayed messages.
+   * Parse timestamp of delayed messages. For non-delayed, it's
+   * 0 and the channel code should set the current timestamp.
    */
   *stamp = 0;
 
-  for (node = message->node->children; node; node = node->next)
+  node = lm_message_node_get_child_with_namespace (message->node, "x",
+      NS_X_DELAY);
+  if (node != NULL)
     {
-      if (strcmp (node->name, "x") == 0)
+      const gchar *stamp_str, *p;
+      struct tm stamp_tm = { 0, };
+
+      stamp_str = lm_message_node_get_attribute (node, "stamp");
+      if (stamp_str != NULL)
         {
-          const gchar *stamp_str, *p;
-          struct tm stamp_tm = { 0, };
-
-          if (!_lm_message_node_has_namespace (node, "jabber:x:delay"))
-            continue;
-
-          stamp_str = lm_message_node_get_attribute (node, "stamp");
-          if (stamp_str == NULL)
-            continue;
-
           p = strptime (stamp_str, "%Y%m%dT%T", &stamp_tm);
           if (p == NULL || *p != '\0')
             {
               g_warning ("%s: malformed date string '%s' for jabber:x:delay",
                          G_STRFUNC, stamp_str);
-              continue;
             }
-
-          *stamp = timegm (&stamp_tm);
+          else
+            {
+              *stamp = timegm (&stamp_tm);
+            }
         }
     }
 
-  if (*stamp == 0)
-    *stamp = time (NULL);
-
-
   /*
    * Parse body if it exists.
    */
